--------------------------------------------------------------------------------------------------------------
--
-- Module     : SIMS Database SQL Server 2005
-- Platform   : SIMS
-- Area       : DBUpgrade Database Patch
-- Copyright  : (c) Capita Education Services 2005-2009
--            : This code has been written by Capita Education Services for use within its own SIMS systems.
--            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any
--            : means or used without the express written permission of Capita Education Services.
-- Patch ID   : 21190
--
--------------------------------------------------------------------------------------------------------------

-- Check this is within the acceptible range
  IF NOT EXISTS(
	SELECT 1 
	FROM	sims.db_version_info
	WHERE	schema_version BETWEEN '3.62.232' AND '3.62.232')
  BEGIN
    DECLARE @error_msg2 VARCHAR(100)
    SET @error_msg2 = CHAR(10) + CHAR(13) + 'This patch is not applicable to this database version.' + CHAR(10) + CHAR(13)
	raiserror(@error_msg2, 18, 1)
  END

-- This is a RunOnce patch - check its not already been applied.
ELSE IF EXISTS(SELECT 1 
	        FROM   sims.db_update_log 
	        WHERE	patch_id = 21190)
  BEGIN
    DECLARE @error_msg1 VARCHAR(100)
    SET @error_msg1 = CHAR(10) + CHAR(13) + 'This patch has already been run on the database and cannot be run again.' + CHAR(10) + CHAR(13)
	raiserror(@error_msg1, 18, 1) 
  END

GO
-- START PATCH CODE



-- START PATCH SCRIPT 1

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2005                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : Schools Workforce Return                                                                        */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2013                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : Unreviewed				                                                                        */
/*--------------------------------------------------------------------------------------------------------------*/

SET NOCOUNT ON
GO

EXEC sims.db_p_drop 'sims.sre_pix_datapackage_swc_GetAdditionalPaymentsDetails', 'P'
GO

ALTER PROCEDURE sims.sre_pix_datapackage_swc_GetAdditionalPaymentsDetails
(
	@school_id     INT,			-- School id
    @return_id     INT,			-- Census return id
    @xml_staff	   TEXT,		-- Eligible staffs
	@show_contractDetails BIT,  -- Value from workforce setting file whether to show contract details or not 
	@show_serviceDetails BIT,   -- Value from workforce setting file whether to show service agreement details or not 
	@for_kent      BIT = 0      -- additional param to indicate whether this sp is called specifically for KENT LA. From sims.sre_pix_datapackage_swc_GetStaffsContractorServiceDetails it is 1 if LA is kent otherwise it is 0 everywhere even if it is called from SR Engine. 
)

AS

/*--------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL                                                                                    */
/* Object     : sims.sre_pix_datapackage_swc_GetAdditionalPaymentsDetails                                       */
/*                                                                                                              */
/* Purpose    : This procedure fetches additional payment details of staffs.                                    */
/* Comments   :                                                                                                 */
/*                                                                                                              */
/* Author     : R Vijayavel                                                                                     */
/* Date       : 27/01/2010                                                                                      */
/*                                                                                                              */
/* Modified by: R Vijayavel                                                                                     */
/* Date       : 04/06/2010                                                                                      */
/* Detail     : MIS reference no.1006-722186: INT - Summer 2010 Main - SWC - Workforce member payment           */
/*              details incorrect.                                                                              */
/*              Added pay_factor for calculation payment_amount.                                                */ 
/*                                                                                                              */       
/* Modified by: R Vijayavel                                                                                     */
/* Date       : 04/02/2011                                                                                      */
/* Detail     : SWAUT11_050: Modified where clause to retrieve additional allowances from previous year census  */
/*                           date.                                                                              */
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 20/02/2012																						*/
/* Details	  : SWCAUT12 3051:For Kent LA the calculation for ?Base Pay? must include the value of Kent         */
/*              specific additional allowances.  																*/
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 21/02/2012																						*/
/* Details	  : SWCAUT12 2518: Modified for calculation of allowances as per new requirement.		 	        */
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 07/03/2012																						*/
/* Details	  : SWCAUT12 4799: Consider allowance as annual reward and divide it by 12 only.					*/ 
/*                                                                                                              */ 	
/* Modified by: R Vijayavel                                                                                     */
/* Date		  : 13/03/2012																						*/
/* Details	  : 1203-105412: Modified length of payfactor column in temp table.             					*/  
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 15/03/2012																						*/
/* Details	  : SWCAUT12 4794: Modify as per Academy allowance calculation.										*/ 
/*                                                                                                              */ 	
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 16/03/2012																						*/
/* Details	  : SWCAUT12 5122: Kent LA - KR Allowances must not be pro rata.									*/ 
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 10/07/2012																						*/
/* Details	  : Kent LA - Consider those allowances which are falling on census date.							*/ 
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 11/09/2012																						*/
/* Details	  : AUT2012 BTR 17888 - Modified to retrive value of Additional Payment for service agreement.      */ 
/*                                                                                                              */ 
/* Modified by: R Vijayavel                                                                                     */
/* Date		  : 19/04/2013																						*/
/* Details	  : User story 25779 - Update additional amount from swc_pay_details table.                         */	
/*                                                                                                              */
/* Modified by: R Vijayavel                                                                                     */
/* Date       : 26/04/2013                                                                                      */
/* Details    : User story 25648 - Academy start date is calculated from date dfe number changed.               */
/*                                                                                                              */
/* Modified by: R Vijayavel                                                                                     */
/* Date       : 29/04/2013                                                                                      */
/* Details    : Retrieve distinct value.                                                                        */
/*                                                                                                              */
/* Modified by: R Vijayavel                                                                                     */
/* Date       : 10/05/2013                                                                                      */
/* Details    : Removed additional allowance claculation from swc_pay_details table.                            */
/*                                                                                                              */
/* Modified by: R Vijayavel                                                                                     */
/* Date       : 22/05/2013                                                                                      */
/* Details    : Bug 30677 - Added next day of previous year census date to @academy_converted_date,             */
/*                          if date_dfe_number_changed is not exist                                             */
/*                                                                                                              */ 	
/* Modified by: Sandeep Kharmate                                                                                */
/* Date		  : 09/04/2014																						                                                                   */
/* Details	  : 48293 - Allowances- restrict tilde for salary pay range.								                                 */ 
/*                                                                                                              */ 			
/* Modified by: Bala A                                                                                          */
/* Date		  : 10/02/2015 4																						                                                                 */
/* Details	  : PRB9269: Getting error 4830 on WFC when a Service Agreement exists without                       */
/*              SWC Additional Payment Amount being entered. To resolve the issue when Payamount does not exist */
/*              do not retrieve payment type.							                                                            */ 
/*                                                                                                              */ 	
/* Modified by: Vikas Belote                                                                                    */
/* Date		  : 04/09/2015																						*/
/* Details	  : PRB11329: Modified the code to calculate spot allowance amount when spot allowance start date   */
/*                        and end date in same month and end date must be less than 28 day of month.            */ 
/*--------------------------------------------------------------------------------------------------------------*/

SET NOCOUNT ON

--- Delcare the local variables
DECLARE @idoc INT
        ,@census_date SMALLDATETIME
        ,@previous_year_census_date SMALLDATETIME        
		,@term VARCHAR(3)
		,@Collection_period_start_date SMALLDATETIME
		,@school_type VARCHAR(10)			   -- To hold school type	
		,@academy_converted_date SMALLDATETIME -- To hold date when school converted to academy
		,@edit_paydetails BIT
		,@lea_numbar INT 
		,@is_spot_allowance_as_actual_amount bit
		,@spot_allowance_id int

-- Declare temp table to store person ids
DECLARE @tblAdditionalPayment TABLE
(                           
    person_id   INT
) 

-- Declare temp table to store additional payment details 
DECLARE @tblAdditionalPaymentInfo TABLE(                           
                                            person_id				INT,
                                            contract_id				INT,
                                            payment_type			VARCHAR(10) COLLATE DATABASE_DEFAULT,
                                            payment_amount			DECIMAL(10,2),
                                            annual_amount			DECIMAL(10,2),
                                            start_date				SMALLDATETIME,
                                            end_date				SMALLDATETIME,
                                            period_month			SMALLINT,
                                            allowance_start_date	SMALLDATETIME,
                                            allowance_end_date	    SMALLDATETIME,
                                            allowance_type_id       INT,
                                            pay_factor				DECIMAL(7,4),
                                            applicable_month		SMALLINT,
                                            contract_service_flag   VARCHAR(1) COLLATE DATABASE_DEFAULT       
			                           ) 
			                           
select @is_spot_allowance_as_actual_amount = spot_allowance_as_actual_amount 
  from sims.swfr_return 
 where return_id = @return_id 
 
 select @spot_allowance_id = lookup_value_id
 from sims.per_contract_allowance_type 
 where code = 'S' --Spot allowance
			                           
-- Assign term
SELECT @term = code FROM sims.swfr_census_term sct
INNER JOIN sims.swfr_return sr ON sr.census_term_id = sct.census_term_id
WHERE sr.return_id = @return_id

-- Set term, if return_id is null
-- Commneted as belwo condition not require - code review
--IF (@return_id IS NULL)
--BEGIN
Declare @IsPRU bit
select @IsPRU = sims.census_fnb_check_school_type_for_PRU() 

If(@IsPRU = 1)
	Set @term = 'PRU'
Else
	Set @term = 'AUT'
--END

--Assigning current census date and previous year census date
SELECT @census_date = census_date
      ,@previous_year_census_date = previous_year_census_date 
      ,@Collection_period_start_date = DATEADD(DD,1,previous_year_census_date)  
FROM sims.swfr_census_term
WHERE code = @term
-- Set school type
SELECT @school_type = st.code from sims.school_type st 
INNER JOIN sims.sims_school ss ON ss.type_id = st.lookup_value_id
WHERE ss.status = 'S'

-- Set academy transformed date
SELECT @academy_converted_date = ISNULL(date_dfe_number_changed, @Collection_period_start_date)
FROM sims.sims_school_previous_information

-- Set SWC configuration setting for edit pay details
SELECT @edit_paydetails = CASE WHEN edit_paydetail_details='T' THEN 1 ELSE 0 END
FROM sims.swc_configure_setting
/*Check lea number for Kent school, if it is 886 then do not apply pro rata*/
SELECT @lea_numbar = lea_number FROM sims.sims_school WHERE STATUS = 'S'

-- main body

--Create an internal representation of the XML document.   
EXEC sp_xml_preparedocument @idoc OUTPUT, @xml_staff

INSERT INTO @tblAdditionalPayment
(
    person_id 
)
SELECT 
       	p

FROM 	OPENXML (@idoc, '/DataPackage/Row',1)  
WITH     
      	(
          	p	INT	'person_id'
      	)
EXEC sp_xml_removedocument @idoc 

-- MAIN PROCEDURE BODY GOES HERE!

IF (@show_contractDetails = 1)
BEGIN

INSERT INTO @tblAdditionalPaymentInfo(                           
                                           person_id,
                                           contract_id,
                                           payment_type,
                                           payment_amount,
                                           annual_amount,
                                           start_date,
                                           end_date,
                                           period_month,
                                           allowance_start_date,
                                           allowance_end_date,
                                           allowance_type_id,
                                           pay_factor,
                                           contract_service_flag
			                          ) 
SELECT
    [person_id]      = con.person_id 
    ,[contract_id]   = con.contract_id 
    ,[payment_type]  = paccat.code 
    ,[payment_amount]= cal.amount * cal.pay_factor -- MIS reference no.1006-722186
    ,CASE WHEN @is_spot_allowance_as_actual_amount = 1 AND cal.allowance_type_id = @spot_allowance_id THEN NULL
          ELSE cal.amount
     END     
    ,CASE WHEN cal.start_date < @Collection_period_start_date
		  THEN @Collection_period_start_date
		  ELSE cal.start_date
	 END		  	
    ,CASE WHEN cal.end_date IS NULL OR cal.end_date > @census_date
		  THEN @census_date
		  ELSE cal.end_date
	 END			  
    ,CASE WHEN cal.end_date IS NOT NULL AND DATEPART(DD,cal.end_date)>= 28  
		  THEN 1 + DATEDIFF(MM,(DATEADD(DD,-(DAY(DATEADD(MM,1,start_date))-1)+27,DATEADD(MM,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(MM,1,cal.end_date))-1)+27,DATEADD(MM,0,cal.end_date))))	
		  WHEN cal.end_date IS NOT NULL
		  THEN DATEDIFF(MM,(DATEADD(DD,-(DAY(DATEADD(MM,1,start_date))-1)+27,DATEADD(MM,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(MM,1,cal.end_date))-1)+27,DATEADD(MM,0,cal.end_date))))	
		  WHEN cal.end_date IS NULL
		  THEN 12
	 END
	 ,cal.start_date
     ,cal.end_date
     ,cal.allowance_type_id
     ,cal.pay_factor
     ,'C'
FROM
    @tblAdditionalPayment tap
    INNER JOIN sims.per_contract con
        ON tap.person_id = con.person_id
    INNER JOIN sims.per_service_term st
        ON con.service_term_id = st.service_term_id         
	LEFT JOIN sims.per_employment_type pet 
		ON pet.lookup_value_id = con.employment_type_id
	LEFT JOIN sims.per_employment_type_category petc
        ON petc.category_id = pet.category_id
     AND
        (
            sims.swfr_fnb_check_regular_service_duration(petc.code,con.contract_start_date,con.contract_end_date,@term) = 1
         )
    LEFT JOIN sims.per_post spp 
        ON spp.post_id = con.post_id
    LEFT JOIN sims.per_swfr_post spsp 
        ON spp.swr_post_id = spsp.lookup_value_id  
    LEFT JOIN sims.per_contract_allowance cal
        ON con.contract_id = cal.contract_id
    LEFT JOIN sims.per_allowance pal
        ON cal.allowance_id = pal.allowance_id
    LEFT JOIN sims.per_allowance_category pac
        ON pal.category_of_additional_payment_id = pac.lookup_value_id
	LEFT JOIN sims.per_allowance_category_category paccat
        ON pac.category_id = paccat.category_id 
    LEFT JOIN sims.per_service_term pst
        ON pst.service_term_id = pal.service_term_id    
    WHERE (pac.code IS NOT NULL AND cal.amount IS NOT NULL)
	  AND ISNULL(cal.end_date,@census_date) > @previous_year_census_date 
	  AND cal.start_date <= @census_date
	  AND (			   -- Don't two customised Kent Service Terms
			( @for_kent = 0 
			  AND NOT (substring(pal.code,1,1) = '~' ) 
			  AND NOT (@lea_numbar = '886' AND ((st.code = 'KR' and st.description = 'Kent Range')
			                        or (st.code = 'KN' and st.description = 'Kent Range NN 32.5 Full Time')) 
									AND ((pal.[description] = 'Kent Range Lump Sum' and pal.code = 'KRLS') or
										(pal.[description] = 'Kent Range Salary Adjustment' and pal.code = 'KRSA'))
									AND ( paccat.description = 'other'and paccat.code ='OTH'))
			)
			OR
			(
			   @for_kent = 1 
			   AND (@census_date BETWEEN cal.start_date AND ISNULL(cal.end_date,@census_date)) /*Consider those allowances which are falling on census date*/
			   -- Include two customised Kent Service Terms
			    AND ((substring(pal.code,1,1) = '~') 
			          or  
			         (@lea_numbar = '886' AND ((st.code = 'KR' and st.description = 'Kent Range')
						          or (st.code = 'KN' and st.description = 'Kent Range NN 32.5 Full Time'))
										AND ((pal.[description] = 'Kent Range Lump Sum' and pal.code = 'KRLS') or
											(pal.[description] = 'Kent Range Salary Adjustment' and pal.code = 'KRSA'))
										AND ( paccat.description = 'other'  and paccat.code ='OTH')
					 )	
					)
			)
		  )	
    ORDER BY con.person_id

/* When school type is academis then consider start date as Academy start date for calculation.*/
IF @school_type = '49'
BEGIN

	UPDATE @tblAdditionalPaymentInfo
	   SET start_date			= @academy_converted_date,
	       allowance_start_date = @academy_converted_date
	 WHERE start_date < @academy_converted_date	

END

/*
SWC AUT 2012 - 
1.If an allowance starts and ends within the collection period then the total amount must be included. 
2.If dates are not lying between collection period then divide amount by 12 and multiply by month for which 
3.Consider payment date as 28th of every month.  
4.calculation for payment amount = (Payment amount/Month period) * Applicable Month*
*/
UPDATE @tblAdditionalPaymentInfo
   SET payment_amount = CASE WHEN DATEPART(DD,start_date) <= 28 AND DATEPART(DD,end_date) >= 28
                             THEN (payment_amount/ 12 ) * (1 + DATEDIFF(MM,(DATEADD(dd,-(DAY(DATEADD(mm,1,start_date))-1)+27,DATEADD(mm,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(mm,1,end_date))-1)+27,DATEADD(mm,0,end_date))))) 
                             WHEN  DATEPART(DD,start_date) > 28 AND DATEPART(DD,end_date) < 28
                             THEN (payment_amount/ 12 ) * (-1 + DATEDIFF(MM,(DATEADD(dd,-(DAY(DATEADD(mm,1,start_date))-1)+27,DATEADD(mm,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(mm,1,end_date))-1)+27,DATEADD(mm,0,end_date))))) 
                             ELSE (payment_amount/ 12 ) * (DATEDIFF(MM,(DATEADD(dd,-(DAY(DATEADD(mm,1,start_date))-1)+27,DATEADD(mm,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(mm,1,end_date))-1)+27,DATEADD(mm,0,end_date))))) 
						END,
       applicable_month = CASE WHEN DATEPART(DD,start_date) <= 28 AND DATEPART(DD,end_date) >= 28
                               THEN (1 + DATEDIFF(MM,(DATEADD(dd,-(DAY(DATEADD(mm,1,start_date))-1)+27,DATEADD(mm,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(mm,1,end_date))-1)+27,DATEADD(mm,0,end_date))))) 
                               WHEN DATEPART(DD,start_date) > 28 AND DATEPART(DD,end_date) < 28
                               THEN (-1 + DATEDIFF(MM,(DATEADD(dd,-(DAY(DATEADD(mm,1,start_date))-1)+27,DATEADD(mm,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(mm,1,end_date))-1)+27,DATEADD(mm,0,end_date))))) 
                               ELSE (DATEDIFF(MM,(DATEADD(dd,-(DAY(DATEADD(mm,1,start_date))-1)+27,DATEADD(mm,0,start_date))),(DATEADD(dd,-(DAY(DATEADD(mm,1,end_date))-1)+27,DATEADD(mm,0,end_date))))) 
						  END                             						                             
FROM @tblAdditionalPaymentInfo						
WHERE period_month > 0		
  AND end_date > start_date
  AND @for_kent = 0
  AND (@is_spot_allowance_as_actual_amount = 0 OR (@is_spot_allowance_as_actual_amount = 1 AND allowance_type_id != @spot_allowance_id))
  
  DELETE FROM @tblAdditionalPaymentInfo
  WHERE payment_amount <= 0
  
END

/*Insert service agreement details values*/
IF @show_serviceDetails = 1
BEGIN

	INSERT INTO @tblAdditionalPaymentInfo(                           
											   person_id,
											   contract_id,
											   payment_type,
											   payment_amount,
											   annual_amount,
											   start_date,
											   end_date,
											   period_month,
											   allowance_start_date,
											   allowance_end_date,
											   pay_factor,
											   contract_service_flag
										  ) 
	SELECT
		spsa.person_id,
		spsa.service_agreement_id,
		'OTH',
		spsa.swc_additional_payment,
		null,
		null,
		null,
		12,
		null,
		null,
		null,
		'S'
    FROM @tblAdditionalPayment tap 
   INNER JOIN sims.per_service_agreement spsa 
		ON spsa.person_id = tap.person_id
   LEFT JOIN sims.per_service_agreement_type spsat 
		ON spsat.lookup_value_id = spsa.service_type_id                  
   LEFT JOIN sims.per_employee spe1 
		ON spsa.person_id = spe1.person_id  
   LEFT JOIN (select peh1.person_id,peh1.employment_start_date 
                from sims.per_employment_history peh1
                LEFT JOIN sims.per_employment_history peh2 
                  ON peh1.person_id  = peh2.person_id
                 AND peh2.employment_start_date < peh1.employment_start_date
               WHERE peh2.ID IS NULL AND peh1.employment_start_date is not null        				              
              ) AS peh
        ON  spe1.person_id = peh.person_id   
    LEFT JOIN sims.per_swfr_post spsp
        ON spsa.swr_post_id = spsp.lookup_value_id   
    INNER JOIN sims.per_swfr_post psp 
		ON psp.lookup_value_id = spsa.swr_post_id
	WHERE spsa.daily_rate = 0
	  AND
       (
		sims.swfr_fnb_check_regular_service_duration(spsat.code,spsa.service_start_date,spsa.service_end_date,@term) = 1
       )		
END


update @tblAdditionalPaymentInfo
set payment_type = null
where payment_amount is null

-- Retrieve data
  SELECT                   
	distinct person_id,
    contract_id,
    payment_type, 
    payment_amount,
    annual_amount,
    allowance_start_date,
    allowance_end_date,
    pay_factor,
    applicable_month,
    contract_service_flag
FROM @tblAdditionalPaymentInfo
WHERE ((@for_kent = 0 AND period_month > 0) OR @for_kent = 1 or (@is_spot_allowance_as_actual_amount = 1 AND allowance_type_id = @spot_allowance_id))
  AND ( @school_type <> '49'
		 OR
		(@school_type = '49' AND ISNULL(allowance_end_date,@academy_converted_date) >= @academy_converted_date)		 
	  )	
 
  
-- END OF PROCEDURE BODY!
-----------------------------------------------------------
RETURN 0 -- return value if all OK.
GO


-----------------------------------------------------------
-- START OF UNIT TEST SECTION

/*  TEST
Description : Test of functioning
Author      : R Vijayavel
Date        : 13/03/2012
Dataset     : English Secondary
 
-- Perform the test
declare @xml_staff varchar(8000)
	set @xml_staff = '<DataPackage><Row><person_id>6</person_id></Row><Row><person_id>47</person_id></Row>
	                  <Row><person_id>8872</person_id></Row><Row><person_id>8873</person_id></Row>
                   </DataPackage>'
--get the staff id for the Return,and pass it as parameter to Get Additional Payments SP
exec sims.sre_pix_datapackage_swc_GetAdditionalPaymentsDetails 1,16, @xml_staff,1,1
 
-- Result after execution of sp

  person_id   contract_id payment_type payment_amount                          annual_amount                           allowance_start_date    allowance_end_date      pay_factor                              applicable_month
----------- ----------- ------------ --------------------------------------- --------------------------------------- ----------------------- ----------------------- --------------------------------------- ----------------
6           270         TLR          4224.00                                 4224.00                                 2010-09-01 00:00:00     NULL                    1.0000                                  12
8872        374         TLR          3293.01                                 10702.00                                2011-01-10 00:00:00     NULL                    0.3077                                  12
8873        370         MAL          183.33                                  2200.00                                 2011-12-01 00:00:00     2011-12-31 23:59:00     1.0000                                  1
8873        370         MAL          276.00                                  3312.00                                 2012-01-01 00:00:00     2012-02-14 23:59:00     1.0000                                  1
8873        370         TLR          4375.00                                 7500.00                                 2012-02-18 00:00:00     2012-08-31 00:00:00     1.0000                                  7
8873        370         TLR          1300.00                                 7800.00                                 2012-09-01 00:00:00     NULL                    1.0000                                  2




*/
-- END OF UNIT TEST SECTION
-----------------------------------------------------------








-- END PATCH SCRIPT 1
GO


-- START PATCH SCRIPT 2

------------------------------------------------------------------------------------------------------------------
--                                                                                                              --
-- Module     : SIMS Database SQL Server 2008                                                                   --
-- Platform   : Pulsar                                                                                          --
-- Area       : Curriculum                                                                                      --
-- Copyright  : (c) Capita Education Services 1984-2015                                                         --
--            : This code has been written by Capita Education Services for use within its own SIMS systems.    --
--            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    --
--            : means or used without the express written permission of Capita Education Services.              --
-- Status     : UNREVIEWED                                                                                      --
--                                                                                                              --
------------------------------------------------------------------------------------------------------------------
--                                                                                                              --
-- Format     : Transact-SQL                                                                                    --
-- Object     : sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2                                                --
--                                                                                                              --
-- Purpose    :                                                                                                 --
-- Comments   :                                                                                                 --
--                                                                                                              --
-- Author     : David Stott                                                                                     --
-- Date       : 30 Mar 2012                                                                                     --
--                                                                                                              --
-- Modified By: JPhatan																							--
-- Date       : 14 Sep 2012																						--
-- Purpose    : PRB2271 Fix Applicaiton crash when there is no "next" academic year created,and a student has   --
--              a course membership that extends into the time after the current AY finishes 	                --
--                                                                                                              --
-- Modified By: Ged  																							--
-- Date       : Apr 2015																						--
-- Purpose    : Extended end date for first year of a 2-year course                                             --
--
-- Modified By: Ged
-- Date       : May 2015
-- Purpose    : Modified handling of learning aims to deal with records having Retake or UKPRN but no completion status

-- Modified By: Manisha
-- Date       : 31/08/2015
-- Purpose    : PRB11085 (Post 16 Learning Aims - Transferred status and continuity tolerance)
--				Update/Remove Transferred status if student rejoins class within limit of continuity tolerance
------------------------------------------------------------------------------------------------------------------

set nocount on
go

--------------------------------------------------------------------------------------------------------------

exec sims.db_p_drop     'sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2'
,                       'p'
go
ALTER     procedure sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2
(
     @AcadYear_event_instance_id INT
    ,@PersonID       INT  = 0     -- optional parameter if to be executed only for an individual student
    ,@PersonXml      TEXT = NULL  -- optional parameter if to be executed only for a set of students
    ,@DebugStudentId INT  = 0     -- debug parameter : will report on data for this StudentId
    ,@DebugCourseId  INT  = 0     -- debug parameter : will report on data for this CourseId
    ,@Diagnosis      INT  = 0     -- debug parameter : will explain why certain courses and learning aims were not updated
)

/* example 
       @PersonXml = '<data> 
				<row PersonId=''1234'' /> 
				<row PersonId=''1236'' /> 
				<row PersonId=''1334'' />  
                    </data>'
*/

--with encryption
as

/*****************************************************************************************************************
The stored procedure consists of the following sections

--==KB95294==
Legacy call to tidy Class memberships where the person is not actually a student or an applicant

--==INITVARS==
Initialise the variables that are required (eg AY dates, departments)

--==CREATETEMPTABLES==
Declare temporary tables (#Students, #NCYearMembers, #OldCourseMembers, #UnboundedCourseMembers, #NewCourseMembers, #MembershipBounds)

--==STUDENTS=
Populate #Students with specified list of students to process (if restricted)

--==COURSETOLERANCES==
Populate #Tolerances with calculated continuity tolerances, completion tolerances and course lengths for each course

--==OLDCOURSEMEMBERS==
Populate #OldCourseMembers with all relevant course memberships which intersect the specified AY

--==POPULATEFROMCLASSMEMBERSHIPS==
Populate #NewCourseMembers with course memberships calculated from class memberships within the specified AY

--==POPULATEBOUNDARYFRACTIONS==
Populate #NewCourseMembers with "outside" fractions of existing course memberships that overlap AY boundaries

--==POPULATENEIGHBOURS==
Populate #NewCourseMembers with existing course memberships from neighbouring AYs that match ones already in #NewCourseMembers

--==MERGECOURSEMEMBERS==
Iterate merge in #NewCourseMembers until records are all distinct

--==UPDATEMEMBERSHIP==
Update sims_member table: delete/add records that do not exactly match records in #NewCourseMembers

--==ACTIVEFLAG==
Legacy call to recalculate the "active flag" status

--==LEARNINGAIMS==
Update Learning Aims (curr_p16_student_aim)

--==DROPTEMPTABLES===
Drop the temporary tables
******************************************************************************************************************/

--print 'sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2'
set nocount on

declare @retcode   int
select  @retcode   = 0

--==KB95294================================================================================================
-- KB95294 start
if @PersonID = 0
begin
    delete		mem
    from		sims.sims_member mem
    join		sims.sims_base_group bg
    on			bg.base_group_id = mem.base_group_id
    join		sims.sims_base_group_type bgt
    on			bg.base_group_type_id = bgt.base_group_type_id	
    left join	sims.stud_student stud
    on			stud.person_id = mem.person_id	
    where		(bgt.code in ('Class','Curric') or bgt.on_roll_only = 'T')	
    and			stud.person_id is null	
    and not exists ( select 1 
				    from	sims.adm_application ap
				    join	sims.adm_application_status apst
				    on		ap.application_status_id = apst.lookup_value_id
				    join	sims.adm_application_status_category cat
				    on		cat.category_id = apst.category_id
				    and		not cat.code in ('rejected','withdrawl') 
				    where	ap.person_id = mem.person_id 
				    ) 	

    exec sims.gm_pia_virtual_groups_rebuild_all null, null  -- Virtual group rebuild.
    -- KB95294 end
end 

--==INITVARS=======================================================================================
-------------------------------------------------
--Initialise date variables  

DECLARE @ThisAYStart DateTime
DECLARE @ThisAYEnd DateTime
DECLARE @AcadYearInNormalRange BIT
DECLARE @PrevAYStart smalldatetime
DECLARE @PrevAYEnd smalldatetime
DECLARE @NextAYStart smalldatetime
DECLARE @NextAYEnd smalldatetime
DECLARE @EndOfTime smalldatetime
DECLARE @CurrentDate date

DECLARE @TotalRecordsBefore INT
DECLARE @TotalRecordsAfterInitialMerge INT
DECLARE @TotalRecordsAfterFullMerge INT

SET @EndOfTime = '20790606'

--set @CurrentDate (a raw Date type) to the START of tomorrow
--used by Learning Aims section: do not create Learning Aim record if new actual course membership end date is earlier than tomorrow
SELECT @CurrentDate = DATEADD(DAY,1,GETDATE())

-- Set Start and End dates of the selected Academic Year and the ones either side
SELECT  @ThisAYStart = sei.event_start, @ThisAYEnd = sei.event_end   
  FROM  sims.sims_event_instance sei
 WHERE sei.event_instance_id = @AcadYear_event_instance_id
 
SELECT @PrevAYStart = event_start, @PrevAYEnd = event_end -- these may be NULL
  FROM sims.sims_event_instance sei
  JOIN sims.sims_event se ON sei.event_id = se.event_id
  JOIN sims.sims_event_type st on se.event_type_id = st.event_type_id 
  WHERE st.code = 'AcadYear' AND sei.event_end = @ThisAYStart - '00:01'

--PRB2271  
IF @PrevAYEnd IS NULL
BEGIN
   SELECT @PrevAYEnd = DATEADD(MINUTE,-1,@ThisAYStart) 
   SELECT @PrevAYStart = DATEADD(YEAR,-1,@PrevAYEnd)
END
  
  
SELECT @NextAYStart = event_start, @NextAYEnd = event_end -- these may be NULL
  FROM sims.sims_event_instance sei
  JOIN sims.sims_event se ON sei.event_id = se.event_id
  JOIN sims.sims_event_type st on se.event_type_id = st.event_type_id 
  WHERE st.code = 'AcadYear' AND sei.event_start = @ThisAYEnd + '00:01' 
  
--PRB2271
IF @NextAYStart IS NULL
BEGIN
   SELECT @NextAYStart = DATEADD(MINUTE,1,@ThisAYEnd) 
   SELECT @NextAYEnd = DATEADD(YEAR,1,@NextAYStart)
END 
  

--Normally the AY boundary lies in August or September. Adjusted Course Length check in amalgamation routine should only be used if this is true
SELECT @AcadYearInNormalRange = CASE WHEN DATEPART(MONTH,@ThisAYStart) IN (8,9) THEN 1 ELSE 0 END

 -------------------------------------------------
-- Find the "departments" that are Curriculum and Alternative Curriculum 
-- We are not intersted in other departments (eg Pastoral)
DECLARE @curriculum_department_id int
SELECT @curriculum_department_id = department_id from sims.curr_department where name = 'Curriculum'

DECLARE @alternative_department_id int
SELECT @alternative_department_id = department_id from sims.curr_department where name like 'Alternative%'

--==CREATETEMPTABLES================================================================================
--Create temporary tables for course membership calculation

CREATE TABLE #NCYearMembers -- the memberships of the qualifying NCYears (11..14)
( 
   person_id         int
  ,startDate         smalldatetime
  ,endDate           smalldatetime
)

CREATE TABLE #OldCourseMembers -- filled with the course membership records that intersect the selected AY. Used for Learning Aim calculations
(
   PersonId         int
  ,CourseId         int
  ,StartDate        smalldatetime
  ,EndDate          smalldatetime
)

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF @Diagnosis=1
BEGIN
	CREATE TABLE #ExcludedOldCourseMembers -- filled with the course membership records that intersect the selected AY but where the student's NCYear is below the min YTI. 
	(
	   PersonId         int
	  ,CourseId         int
	  ,StartDate        smalldatetime
	  ,EndDate          smalldatetime
	)
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS

CREATE TABLE #UnboundedCourseMembers -- filled with the course membership records that have null end dates. Will be excluded
(
   PersonId         int
  ,CourseId         int
  ,StartDate        smalldatetime
)

CREATE TABLE #NewCourseMembers -- filled with new course membership records calculated from class memberhsips, and from the surrounding AYs
( 
   rec_id            int identity(1, 1)
  ,person_id         int
  ,course_id         int
  ,start_date        smalldatetime
  ,end_date          smalldatetime
)

CREATE TABLE #NewCourseMembersSorted -- filled after the first merge, in prepration for the "continuity" merge procedure
( 
   rec_id            int identity(1, 1)
  ,person_id         int
  ,course_id         int
  ,start_date        smalldatetime
  ,end_date          smalldatetime
)

CREATE TABLE #MembershipBounds -- calculated for each student/course combination from the earliest start and latest end dates, in order to specify a scope for the additions and deletions
( 
   person_id         int
  ,course_id         int
  ,minDate           smalldatetime
  ,maxDate           smalldatetime
)

CREATE TABLE #Students -- a list of the students that have been specified for the whole process (if any restriction has been made)
(
   PersonId         int
)

CREATE TABLE #Tolerances -- continuity tolerance and maximum length data calculated for each course from the default and specific values in the DB
( 
   course_id             int
  ,cont_tolerance_code   varchar(8)
  ,comp_tolerance_code   varchar(8)
  ,contMagnitude         int
  ,contTolType			 char
  ,compMagnitude         int
  ,compTolType			 char
  ,maxlength		     int 
)

--==STUDENTS========================================================================================
--Build the list of students to process (#Students), if restricted set supplied. Otherwise @PersonId is 0.

/* example @PersonXml = '<data> 
				<row PersonId=''1234'' /> 
				<row PersonId=''1236'' /> 
				<row PersonId=''1334'' />  
                        </data>'
*/

IF (@PersonXml IS NOT NULL) 
BEGIN
    SET @PersonID = -1
	DECLARE @iDoc INT
	EXEC SP_XML_PREPAREDOCUMENT @iDoc OUTPUT, @PersonXml
	INSERT INTO #Students (PersonId)
	SELECT DISTINCT PersonId FROM OPENXML(@iDoc,'//data/row', 1)
	WITH ( PersonId int )
	EXEC SP_XML_REMOVEDOCUMENT @iDoc 
END ELSE
IF (@PersonID > 0)
BEGIN
   INSERT INTO #Students (PersonId) 
   VALUES (@PersonID)
END


--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS         
IF (@Diagnosis = 1)
BEGIN
   DECLARE @CourseDiagnosis TABLE
   ( CourseId int
    ,PersonId int
    ,Course   varchar(32)
    ,Student  varchar(60)
    ,Comment  varchar(200)
    )
INSERT INTO @CourseDiagnosis        
   SELECT DISTINCT
        [course_id]    = ccd.course_group_id   
       ,[person_id]    = sm.person_id    
       ,[Course]       = sbg.short_name          
       ,[Student]      = sp.surname+', '+sp.forename     
       ,[Reason]       = 'Auto-Update Course is OFF'    
 FROM sims.sims_member sm -- the class membership
 JOIN sims.sims_person sp ON sm.person_id = sp.person_id
 JOIN sims.curr_component_delivery ccd 
   ON sm.base_group_id = ccd.class_group_id
 JOIN sims.curr_course cc
   ON cc.base_group_id = ccd.course_group_id -- make sure its a Curriculum course or an Alternative Curriculum course
  AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
 JOIN sims.sims_base_group sbg ON ccd.course_group_id= sbg.base_group_id

 LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
  AND cc.auto_update_membership = 0 -- the course is NOT flagged for auto update of membership
  AND sm.start_date < @ThisAYEnd AND @ThisAYStart < ISNULL(sm.end_date, @EndOfTime) -- class membership intersects AY 
  AND ccd.start_date < @ThisAYEnd AND @ThisAYStart < ISNULL(ccd.end_date, @EndOfTime) -- class/course intersects AY 
  AND sm.start_date < ISNULL(ccd.end_date, @EndOfTime) AND ccd.start_date < ISNULL(sm.end_date, @EndOfTime) -- Make sure ccd and sm date ranges overlap!
  
INSERT INTO @CourseDiagnosis        
   SELECT DISTINCT
        [course_id]    = cce.course_id   
       ,[person_id]    = cce.person_id    
       ,[Course]       = sbg.short_name          
       ,[Student]      = sp.surname+', '+sp.forename     
       ,[Reason]       = 'Course Exception Record'    
  FROM sims.curr_course_exception cce
 JOIN sims.sims_person sp ON cce.person_id = sp.person_id
 JOIN sims.sims_base_group sbg ON cce.course_id = sbg.base_group_id 
 LEFT JOIN #Students s ON s.PersonId = cce.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
   AND cce.academic_year_id = @AcadYear_event_instance_id 
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS  


--==OLDCOURSEMEMBERS================================================================================
--Build list of existing course memberships that intersect the selected date range(academic year)
--This is used to create NEW Learning Aims. So we only want those that intersect memberships of 
--NCYears equal to or greater than the min YTI specified in course_manager_setting

DECLARE @MinYTIOrderIndex INT = 0

SELECT @MinYTIOrderIndex = sbgsl.display_order from sims.course_manager_setting cms
JOIN sims.sims_base_group_set_link sbgsl ON cms.course_manager_yti_id = sbgsl.base_group_id
JOIN sims.sims_base_group_set sbgs ON sbgsl.base_group_set_id = sbgs.base_group_set_id
JOIN sims.sims_base_group_type sbgt on sbgs.base_group_type_id = sbgt.base_group_type_id AND sbgt.code = 'NCYEAR'
WHERE GETDATE() BETWEEN ISNULL(sbgs.start_date,'19010101') AND ISNULL(sbgs.end_date,'20760606')

DECLARE @NCYears TABLE ( YearId int)
INSERT INTO @NCYears SELECT sbg.base_group_id 
FROM sims.sims_base_group sbg
JOIN sims.sims_base_group_set_link sbgsl ON sbg.base_group_id = sbgsl.base_group_id
JOIN sims.sims_base_group_set sbgs ON sbgsl.base_group_set_id = sbgs.base_group_set_id
JOIN sims.sims_base_group_type sbgt on sbgs.base_group_type_id = sbgt.base_group_type_id AND sbgt.code = 'NCYEAR'
WHERE sbg.active_state = 'A' AND GETDATE() BETWEEN ISNULL(sbgs.start_date,'19010101') AND ISNULL(sbgs.end_date,'20760606')
AND sbgsl.display_order >= @MinYTIOrderIndex


INSERT INTO #NCYearMembers ( person_id, startDate, endDate )
SELECT sm.person_id, sm.start_date, ISNULL(sm.end_date,@EndOfTime) 
FROM sims.sims_member sm
JOIN @NCYears ncy ON sm.base_group_id = ncy.YearId
LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 

-------------------------------------------------
-- now populate #OldCourseMembers with existing course memberships
INSERT INTO #OldCourseMembers
(
  PersonId
 ,CourseId
 ,StartDate  
 ,EndDate
)
SELECT DISTINCT 
        [person_id]  = sm.person_id
       ,[course_id]  = sm.base_group_id 
       ,[start_date] = sm.start_date
       ,[end_date]   = sm.end_date 
  FROM sims.sims_member sm
  JOIN sims.curr_course cc
    ON sm.base_group_id = cc.base_group_id  -- make sure its a Curriculum course or an Alternative Curriculum course
   AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
  JOIN #NCYearMembers ncm ON sm.person_id = ncm.person_id AND sm.start_date < ncm.endDate AND ncm.startDate < sm.end_date
  LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
   AND (sm.start_date < @ThisAYEnd)
   AND (@ThisAYStart < sm.end_date)
 
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS 
IF (@Diagnosis = 1)
BEGIN 
	INSERT INTO #ExcludedOldCourseMembers
	(
	  PersonId
	 ,CourseId
	 ,StartDate  
	 ,EndDate
	)
	SELECT DISTINCT 
			[person_id]  = sm.person_id
		   ,[course_id]  = sm.base_group_id 
		   ,[start_date] = sm.start_date
		   ,[end_date]   = sm.end_date 
	  FROM sims.sims_member sm
	  JOIN sims.curr_course cc
		ON sm.base_group_id = cc.base_group_id  -- make sure its a Curriculum course or an Alternative Curriculum course
	   AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
	  LEFT JOIN #NCYearMembers ncm ON sm.person_id = ncm.person_id AND sm.start_date < ncm.endDate AND ncm.startDate < sm.end_date
	  LEFT JOIN #Students s ON s.PersonId = sm.person_id 
	 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
	   AND (sm.start_date < @ThisAYEnd)
	   AND (@ThisAYStart < sm.end_date)
	   AND ncm.person_id IS NULL
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG1]     = 'OldCourseMembers'
        ,[StudentId]  = ocm.PersonId
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ocm.CourseId
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),ocm.StartDate,106)
        ,[EndDate]    = CONVERT(varchar(12),ocm.EndDate,106)
   FROM #OldCourseMembers ocm 
   JOIN sims.sims_base_group sbg ON ocm.CourseId = sbg.base_group_id
   JOIN sims.sims_person sp ON ocm.PersonId = sp.person_id
  WHERE (ocm.PersonId = @DebugStudentId OR @DebugStudentId = 0) 
    AND (ocm.CourseId = @DebugCourseId  OR @DebugCourseId = 0)
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-  
  
        
  
INSERT INTO #UnboundedCourseMembers (PersonId,CourseId,StartDate)
SELECT DISTINCT 
        [person_id]  = sm.person_id
       ,[course_id]  = sm.base_group_id 
       ,[start_date] = sm.start_date
  FROM sims.sims_member sm
  JOIN sims.curr_course cc
    ON sm.base_group_id = cc.base_group_id  -- make sure its a Curriculum course or an Alternative Curriculum course
   AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
  LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
   AND (sm.end_date IS NULL)

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS 
IF (@Diagnosis = 1)
BEGIN 
INSERT INTO @CourseDiagnosis        
   SELECT DISTINCT
        [course_id]    = ucm.CourseId
       ,[person_id]    = ucm.PersonId    
       ,[Course]       = sbg.short_name          
       ,[Student]      = sp.surname+', '+sp.forename     
       ,[Reason]       = 'Unbounded Course Membership [from '+CONVERT(varchar(12),ucm.startDate,106)+']'    
  FROM #UnboundedCourseMembers ucm
 JOIN sims.sims_person sp ON ucm.PersonId = sp.person_id
 JOIN sims.sims_base_group sbg ON ucm.CourseId = sbg.base_group_id 
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS 

--==COURSETOLERANCES========================================================================
--Build local table to translate length ids into number of days

DECLARE  @MaxLengthDays TABLE (length_id int, numdays int)  
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,7*2   FROM sims.course_manager_duration where code = '1' --2 Weeks
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,7*24  FROM sims.course_manager_duration where code = '2' --24 Weeks  
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,365*1 FROM sims.course_manager_duration where code = '3' --One Year
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,365*2 FROM sims.course_manager_duration where code = '4' --Two Years
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,365*3 FROM sims.course_manager_duration where code = '5' --Three Years
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,365*4 FROM sims.course_manager_duration where code = '6' --Four Years
INSERT INTO @MaxLengthDays(length_id,numdays) SELECT id,365*5 FROM sims.course_manager_duration where code = '7' --Five Years

--Fill temporary table #Tolerances with calculated continuity tolerance values (default overide mechanism)
INSERT INTO #Tolerances (course_id,cont_tolerance_code)
SELECT [CourseId]  = cc.base_group_id
      ,[Tol]       = ISNULL(ct2.code,ISNULL(ct1.code,ct0.code))
FROM sims.curr_course cc
CROSS JOIN sims.course_manager_setting def
JOIN sims.course_manager_tolerance ct0 ON def.course_manager_continuity_tolerance_id = ct0.id
LEFT JOIN sims.curr_course_classification_link cccl 
     JOIN sims.course_manager_setting_level_config lev ON lev.level_id = cccl.curr_course_classification_id  
     JOIN sims.course_manager_tolerance ct1 ON lev.continuity_tolerance_id = ct1.id
  ON cc.base_group_id = cccl.course_id   
LEFT JOIN sims.course_manager_setting_course_config cmsc 
     JOIN sims.course_manager_tolerance ct2 ON cmsc.continuity_tolerance_id = ct2.id
  ON cc.base_group_id = cmsc.course_id   
  
UPDATE t SET comp_tolerance_code = ISNULL(ct2.code,ISNULL(ct1.code,ct0.code))
FROM #Tolerances t
JOIN sims.curr_course cc ON t.course_id = cc.base_group_id
CROSS JOIN sims.course_manager_setting def
JOIN sims.course_manager_tolerance ct0 ON def.course_manager_completion_tolerance_id = ct0.id
LEFT JOIN sims.curr_course_classification_link cccl 
     JOIN sims.course_manager_setting_level_config lev ON lev.level_id = cccl.curr_course_classification_id  
     JOIN sims.course_manager_tolerance ct1 ON lev.completion_tolerance_id = ct1.id
  ON cc.base_group_id = cccl.course_id   
LEFT JOIN sims.course_manager_setting_course_config cmsc 
     JOIN sims.course_manager_tolerance ct2 ON cmsc.completion_tolerance_id = ct2.id
  ON cc.base_group_id = cmsc.course_id    
  
--Set Magnitude and type of Continuity Tolerance (Days/Weeks/Months) by examining first two chars of tolerancecode (2Weeks, 3Months etc)      
UPDATE t SET contMagnitude = CAST(left(cont_tolerance_code,1) AS int), contTolType = SUBSTRING(cont_tolerance_code,2,1) 
FROM #Tolerances t
WHERE left(cont_tolerance_code,1) IN ('1','2','3','4','5','6','7','8','9')
   
--Set Magnitude and type of Completion Tolerance (Days/Weeks/Months) by examining first two chars of tolerancecode (2Weeks, 3Months etc)      
UPDATE t SET compMagnitude = CAST(left(comp_tolerance_code,1) AS int), compTolType = SUBSTRING(comp_tolerance_code,2,1) 
FROM #Tolerances t
WHERE left(comp_tolerance_code,1) IN ('1','2','3','4','5','6','7','8','9')   
        
--Set MaxLength for each course (another default overide mechanism)
UPDATE tol SET MaxLength = ISNULL(days2.numdays,ISNULL(days1.numdays,days0.numdays))
FROM #Tolerances tol
CROSS JOIN sims.course_manager_setting def
JOIN @MaxLengthDays days0 ON def.course_manager_normal_duration_id = days0.length_id
LEFT JOIN sims.curr_course_classification_link cccl 
     JOIN sims.course_manager_setting_level_config lev ON lev.level_id = cccl.curr_course_classification_id  
     JOIN @MaxLengthDays days1 ON lev.normal_duration_id = days1.length_id
  ON tol.course_id = cccl.course_id  
LEFT JOIN sims.course_manager_setting_course_config cmsc 
     JOIN  @MaxLengthDays days2 ON cmsc.normal_duration_id = days2.length_id
  ON tol.course_id = cmsc.course_id  
       
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
   SELECT [DEBUG2] = 'Tolerances'
   ,[Coursename] = sbg.short_name
   ,tol.*
   FROM #Tolerances tol
   JOIN sims.sims_base_group sbg ON tol.course_id = sbg.base_group_id
   WHERE tol.course_id = @DebugCourseId OR @DebugCourseId = 0
END
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-        
              
--==POPULATEFROMCLASSMEMBERSHIPS========================================================================
-------------------------------------------------
-- calculate simple list of new course memberships from the class memberships
-- (modified by Ged Apr 2015 to extend end date for 2 year courses)
INSERT INTO #NewCourseMembers
(
  person_id
 ,course_id
 ,start_date  
 ,end_date
)
SELECT 
        [person_id]    = sm.person_id
       ,[course_id]    = ccd.course_group_id  
       ,[start_date]   = sims.pub_fnd_maxdate(sm.start_date, ccd.start_date)
       ,[end_date]     = sims.curr_fnd_get_course_end_date( ccd.course_group_id, ccd.class_group_id,
						 CASE 
                           WHEN sm.end_date IS NULL THEN ccd.end_date
                           WHEN ccd.end_date IS NULL THEN sm.end_date
                           ELSE sims.pub_fnd_mindate(sm.end_date, ccd.end_date) 
                         END )
                
 FROM sims.sims_member sm -- the class membership
 JOIN sims.curr_component_delivery ccd 
   ON sm.base_group_id = ccd.class_group_id
 JOIN sims.curr_course cc
   ON cc.base_group_id = ccd.course_group_id -- make sure its a Curriculum course or an Alternative Curriculum course
  AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 

 LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
  AND cc.auto_update_membership = 1 -- the course is flagged for auto update of membership
  AND sm.start_date < @ThisAYEnd AND @ThisAYStart < ISNULL(sm.end_date, @EndOfTime) -- class membership intersects AY 
  AND ccd.start_date < @ThisAYEnd AND @ThisAYStart < ISNULL(ccd.end_date, @EndOfTime) -- class/course intersects AY 
  AND sm.start_date < ISNULL(ccd.end_date, @EndOfTime) AND ccd.start_date < ISNULL(sm.end_date, @EndOfTime) -- Make sure ccd and sm date ranges overlap!
  
-------------------------------------------------
--Just in case the class memberships didn't lie wholly inside AY: trim ranges in #NewCourseMembers back to Academic Year
UPDATE #NewCourseMembers SET start_date = @ThisAYStart WHERE start_date < @ThisAYStart
-- (modified by Ged Apr 2015 to exclude 2-year courses from this bit)
UPDATE #NewCourseMembers SET end_date = @ThisAYEnd WHERE (end_date > @ThisAYEnd OR end_date IS NULL) and sims.curr_fni_course_years(course_id) <= 1
DELETE FROM #NewCourseMembers WHERE end_date < start_date

--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG3]     = 'CRSMEM from classes'
        ,[StudentId]  = ncm.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ncm.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),ncm.start_date,106)
        ,[EndDate]    = CONVERT(varchar(12),ncm.end_date,106)
   FROM #NewCourseMembers ncm 
   JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id
   JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
  WHERE (ncm.person_id = @DebugStudentId OR @DebugStudentId = 0) 
    AND (ncm.course_id = @DebugCourseId  OR @DebugCourseId = 0)
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-


--==POPULATEBOUNDARYFRACTIONS==============================================================
-------------------------------------------------
--Add in fractions of old course memberships from previous AY (if it exists) which overlap boundary
--NB These are derived from course memberships that DO intersect the AY
INSERT INTO #NewCourseMembers
(
  person_id
 ,course_id
 ,start_date  
 ,end_date
)
SELECT 
  sm.person_id
 ,sm.base_group_id
 ,sm.start_date  
 ,@PrevAYEnd
 FROM sims.sims_member sm -- the course membership
 JOIN sims.curr_course cc
    ON sm.base_group_id = cc.base_group_id  -- make sure its a Curriculum course or an Alternative Curriculum course
   AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
   AND (cc.auto_update_membership = 1)
 LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
   AND sm.start_date < @ThisAYStart AND sm.end_date > @ThisAYStart
   AND @PrevAYEnd IS NOT NULL
   
-------------------------------------------------
--Add in fractions of old course memberships from following AY (if it exists) which overlap boundary
--NB These are derived from course memberships that DO intersect the AY
INSERT INTO #NewCourseMembers
(
  person_id
 ,course_id
 ,start_date  
 ,end_date
)
SELECT 
  sm.person_id
 ,sm.base_group_id
 ,@NextAYStart
 ,sm.end_date
 FROM sims.sims_member sm -- the course membership
 JOIN sims.curr_course cc
    ON sm.base_group_id = cc.base_group_id  -- make sure its a Curriculum course or an Alternative Curriculum course
   AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
   AND (cc.auto_update_membership = 1)
 LEFT JOIN #Students s ON s.PersonId = sm.person_id 
 LEFT JOIN #NewCourseMembers ncm2 -- check that we are not adding a duplicate of the same record because eg a 2 year course
        ON ncm2.person_id = sm.person_id
       AND ncm2.course_id = sm.base_group_id
	   AND ncm2.start_date = sm.start_date
 WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
   AND sm.start_date <= @ThisAYEnd AND sm.end_date > @ThisAYEnd   
   AND @NextAYStart IS NOT NULL
   AND ncm2.person_id IS NULL
   
 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG4]     = 'CRSMEM with Boundaries'
        ,[StudentId]  = ncm.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ncm.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),ncm.start_date,106)
        ,[EndDate]    = CONVERT(varchar(12),ncm.end_date,106)
   FROM #NewCourseMembers ncm 
   JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id
   JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
  WHERE (ncm.person_id = @DebugStudentId OR @DebugStudentId = 0) 
    AND (ncm.course_id = @DebugCourseId  OR @DebugCourseId = 0)
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG- 
   
--==POPULATENEIGHBOURS==============================================================
-------------------------------------------------
--Add in latest matching course memberships that end in previous AY
--NB These are derived from course memberships that DO NOT intersect the AY
INSERT INTO #NewCourseMembers
(
  person_id
 ,course_id
 ,start_date  
 ,end_date
)
SELECT DISTINCT
  sm1.person_id
 ,sm1.base_group_id
 ,sm1.start_date  
 ,sm1.end_date
FROM #NewCourseMembers ncm
JOIN sims.sims_member sm1 ON ncm.person_id = sm1.person_id AND ncm.course_id = sm1.base_group_id
LEFT JOIN sims.sims_member sm2 
  ON ncm.person_id = sm2.person_id AND ncm.course_id = sm2.base_group_id
 AND sm2.start_date > sm1.start_date AND sm2.end_date <= @PrevAYEnd
WHERE sm1.end_date BETWEEN @PrevAYStart AND @PrevAYEnd -- ends during previous AY
  AND sm2.person_id IS NULL -- there isnt a later example

-------------------------------------------------
--Add in matching course memberships from following AY
--NB These are derived from course memberships that DO NOT intersect the AY
INSERT INTO #NewCourseMembers
(
  person_id
 ,course_id
 ,start_date  
 ,end_date
)
SELECT DISTINCT
  sm1.person_id
 ,sm1.base_group_id
 ,sm1.start_date  
 ,sm1.end_date
FROM #NewCourseMembers ncm
JOIN sims.sims_member sm1 ON ncm.person_id = sm1.person_id AND ncm.course_id = sm1.base_group_id
LEFT JOIN sims.sims_member sm2 
  ON ncm.person_id = sm2.person_id AND ncm.course_id = sm2.base_group_id
 AND sm2.start_date > @NextAYStart AND sm2.start_date < sm1.start_date
WHERE sm1.start_date BETWEEN @NextAYStart AND @NextAYEnd -- starts during following AY
  AND sm2.person_id IS NULL -- there isnt an earlier example

--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG5]     = 'CRSMEM with Neighbours'
        ,[StudentId]  = ncm.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ncm.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),ncm.start_date,106)
        ,[EndDate]    = CONVERT(varchar(12),ncm.end_date,106)
   FROM #NewCourseMembers ncm 
   JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id
   JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
  WHERE (ncm.person_id = @DebugStudentId OR @DebugStudentId = 0) 
    AND (ncm.course_id = @DebugCourseId  OR @DebugCourseId = 0)
    
  SELECT @TotalRecordsBefore = COUNT(*) FROM #NewCourseMembers  
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG- 

--==MERGECOURSEMEMBERS========================================================================
-------------------------------------------------
--Build performance index on #NewCourseMembers
CREATE INDEX NewCourseMembers_person_id ON  #NewCourseMembers (rec_id,person_id,course_id)

-------------------------------------------------
--Initial amalgamation of #NewCourseMembers : repeat until no more merging occurs.
DECLARE @merged int
SELECT @merged = 1
WHILE @merged > 0
BEGIN

 --if two corresponding course memberships abut or overlap, [M1] ends earlier than [M2], set end date of [M1] forward to end date of [M2]   
   UPDATE ncm1 SET end_date = ncm2.end_date
   FROM #NewCourseMembers ncm1
   JOIN #NewCourseMembers ncm2 ON ncm1.person_id = ncm2.person_id AND ncm1.course_id = ncm2.course_id 
   WHERE ncm1.end_date < ncm2.end_date 
     AND ncm1.end_date >= ncm2.start_date - '00:01' --(adjustment to include abutting records)  

--delete redundant course memberships where [M1] lies inside [M2], or where [M1] and [M2] are identical but [M1] has larger rec_id.  
   DELETE ncm1 
   FROM #NewCourseMembers ncm1
   JOIN #NewCourseMembers ncm2 ON ncm1.person_id = ncm2.person_id AND ncm1.course_id = ncm2.course_id
   WHERE (ncm1.start_date > ncm2.start_date AND ncm1.end_date <= ncm2.end_date)
      OR (ncm1.start_date >= ncm2.start_date AND ncm1.end_date < ncm2.end_date) 
      OR (ncm1.start_date = ncm2.start_date AND ncm1.end_date = ncm2.end_date AND ncm1.rec_id > ncm2.rec_id)

  SELECT @merged = @@ROWCOUNT
END

--The records are now distinct, not abutting or overlapping
--So what remains to be done are the continuity calculations. This must be addressed serially, not using set-based logic

--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
   SELECT @TotalRecordsAfterInitialMerge = COUNT(*) FROM #NewCourseMembers
END
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-

-------------------------------------------------
--Copy records of #NewCourseMembers into sorted table: put all records of a given student/course are together, and in date order
INSERT INTO #NewCourseMembersSorted (person_id, course_id, start_date, end_date)
SELECT ncm.person_id, ncm.course_id, ncm.start_date, ncm.end_date 
FROM #NewCourseMembers ncm
ORDER BY ncm.person_id, ncm.course_id, ncm.start_date

-------------------------------------------------
--Build performance index on #NewCourseMembersSorted
CREATE INDEX NewCourseMembersSorted_person_id ON  #NewCourseMembersSorted (rec_id,person_id,course_id)

-------------------------------------------------
--Amalgamate #NewCourseMembersSorted : have to run through this in order, which isn't very efficient.
DECLARE @recId int
DECLARE @PrevPerson int
DECLARE @PrevCourse int
DECLARE @PrevStartDate smalldatetime
DECLARE @PrevEndDate smalldatetime
DECLARE @EffectiveStartDate smalldatetime
DECLARE @FirstEndDate smalldatetime
DECLARE @MaxRecId int
SELECT @MaxRecId = MAX(rec_id) FROM #NewCourseMembersSorted

SELECT @recId = 1 
WHILE @recId <= @MaxRecId
BEGIN

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF (@Diagnosis = 1)
BEGIN
   IF DATEPART(MONTH,@PrevStartDate) IN (5,6,7) AND @AcadYearInNormalRange = 1
   BEGIN
      INSERT INTO @CourseDiagnosis        
      SELECT DISTINCT
        [course_id]    = ncm.course_id
       ,[person_id]    = ncm.person_id
       ,[Course]       = sbg.short_name          
       ,[Student]      = sp.surname+', '+sp.forename     
       ,[Reason]       = 'Early Start Logic: ('+CONVERT(varchar(12),@PrevStartDate,106)+'-'+CONVERT(varchar(12),@PrevEndDate,106)+') merged with '+
                                            '('+CONVERT(varchar(12),ncm.start_date,106)+'-'+CONVERT(varchar(12),ncm.end_date,106)+') - continuity tolerance ('+tol.cont_tolerance_code+')'       
      FROM #NewCourseMembersSorted ncm
      JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
      JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id 
      JOIN #Tolerances tol ON ncm.course_id = tol.course_id
      WHERE ncm.rec_id = @recId                                                                         --just consider the single record
        AND ncm.course_id = @PrevCourse AND ncm.person_id = @PrevPerson                                 --its the same person/course as the previous record
        AND ncm.start_date > @PrevStartDate                                                             --otherwise no point merging
     AND DATEDIFF(DAY,@FirstEndDate,ncm.end_date) < tol.maxlength                                       --ensure the new record wouldn't be longer than the maximum length
     AND DATEDIFF(DAY,@PrevStartDate,ncm.end_date) >= tol.maxlength                                     --ensure this requires the special Early Start rule
     AND (
         (DATEADD(MONTH,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'M')  --the gap between previous end date and start of this one is less than number of Months
      OR (DATEADD(WEEK,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'W')   --the gap between previous end date and start of this one is less than number of Weeks
      OR (DATEADD(DAY,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'D')    --the gap between previous end date and start of this one is less than number of Days
         ) 
   END
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS


--normally the @EffectiveStartDate is the start date of the previous record (the earlier one that we are merging the current one into)
--however there is a rule that says that if this lies in May/June/July then the @EffectiveStartDate should be taken as being the first END date of the sequence.
--this rule only applies if the AY boundary lies (as is usual) in August or September
SELECT @EffectiveStartDate = CASE WHEN DATEPART(MONTH,@PrevStartDate) IN (5,6,7) AND @AcadYearInNormalRange = 1 THEN @FirstEndDate ELSE @PrevStartDate END

update csa -- PRB11085
set csa.completion_status_id = null
from sims.curr_p16_student_aim csa
where csa.course_id = @PrevCourse and csa.person_id = @PrevPerson and  csa.start_date = @PrevStartDate 
and exists(select 1 from #NewCourseMembersSorted ncm
			JOIN #Tolerances tol ON ncm.course_id = tol.course_id
			WHERE ncm.rec_id = @recId                                                                         --just consider the single record
			AND ncm.course_id = csa.course_id and ncm.person_id = csa.person_id  
			AND ncm.start_date > @PrevStartDate                                                             --otherwise no point merging
			AND DATEDIFF(DAY,@EffectiveStartDate,ncm.end_date) < tol.maxlength                              --ensure the new record wouldn't be longer than the maximum length
			AND (
				(DATEADD(MONTH,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'M')  --the gap between previous end date and start of this one is less than number of Months
				OR (DATEADD(WEEK,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'W')   --the gap between previous end date and start of this one is less than number of Weeks
				OR (DATEADD(DAY,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'D')    --the gap between previous end date and start of this one is less than number of Days
				)
			)

--amalgamate corresponding course memberships if intervening gap is less than the continuity tolerance and resulting length would not exceed maximum allowed for the course
--@PrevStartDate, @PrevEndDate set as null beginning and set the values at the end
   UPDATE ncm set start_date = @PrevStartDate
   FROM #NewCourseMembersSorted ncm
   JOIN #Tolerances tol ON ncm.course_id = tol.course_id
   WHERE ncm.rec_id = @recId                                                                         --just consider the single record
     AND ncm.course_id = @PrevCourse AND ncm.person_id = @PrevPerson                                 --its the same person/course as the previous record
     AND ncm.start_date > @PrevStartDate                                                             --otherwise no point merging
     AND DATEDIFF(DAY,@EffectiveStartDate,ncm.end_date) < tol.maxlength                              --ensure the new record wouldn't be longer than the maximum length
     AND (
         (DATEADD(MONTH,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'M')  --the gap between previous end date and start of this one is less than number of Months
      OR (DATEADD(WEEK,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'W')   --the gap between previous end date and start of this one is less than number of Weeks
      OR (DATEADD(DAY,tol.contMagnitude,@PrevEndDate+'00:01') > ncm.start_date AND tol.contTolType = 'D')    --the gap between previous end date and start of this one is less than number of Days
         )
   
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF (@Diagnosis = 1)
BEGIN
   IF DATEPART(MONTH,@PrevStartDate) IN (5,6,7) AND @AcadYearInNormalRange = 1
   BEGIN
      INSERT INTO @CourseDiagnosis        
      SELECT DISTINCT
        [course_id]    = ncm.course_id
       ,[person_id]    = ncm.person_id
       ,[Course]       = sbg.short_name          
       ,[Student]      = sp.surname+', '+sp.forename     
       ,[Reason]       = 'Early Start Logic: ('+CONVERT(varchar(12),@PrevStartDate,106)+'-'+CONVERT(varchar(12),@PrevEndDate,106)+') merged with '+
                                            '('+CONVERT(varchar(12),ncm.start_date,106)+'-'+CONVERT(varchar(12),ncm.end_date,106)+') - not delivered in the interim'       
      FROM #NewCourseMembersSorted ncm
      JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
      JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id 
      JOIN #Tolerances tol ON ncm.course_id = tol.course_id
   LEFT JOIN sims.curr_component_delivery ccd 
        JOIN sims.curr_destinationgroup cdg ON ccd.class_group_id = cdg.base_group_id
     ON ncm.course_id = ccd.course_group_id 
    AND ccd.start_date < ncm.start_date AND ccd.end_date > @PrevEndDate --course-class link intersects gap [@PrevEndDate <----> ncm.start_date]
    AND cdg.start_date < ncm.start_date AND cdg.end_date > @PrevEndDate --class availability of class intersects gap   
    AND cdg.start_date < ccd.end_date AND ccd.start_date < cdg.end_date --course-class link intersects class availability
   WHERE ncm.rec_id = @recId                                            --just consider the single record
     AND ncm.course_id = @PrevCourse AND ncm.person_id = @PrevPerson    --its the same person/course as the previous record
     AND ccd.comp_delivery_id IS NULL                                   --its not delivered  
     AND ncm.start_date > @PrevStartDate                                --otherwise no point merging
     AND DATEDIFF(DAY,@FirstEndDate,ncm.end_date) < tol.maxlength       --ensure the new record wouldn't be longer than the maximum length
     AND DATEDIFF(DAY,@PrevStartDate,ncm.end_date) >= tol.maxlength     --ensure this requires the special Early Start rule
   END
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS   
                     
 --amalgamate corresponding course memberships if course is not delivered in the intervening gap - and resulting length would not exceed maximum allowed for the course
   UPDATE ncm SET start_date = @PrevStartDate
   FROM #NewCourseMembersSorted ncm
   JOIN #Tolerances tol ON ncm.course_id = tol.course_id
   LEFT JOIN sims.curr_component_delivery ccd 
        JOIN sims.curr_destinationgroup cdg ON ccd.class_group_id = cdg.base_group_id
     ON ncm.course_id = ccd.course_group_id 
    AND ccd.start_date < ncm.start_date AND ccd.end_date > @PrevEndDate --course-class link intersects gap [@PrevEndDate <----> ncm.start_date]
    AND cdg.start_date < ncm.start_date AND cdg.end_date > @PrevEndDate --class availability of class intersects gap   
    AND cdg.start_date < ccd.end_date AND ccd.start_date < cdg.end_date --course-class link intersects class availability
   WHERE ncm.rec_id = @recId                                            --just consider the single record
     AND ncm.course_id = @PrevCourse AND ncm.person_id = @PrevPerson    --its the same person/course as the previous record
     AND ccd.comp_delivery_id IS NULL                                   --its not delivered  
     AND ncm.start_date > @PrevStartDate                                --otherwise no point merging
     AND DATEDIFF(DAY,@EffectiveStartDate,ncm.end_date) < tol.maxlength --ensure the new record wouldn't be longer than the maximum length

   --now reset variables for next time round and increment the loop counter
  
   SELECT @FirstEndDate = end_Date -- if its NOT the same person/course as last time round, reset the @FirstEndDate variable.
   FROM #NewCourseMembersSorted WHERE rec_id = @recId AND (@PrevCourse <> course_id OR @PrevPerson <> person_id) 

   SELECT @PrevStartDate = start_date, @PrevEndDate = end_Date, @PrevCourse = course_id, @PrevPerson = person_id 
   FROM #NewCourseMembersSorted WHERE rec_id = @recId
   SELECT @recId = @recId + 1
END

--delete redundant course memberships where [M1] lies inside [M2], or where [M1] and [M2] are identical but [M1] has larger rec_id.  
   DELETE ncm1 
   FROM #NewCourseMembersSorted ncm1
   JOIN #NewCourseMembersSorted ncm2 ON ncm1.person_id = ncm2.person_id AND ncm1.course_id = ncm2.course_id
   WHERE (ncm1.start_date > ncm2.start_date AND ncm1.end_date <= ncm2.end_date)
      OR (ncm1.start_date >= ncm2.start_date AND ncm1.end_date < ncm2.end_date) 
      OR (ncm1.start_date = ncm2.start_date AND ncm1.end_date = ncm2.end_date AND ncm1.rec_id > ncm2.rec_id)

-------------------------------------------------

--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG6]     = 'CRSMEM Merged'
        ,[StudentId]  = ncm.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ncm.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),ncm.start_date,106)
        ,[EndDate]    = CONVERT(varchar(12),ncm.end_date,106)
        ,[Length]     = DATEDIFF(DAY,ncm.start_date,ncm.end_date)
   FROM #NewCourseMembersSorted ncm 
   JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id
   JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
  WHERE (ncm.person_id = @DebugStudentId OR @DebugStudentId = 0) 
    AND (ncm.course_id = @DebugCourseId  OR @DebugCourseId = 0)
    
  SELECT @TotalRecordsAfterFullMerge = COUNT(*) FROM #NewCourseMembersSorted  
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG- 

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF (@Diagnosis = 1)
BEGIN
   INSERT INTO @CourseDiagnosis        
   SELECT DISTINCT
        [course_id]    = ncm1.course_id
       ,[person_id]    = ncm1.person_id
       ,[Course]       = sbg.short_name          
       ,[Student]      = sp.surname+', '+sp.forename     
       ,[Reason]       = 'Not Merged ('+CONVERT(varchar(12),ncm1.end_date,106)+'<-->'+CONVERT(varchar(12),ncm2.start_date,106)+') : '+ 
          CASE 
             WHEN   DATEDIFF(DAY,ncm1.start_date,ncm2.end_date) > tol.maxlength THEN 'Would have exceeded max length ('+ltrim(str(tol.maxlength))+' days)'
             ELSE CASE 
					WHEN   (DATEADD(MONTH,tol.contMagnitude,ncm1.end_date) < ncm2.start_date AND tol.contTolType = 'M')  
					   OR  (DATEADD(WEEK,tol.contMagnitude,ncm1.end_date) < ncm2.start_date AND tol.contTolType = 'W')  
					   OR  (DATEADD(DAY,tol.contMagnitude,ncm1.end_date) < ncm2.start_date AND tol.contTolType = 'D')  
					   THEN 'greater than continuity tolerance ('+tol.cont_tolerance_code+')'
					ELSE 'Within tolerance ('+tol.cont_tolerance_code+'):SHOULD HAVE MERGED!'
				  END + ISNULL( (SELECT TOP 1 ' and provided by '+cls.short_name+
					  ' between '+CONVERT(varchar(12),sims.pub_fnd_maxdate(ccp.start_date,ccd.start_date),106) +' and '+ 
								  CONVERT(varchar(12),sims.pub_fnd_maxdate(ccp.end_date,ccd.end_date),106)
						FROM sims.sims_base_group cls
						JOIN sims.curr_component_delivery ccd ON cls.base_group_id = ccd.class_group_id AND ccd.course_group_id = ncm1.course_id
						 AND ccd.start_date < ncm2.start_date AND ccd.end_date > ncm1.end_date
						JOIN sims.curr_class_period ccp ON ccp.base_group_id = cls.base_group_id
						 AND ccp.start_date < ncm2.start_date AND ccp.end_date > ncm1.end_date
						WHERE ccp.start_date < ccd.end_date AND ccd.start_date < ccp.end_date 
						),' but not provided:SHOULD HAVE MERGED!')
		  END
   FROM #NewCourseMembersSorted ncm1
   JOIN #NewCourseMembersSorted ncm2 ON ncm1.course_id = ncm2.course_id AND ncm1.person_id = ncm2.person_id AND ncm1.end_date < ncm2.start_date  
   LEFT JOIN #NewCourseMembersSorted ncmx ON ncm1.course_id = ncmx.course_id AND ncm1.person_id = ncmx.person_id AND ncm1.end_date < ncmx.start_date AND ncmx.end_date < ncm2.start_date   
   JOIN sims.sims_person sp ON ncm1.person_id = sp.person_id
   JOIN sims.sims_base_group sbg ON ncm1.course_id = sbg.base_group_id 
   JOIN #Tolerances tol ON ncm1.course_id = tol.course_id  
   WHERE ncmx.course_id IS NULL
END 

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS

--==UPDATEMEMBERSHIP==========================================================================
--Calculate Bounds of membership records in #NewCourseMembers for each student/course - but always include AY
INSERT INTO #MembershipBounds (person_id, course_id, minDate, maxDate)
SELECT ncm.person_id
      ,ncm.course_id
      ,sims.pub_fnd_mindate(MIN(start_date),@ThisAYStart)
      ,sims.pub_fnd_maxdate(MAX(end_date),@ThisAYEnd)
 FROM #NewCourseMembersSorted ncm
 GROUP BY ncm.person_id,ncm.course_id

 -- Extend max date for courses over one year
 UPDATE mb
    SET maxDate = DATEADD(day, tol.maxlength - 365, maxDate)
   FROM #MembershipBounds mb
   JOIN #Tolerances tol 
     ON mb.course_id = tol.course_id  
  WHERE tol.maxlength > 365
 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG7]     = 'CRSMEM Bounds'
        ,[StudentId]  = bnds.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = bnds.course_id
        ,[Course]     = sbg.short_name
        ,[MinDate]  = CONVERT(varchar(12),bnds.minDate,106)
        ,[MaxDate]    = CONVERT(varchar(12),bnds.maxDate,106)
   FROM #MembershipBounds bnds 
   JOIN sims.sims_base_group sbg ON bnds.course_id = sbg.base_group_id
   JOIN sims.sims_person sp ON bnds.person_id = sp.person_id
  WHERE (bnds.person_id = @DebugStudentId OR @DebugStudentId = 0) 
    AND (bnds.course_id = @DebugCourseId  OR @DebugCourseId = 0)
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG- 

-- Delete the course membership records in sims.sims_member that are no longer wanted (i.e. in are not found in #NewCourseMembers)
-- check its for a course that has auto_update_membership; check its for one of the specified students
-- check the start date is within calculated bounds (OR within AY if no bounds found i.e. no new memberships for the student/course)


--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG8]     = 'SM Deleted'
        ,[StudentId]  = sm.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = sm.base_group_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),sm.start_date,106)
        ,[EndDate]    = CONVERT(varchar(12),sm.end_date,106)
	FROM sims.sims_member sm
	JOIN sims.sims_base_group sbg ON sm.base_group_id = sbg.base_group_id
	JOIN sims.sims_person sp ON sm.person_id = sp.person_id
	JOIN sims.curr_course cc ON sm.base_group_id = cc.base_group_id AND cc.auto_update_membership = 1
	LEFT JOIN #Students s ON s.PersonId = sm.person_id 
	LEFT JOIN #NewCourseMembersSorted ncm
	ON sm.base_group_id = ncm.course_id and sm.person_id=ncm.person_id and sm.start_date=ncm.start_date and sm.end_date = ncm.end_date
	LEFT JOIN #MembershipBounds bnds
	ON sm.base_group_id = bnds.course_id and sm.person_id=bnds.person_id

	LEFT JOIN sims.curr_course_exception cce 
	ON sm.person_id = cce.person_id AND sm.base_group_id = cce.course_id 
	AND cce.academic_year_id = @AcadYear_event_instance_id
	
	LEFT JOIN #UnboundedCourseMembers ucm
	ON sm.person_id = ucm.PersonId AND sm.base_group_id = ucm.CourseId
	
	WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
	  AND sm.end_date >= ISNULL(bnds.minDate,@ThisAYStart) -- delete any existing membership intersecting the bounds
	  AND sm.start_date <= ISNULL(bnds.maxDate,@ThisAYEnd)
	  AND ncm.rec_id IS NULL
	  AND (sm.person_id = @DebugStudentId OR @DebugStudentId = 0) 
	  AND (sm.base_group_id = @DebugCourseId  OR @DebugCourseId = 0)
	  AND cce.person_id IS NULL
	  AND ucm.PersonId IS NULL
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-

DELETE sims.sims_member 
FROM sims.sims_member sm
JOIN sims.curr_course cc ON sm.base_group_id = cc.base_group_id AND cc.auto_update_membership = 1
LEFT JOIN #Students s ON s.PersonId = sm.person_id 
LEFT JOIN #NewCourseMembersSorted ncm -- to check the exact same membership doesn't already exist
  ON sm.base_group_id = ncm.course_id and sm.person_id=ncm.person_id and sm.start_date=ncm.start_date and sm.end_date = ncm.end_date
LEFT JOIN #MembershipBounds bnds
  ON sm.base_group_id = bnds.course_id and sm.person_id=bnds.person_id
LEFT JOIN sims.curr_course_exception cce -- not exception
  ON sm.person_id = cce.person_id AND sm.base_group_id = cce.course_id 
  AND cce.academic_year_id = @AcadYear_event_instance_id
LEFT JOIN #UnboundedCourseMembers ucm -- not a null end date
	ON sm.person_id = ucm.PersonId AND sm.base_group_id = ucm.CourseId
WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
  AND sm.end_date >= ISNULL(bnds.minDate,@ThisAYStart) -- delete any existing membership intersecting the bounds
  AND sm.start_date <= ISNULL(bnds.maxDate,@ThisAYEnd)
  AND ncm.rec_id IS NULL
  AND cce.person_id IS NULL
  AND ucm.PersonId IS NULL

-------------------------------------------------
-- Create the course membership records in sims.sims_member that don't already exist

--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG9]     = 'SM Created'
        ,[StudentId]  = ncm.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ncm.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = CONVERT(varchar(12),ncm.start_date,106)
        ,[EndDate]    = CONVERT(varchar(12),ncm.end_date,106)
	FROM #NewCourseMembersSorted ncm
	JOIN sims.sims_base_group sbg ON ncm.course_id = sbg.base_group_id
	JOIN sims.sims_person sp ON ncm.person_id = sp.person_id
	LEFT JOIN sims.sims_member sm
	ON sm.base_group_id = ncm.course_id and sm.person_id=ncm.person_id and sm.start_date=ncm.start_date and sm.end_date = ncm.end_date
	

	LEFT JOIN sims.curr_course_exception cce 
	ON ncm.person_id = cce.person_id AND ncm.course_id = cce.course_id 
	AND cce.academic_year_id = @AcadYear_event_instance_id
	
    LEFT JOIN #UnboundedCourseMembers ucm
    ON ncm.person_id = ucm.PersonId AND ncm.course_id = ucm.CourseId 

	WHERE sm.person_id IS NULL    
	  AND cce.person_id IS NULL   
	  AND ucm.PersonId IS NULL   
	  AND (ncm.person_id = @DebugStudentId OR @DebugStudentId = 0) 
	  AND (ncm.course_id = @DebugCourseId  OR @DebugCourseId = 0)   
  
END
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-


INSERT INTO sims.sims_member (person_id,base_group_id,start_date,end_date)
SELECT ncm.person_id, ncm.course_id, ncm.start_date, ncm.end_date
FROM #NewCourseMembersSorted ncm
LEFT JOIN sims.sims_member sm
  ON sm.base_group_id = ncm.course_id and sm.person_id=ncm.person_id and sm.start_date=ncm.start_date and sm.end_date = ncm.end_date
	
LEFT JOIN sims.curr_course_exception cce 
  ON ncm.person_id = cce.person_id AND ncm.course_id = cce.course_id 
 AND cce.academic_year_id = @AcadYear_event_instance_id
LEFT JOIN #UnboundedCourseMembers ucm
  ON ncm.person_id = ucm.PersonId AND ncm.course_id = ucm.CourseId 
WHERE sm.person_id IS NULL
 AND cce.person_id IS NULL
 AND ucm.PersonId IS NULL 
 
 
--==ACTIVEFLAG======================================================================
--Update Group active status
if @PersonID = 0
begin
    -- all group types which are in sets (including those with no end date) - this previously affected
    -- the active flags of class and course memberships.

    declare @from_date smalldatetime, @to_date smalldatetime
    select @from_date = getdate() - 14 -- 2 weeks ago
    select @to_date = getdate() + 14 -- in 2 weeks time
    exec @retcode = sims.curr_pia_UpdateGroupActiveStatus @from_date, @to_date
    if @retcode > 3 
       return @retcode       
end 

--==LEARNINGAIMS=====================================================================================
-- Update Learning Aims (curr_p16_student_aim)
-- Note: Ged April 2015: This code seems to assume learning aims have independent membership dates from course memberships
--                       but the curr_p16_student_aim table is really an extension to the sims_member table.
--                       It has to use the compound key person_id, course_id, start_date because that is the only primary key on the sims_member table.
-------------------------------------------------
 --a) Clean before we start - Delete Learning Aim records which intersect a LATER Learning Aim record (shouldn't be any!)
  --  |           [****AIM*****]
  --  |                    [****AIM*****]
  
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG10]     = 'AIM:Clean'
        ,[StudentId]  = psa1.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = psa1.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = psa1.start_date
        ,[PlannedEnd] = psa1.planned_end
        ,[OtherStart] = psa2.start_date
        ,[OtherEnd]   = psa2.planned_end        
  FROM sims.curr_p16_student_aim psa1
  JOIN sims.sims_base_group sbg ON psa1.course_id = sbg.base_group_id
  JOIN sims.sims_person sp ON psa1.person_id = sp.person_id
  JOIN sims.curr_p16_student_aim psa2 ON psa1.person_id = psa2.person_id AND psa1.course_id = psa2.course_id 
        AND psa1.start_date < psa2.planned_end AND psa2.start_date < psa1.planned_end
  WHERE psa1.start_date < psa2.start_date 
  	AND (psa1.person_id = @DebugStudentId OR @DebugStudentId = 0) 
	AND (psa1.course_id = @DebugCourseId  OR @DebugCourseId = 0)
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-    
    
  DELETE psa1  
  FROM sims.curr_p16_student_aim psa1 
  JOIN sims.curr_p16_student_aim psa2 ON psa1.person_id = psa2.person_id AND psa1.course_id = psa2.course_id 
        AND psa1.start_date < psa2.planned_end AND psa2.start_date < psa1.planned_end
  WHERE psa1.start_date < psa2.start_date 
  
  --So now Learning Aim records have distinct non-overlapping date ranges.
  -------------------------------------------------

  --b) Delete Learning Aim records which start during AY if there is no corresponding course membership during the period of the Learning Aim [Start<->Planned End]
  --  |           [****AIM*****]
  --  | {--MEM--}                      {--MEM--}
  
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG11]     = 'AIM:Del:NoCrsMem'
        ,[StudentId]  = psa.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = psa.course_id
        ,[Course]     = sbg.short_name
        ,[StartDate]  = psa.start_date
        ,[PlannedEnd] = psa.planned_end      
  FROM sims.curr_p16_student_aim psa
  JOIN sims.sims_base_group sbg ON psa.course_id = sbg.base_group_id
  JOIN sims.sims_person sp ON psa.person_id = sp.person_id
  LEFT JOIN sims.sims_member sm ON psa.person_id = sm.person_id AND psa.course_id = sm.base_group_id 
        AND sm.start_date < psa.planned_end AND psa.start_date < sm.end_date
  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
    AND sm.person_id IS NULL
    AND psa.start_date BETWEEN @ThisAYStart AND @ThisAYEnd
  	AND (psa.person_id = @DebugStudentId OR @DebugStudentId = 0) 
	AND (psa.course_id = @DebugCourseId  OR @DebugCourseId = 0)    
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-  
  
  DELETE psa  
  FROM sims.curr_p16_student_aim psa 
  LEFT JOIN sims.sims_member sm ON psa.person_id = sm.person_id AND psa.course_id = sm.base_group_id 
        AND sm.start_date < psa.planned_end AND psa.start_date < sm.end_date
  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL) 
    AND sm.person_id IS NULL
    AND psa.start_date BETWEEN @ThisAYStart AND @ThisAYEnd
  
  --So now every Learning Aim record MUST intersect at least one course membership
  -------------------------------------------------

-- TFS72460 Don't delete 
--  --c) Delete Learning Aim records regardless of start date if the planned end date lies INSIDE a corresponding course membership
--  --  |           [****AIM*****]
--  --  | {--MEM--}         {-----MEM-------}  
  
----DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
--IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
--BEGIN
--  SELECT [DEBUG12]     = 'AIM:Del:PED in CRSMEM'
--        ,[StudentId]  = psa.person_id
--        ,[Student]    = sp.surname+', '+sp.forename
--        ,[CourseId]   = psa.course_id
--        ,[Course]     = sbg.short_name
--        ,[StartDate]  = psa.start_date
--        ,[PlannedEnd] = psa.planned_end    
--        ,[CrsStart]   = sm.start_date
--        ,[CrsEnd]     = sm.end_date  
--  FROM sims.curr_p16_student_aim psa
--  JOIN sims.sims_base_group sbg ON psa.course_id = sbg.base_group_id
--  JOIN sims.sims_person sp ON psa.person_id = sp.person_id
--  JOIN sims.sims_member sm ON psa.person_id = sm.person_id AND psa.course_id = sm.base_group_id 
--       AND sm.start_date <= psa.planned_end AND sm.end_date >= psa.planned_end
--  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
--  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
--  	AND (psa.person_id = @DebugStudentId OR @DebugStudentId = 0) 
--	AND (psa.course_id = @DebugCourseId  OR @DebugCourseId = 0)    
--END 
----DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-  
--  DELETE psa  
--  FROM sims.curr_p16_student_aim psa 
--  JOIN sims.sims_member sm ON psa.person_id = sm.person_id AND psa.course_id = sm.base_group_id 
--       AND sm.start_date <= psa.planned_end AND sm.end_date >= psa.planned_end 
--       AND sm.start_date = psa.start_date
--  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
--  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
  
--  --So now every Learning Aim record MUST intersect at least one course membership, but does not END during one.
--  --i.e. either the Aim record encloses at least one membership, or there is a membership that encloses the Learning Aim's start date.
--  -------------------------------------------------

  --d) Update start date of Learning Aim record to start date of LATEST corresponding course membership
  --where the membership end date lies BETWEEN the Learning Aim start date and planned end date.
  --(NB there MUST be at least one, with an end date STRICTLY BEFORE the planned end date - else would have been deleted in the previous steps!)
  
  --  |           [****AIM*************************]             [****AIM***************]
  --  | {--MEM---------}      {-----MEM-------}           {--MEM---------}
  --  |                       ^                           ^
  
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-   
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
  SELECT [DEBUG13]    = 'AIM:Update:PED in CRSMEM'
        ,[StudentId]  = psa.person_id
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = psa.course_id
        ,[Course]     = sbg.short_name
        ,[OldStart]   = psa.start_date
        ,[PlannedEnd] = psa.planned_end    
        ,[NewCrsStart]= sm1.start_date
        ,[CrsEnd]     = sm1.end_date  
  FROM sims.curr_p16_student_aim psa
  JOIN sims.sims_base_group sbg ON psa.course_id = sbg.base_group_id
  JOIN sims.sims_person sp ON psa.person_id = sp.person_id
  JOIN sims.sims_member sm1 ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id 
       AND sm1.end_date > psa.start_date AND sm1.end_date < psa.planned_end
  LEFT JOIN sims.sims_member sm2 ON psa.person_id = sm2.person_id AND psa.course_id = sm2.base_group_id 
       AND sm2.end_date > psa.start_date AND sm2.end_date < psa.planned_end
       AND sm2.start_date > sm1.start_date
  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
    AND sm2.person_id IS NULL
    AND psa.start_date <> sm1.start_date
  	AND (psa.person_id = @DebugStudentId OR @DebugStudentId = 0) 
	AND (psa.course_id = @DebugCourseId  OR @DebugCourseId = 0)    
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-  
  
  UPDATE psa SET start_date = sm1.start_date 
  FROM sims.curr_p16_student_aim psa 
  JOIN sims.sims_member sm1 ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id 
       AND sm1.end_date > psa.start_date AND sm1.end_date < psa.planned_end
  LEFT JOIN sims.sims_member sm2 ON psa.person_id = sm2.person_id AND psa.course_id = sm2.base_group_id 
       AND sm2.end_date > psa.start_date AND sm2.end_date < psa.planned_end
       AND sm2.start_date > sm1.start_date
  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
    AND sm2.person_id IS NULL
    AND psa.start_date <> sm1.start_date
  
  --NB If start date moved earlier, the corresponding course membership cannot be linked to ANOTHER Learning Aim
  -------------------------------------------------
  
-- TFS 74260 DON'T DELETE P16 RECORDS 
--  --e) Delete Learning Aim record if its corresponding Actual End Date is within the Completion tolerance of the Planned End date

--IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
--BEGIN
--  SELECT [DEBUG14]    = 'AIM:Del:Within Tolerance'
--        ,[StudentId]  = sm.person_id
--        ,[Student]    = sp.surname+', '+sp.forename
--        ,[CourseId]   = sm.base_group_id
--        ,[Course]     = sbg.short_name
--        ,[StartDate]  = sm.start_date
--        ,[EndDate]    = sm.end_date
--        ,[PlannedEnd] = psa.planned_end
--        ,[Tolerance]  = tol.comp_tolerance_code
--  FROM sims.curr_p16_student_aim psa 
--  JOIN sims.sims_base_group sbg ON psa.course_id = sbg.base_group_id
--  JOIN sims.sims_person sp ON psa.person_id = sp.person_id
--  JOIN #Tolerances tol ON psa.course_id = tol.course_id
--  JOIN sims.sims_member sm ON psa.person_id = sm.person_id AND psa.course_id = sm.base_group_id AND psa.start_date = sm.start_date
--   AND (   --REQUIREMENT: Actual End Date must not be within the Completion Tolerance of the original end date!
--         (DATEADD(MONTH,tol.compMagnitude,sm.end_date) >=  psa.planned_end AND tol.compTolType = 'M')  --the gap between actual end date and planned end date is not less than number of Months
--      OR (DATEADD(WEEK,tol.compMagnitude,sm.end_date)  >=  psa.planned_end AND tol.compTolType = 'W') --the gap between actual end date and planned end date is not less than number of Weeks
--      OR (DATEADD(DAY,tol.compMagnitude,sm.end_date)   >=  psa.planned_end AND tol.compTolType = 'D') --the gap between actual end date and planned end date is not less than number of Days
--        )
--  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
--  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
--END


--  DELETE psa  
--  FROM sims.curr_p16_student_aim psa 
--  JOIN #Tolerances tol ON psa.course_id = tol.course_id
--  JOIN sims.sims_member sm ON psa.person_id = sm.person_id AND psa.course_id = sm.base_group_id AND psa.start_date = sm.start_date
--   AND (   --REQUIREMENT: Actual End Date must not be within the Completion Tolerance of the original end date!
--         (DATEADD(MONTH,tol.compMagnitude,sm.end_date) >= psa.planned_end AND tol.compTolType = 'M')  --the gap between actual end date and planned end date is not less than number of Months
--      OR (DATEADD(WEEK,tol.compMagnitude,sm.end_date)  >=  psa.planned_end AND tol.compTolType = 'W') --the gap between actual end date and planned end date is not less than number of Weeks
--      OR (DATEADD(DAY,tol.compMagnitude,sm.end_date)   >=  psa.planned_end AND tol.compTolType = 'D') --the gap between actual end date and planned end date is not less than number of Days
--        )
--  LEFT JOIN #Students s ON s.PersonId = psa.person_id 
--  WHERE (@PersonID = 0 OR s.PersonId IS NOT NULL)
--  AND psa.planned_end BETWEEN @ThisAYStart AND @ThisAYEnd
  


  -------------------------------------------------

  --f) Create new Learning aim records: for each old course membership, consider all intersecting corresponding New memberships
  --if the latest one ends BEFORE the old course membership, create new record keyed on this latest membership (with planned end date matching old course membership)
  --(UNLESS there is already a Learning Aim record for this student/course that would intersect the new one!)
  --NB This works off #OldCourseMembers which is already filtereds on #Students - so there is no need to filter again
  
  --REQUIREMENT: DO NOT CREATE LEARNING AIM RECORD IF NEW ACTUAL END DATE IS IN THE FUTURE!
  --REQUIREMENT: DO NOT CREATE LEARNING AIM RECORD IF NEW ACTUAL END DATE IS WITHIN the COMPLETION TOLERANCE OF THE ORIGINAL END DATE!
  
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF (@Diagnosis = 1)
BEGIN
 INSERT INTO @CourseDiagnosis
 SELECT  [CourseId]   = ocm.CourseId
        ,[StudentId]  = ocm.PersonId
        ,[Course]     = sbg.short_name
        ,[Student]    = sp.surname+', '+sp.forename
        ,[Reason]     = 'No Learning Aim (' + CONVERT(varchar(12),sm1.start_date,106)+') : '+
                CASE 
                    WHEN sm2.person_id IS NOT NULL THEN 'Later Course membership (from '+ CONVERT(varchar(12),sm2.start_date,106)+')'    
                    WHEN psa.person_id IS NOT NULL THEN 'Existing Learning Aim (for start date '+CONVERT(varchar(12),psa.start_date,106)+')'    
                    WHEN sm1.end_date >= @CurrentDate THEN 'Planned End Date is '+CONVERT(varchar(12),ocm.EndDate,106)+', but new Actual End date in Future ('+ CONVERT(varchar(12),sm1.end_date,106)+')' 
                    WHEN (DATEADD(MONTH,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'M') 
                      OR (DATEADD(WEEK,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'W') 
                      OR (DATEADD(DAY,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'D') 
                         THEN CONVERT(varchar(12),sm1.end_date,106)+' is within completion tolerance ('+tol.comp_tolerance_code+') of '+CONVERT(varchar(12),ocm.EndDate,106) 
                END

  FROM #OldCourseMembers ocm
  JOIN #Tolerances tol ON ocm.CourseId = tol.course_id
  JOIN sims.sims_base_group sbg ON ocm.CourseId = sbg.base_group_id
  JOIN sims.sims_person sp ON ocm.PersonId = sp.person_id  
  JOIN sims.sims_member sm1 ON ocm.PersonId = sm1.person_id AND ocm.CourseId = sm1.base_group_id 
       AND ocm.StartDate < sm1.end_date AND sm1.start_date < ocm.EndDate
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm2 ON ocm.PersonId = sm2.person_id AND ocm.CourseId = sm2.base_group_id
       AND ocm.StartDate < sm2.end_date AND sm2.start_date < ocm.EndDate
       AND sm2.start_date > sm1.start_date
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm3 ON ocm.PersonId = sm3.person_id AND ocm.CourseId = sm3.base_group_id
       AND ocm.StartDate < sm3.end_date AND sm3.start_date < ocm.EndDate
       AND sm3.start_date > sm1.start_date AND sm3.start_date < sm2.start_date      
  --make sure there isn't an existing Learning Aim that would intersect this new one.
  LEFT JOIN sims.curr_p16_student_aim psa ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id   
       AND sm1.start_date < psa.planned_end AND psa.start_date < sm1.end_date   
  WHERE (
        (sm2.person_id IS NOT NULL)
    OR  (psa.person_id IS NOT NULL)   
    OR  (sm1.end_date >= @CurrentDate) ---REQUIREMENT: Actual End Date must be earlier than the start of "tomorrow"

    OR  (DATEADD(MONTH,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'M')  --the gap between actual end date and planned end date is not less than number of Months
    OR  (DATEADD(WEEK,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'W')   --the gap between actual end date and planned end date is not less than number of Weeks
    OR  (DATEADD(DAY,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'D')    --the gap between actual end date and planned end date is not less than number of Days
     
        )
    AND sm1.end_date < ocm.EndDate 
    AND sm3.base_group_id IS NULL      
    
    
 INSERT INTO @CourseDiagnosis
 SELECT  [CourseId]   = ocm.CourseId
        ,[StudentId]  = ocm.PersonId
        ,[Course]     = sbg.short_name
        ,[Student]    = sp.surname+', '+sp.forename
        ,[Reason]     = 'No Learning Aim (' + CONVERT(varchar(12),sm1.start_date,106)+') : '+
                        'NCYear below min YTI for Completion management'+
                CASE 
                    WHEN sm2.person_id IS NOT NULL THEN ' and Later Course membership (from '+ CONVERT(varchar(12),sm2.start_date,106)+')'    
                    WHEN psa.person_id IS NOT NULL THEN ' and Existing Learning Aim (for start date '+CONVERT(varchar(12),psa.start_date,106)+')'    
                    WHEN sm1.end_date >= @CurrentDate THEN ' and Planned End Date is '+CONVERT(varchar(12),ocm.EndDate,106)+', but new Actual End date in Future ('+ CONVERT(varchar(12),sm1.end_date,106)+')' 
                    WHEN (DATEADD(MONTH,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'M') 
                      OR (DATEADD(WEEK,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'W') 
                      OR (DATEADD(DAY,tol.compMagnitude,sm1.end_date) >= ocm.EndDate AND tol.compTolType = 'D') 
                         THEN ' and '+CONVERT(varchar(12),sm1.end_date,106)+' is within completion tolerance ('+tol.comp_tolerance_code+') of '+CONVERT(varchar(12),ocm.EndDate,106) 
                    ELSE ' (but thats the only reason!)'     
                END

  FROM #ExcludedOldCourseMembers ocm
  JOIN #Tolerances tol ON ocm.CourseId = tol.course_id
  JOIN sims.sims_base_group sbg ON ocm.CourseId = sbg.base_group_id
  JOIN sims.sims_person sp ON ocm.PersonId = sp.person_id  
  JOIN sims.sims_member sm1 ON ocm.PersonId = sm1.person_id AND ocm.CourseId = sm1.base_group_id 
       AND ocm.StartDate < sm1.end_date AND sm1.start_date < ocm.EndDate
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm2 ON ocm.PersonId = sm2.person_id AND ocm.CourseId = sm2.base_group_id
       AND ocm.StartDate < sm2.end_date AND sm2.start_date < ocm.EndDate
       AND sm2.start_date > sm1.start_date
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm3 ON ocm.PersonId = sm3.person_id AND ocm.CourseId = sm3.base_group_id
       AND ocm.StartDate < sm3.end_date AND sm3.start_date < ocm.EndDate
       AND sm3.start_date > sm1.start_date AND sm3.start_date < sm2.start_date      
  --make sure there isn't an existing Learning Aim that would intersect this new one.
  LEFT JOIN sims.curr_p16_student_aim psa ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id   
       AND sm1.start_date < psa.planned_end AND psa.start_date < sm1.end_date   
  WHERE sm1.end_date < ocm.EndDate 
    AND sm3.base_group_id IS NULL          
    
END 
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG- 
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
 
  SELECT [DEBUG15]    = 'AIM:Create New'
        ,[StudentId]  = ocm.PersonId
        ,[Student]    = sp.surname+', '+sp.forename
        ,[CourseId]   = ocm.CourseId
        ,[Course]     = sbg.short_name
        ,[StartDate]  = sm1.start_date
        ,[PlannedEnd] = ocm.EndDate 
  FROM #OldCourseMembers ocm
  JOIN #Tolerances tol ON ocm.CourseId = tol.course_id
  JOIN sims.sims_base_group sbg ON ocm.CourseId = sbg.base_group_id
  JOIN sims.sims_person sp ON ocm.PersonId = sp.person_id  
  JOIN sims.sims_member sm1 ON ocm.PersonId = sm1.person_id AND ocm.CourseId = sm1.base_group_id 
       AND ocm.StartDate < sm1.end_date AND sm1.start_date < ocm.EndDate
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm2 ON ocm.PersonId = sm2.person_id AND ocm.CourseId = sm2.base_group_id
       AND ocm.StartDate < sm2.end_date AND sm2.start_date < ocm.EndDate
       AND sm2.start_date > sm1.start_date
  --make sure there isn't an existing Learning Aim that would intersect this new one.
  LEFT JOIN sims.curr_p16_student_aim psa ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id   
       AND sm1.start_date < psa.planned_end AND psa.start_date < sm1.end_date   
  WHERE sm2.person_id IS NULL 
    AND psa.person_id IS NULL
    AND sm1.end_date < ocm.EndDate
    
    AND sm1.end_date < @CurrentDate ---REQUIREMENT: Actual End Date must be earlier than the start of "tomorrow"

    AND (   --REQUIREMENT: Actual End Date must not be within the Completion Tolerance of the original end date!
         (DATEADD(MONTH,tol.compMagnitude,sm1.end_date) < ocm.EndDate AND tol.compTolType = 'M')  --the gap between actual end date and planned end date is not less than number of Months
      OR (DATEADD(WEEK,tol.compMagnitude,sm1.end_date) < ocm.EndDate AND tol.compTolType = 'W')   --the gap between actual end date and planned end date is not less than number of Weeks
      OR (DATEADD(DAY,tol.compMagnitude,sm1.end_date) < ocm.EndDate AND tol.compTolType = 'D')    --the gap between actual end date and planned end date is not less than number of Days
        )
    
  	AND (ocm.PersonId = @DebugStudentId OR @DebugStudentId = 0) 
	AND (ocm.CourseId = @DebugCourseId  OR @DebugCourseId = 0)    
END 
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-  
  
  
DECLARE @NewLearningAims TABLE (PersonId int, CourseId int, StartDate smalldatetime, ActualEndDate smalldatetime, PlannedEndDate smalldateTime, closures int, openers int) 
  
INSERT INTO @NewLearningAims (PersonId, CourseId, StartDate, ActualEndDate, PlannedEndDate)  
  SELECT ocm.PersonId
        ,ocm.CourseId
        ,sm1.start_date
        ,sm1.end_date
        ,ocm.EndDate 
  FROM #OldCourseMembers ocm
  JOIN #Tolerances tol ON ocm.CourseId = tol.course_id
  JOIN sims.sims_member sm1 ON ocm.PersonId = sm1.person_id AND ocm.CourseId = sm1.base_group_id 
       AND ocm.StartDate < sm1.end_date AND sm1.start_date < ocm.EndDate
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm2 ON ocm.PersonId = sm2.person_id AND ocm.CourseId = sm2.base_group_id
       AND ocm.StartDate < sm2.end_date AND sm2.start_date < ocm.EndDate
       AND sm2.start_date > sm1.start_date
  --make sure there isn't an existing Learning Aim that would intersect this new one.
  LEFT JOIN sims.curr_p16_student_aim psa ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id   
       AND sm1.start_date < psa.planned_end AND psa.start_date < sm1.end_date   
  WHERE sm2.person_id IS NULL 
    AND psa.person_id IS NULL
    AND sm1.end_date < ocm.EndDate    
    AND sm1.end_date < @CurrentDate ---REQUIREMENT: Actual End Date must be earlier than the start of "tomorrow"
  
    AND (   --REQUIREMENT: Actual End Date must not be within the Completion Tolerance of the original end date!
         (DATEADD(MONTH,tol.compMagnitude,sm1.end_date) < ocm.EndDate AND tol.compTolType = 'M')  --the gap between actual end date and planned end date is not less than number of Months
      OR (DATEADD(WEEK,tol.compMagnitude,sm1.end_date) < ocm.EndDate AND tol.compTolType = 'W')   --the gap between actual end date and planned end date is not less than number of Weeks
      OR (DATEADD(DAY,tol.compMagnitude,sm1.end_date) < ocm.EndDate AND tol.compTolType = 'D')    --the gap between actual end date and planned end date is not less than number of Days
        )
     
INSERT INTO sims.curr_p16_student_aim (person_id, course_id, start_date, planned_end)
  SELECT PersonId, CourseId, StartDate, PlannedEndDate FROM @NewLearningAims
     
-- There may have been learning aims where the course end date has now been moved forward
-- so if there is no completion status and the end date is now later, adjust the planned end date to match
UPDATE p16
   SET p16.planned_end = sm.end_date
  FROM sims.curr_p16_student_aim p16
  JOIN sims.sims_member sm
    ON sm.person_id = p16.person_id
   AND sm.base_group_id = p16.course_id
   AND sm.start_date = p16.start_date
 WHERE sm.end_date > p16.planned_end
   AND p16.completion_status_id IS NULL

 -- The list of learning aims is going to be returned from the procedure so that the completion status can be updated by Curriculum Assignment screens
 -- so we have to add in learning aims which were already existing because of retake or UKPRN which are now finished but with no completion status
INSERT INTO @NewLearningAims (PersonId, CourseId, StartDate, ActualEndDate, PlannedEndDate)  
  SELECT p16.person_id
        ,p16.course_id
        ,p16.start_date
        ,ncm.end_date
        ,p16.planned_end 
  FROM  sims.curr_p16_student_aim p16
  JOIN #NewCourseMembersSorted ncm 
    ON p16.person_id = ncm.person_id
   AND p16.course_id = ncm.course_id 
   AND p16.start_date = ncm.start_date
LEFT JOIN @NewLearningAims nla
       ON nla.PersonId = p16.person_id
	  AND nla.CourseId = p16.course_id
	  AND nla.StartDate = p16.start_date
WHERE nla.PersonId IS NULL
  AND ( (ncm.end_date < p16.planned_end AND p16.completion_status_id IS NULL)
   OR   (ncm.end_date = p16.planned_end AND p16.completion_status_id IS NOT NULL) )
  

  --NB Each new membership can only intersect a SINGLE old membership in such a way that it finishes earlier than the old one.
  --So there is no danger of multiple Learning Aims being created with the same new membership start date.

-------------------------------------------------
--g) update counts of openers and closures: 
--    number of course memberships terminating at the Actual End date of the Learning aim record 
--    number of course memberships starting immediately after the Actual End date of the Learning aim record 
        
UPDATE aims SET closures =  cl.TotalClosures
               ,openers = (SELECT COUNT(*) FROM sims.sims_member sm 
                             JOIN sims.curr_course cc ON sm.base_group_id = cc.base_group_id 
                              AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id)                        
                           WHERE aims.PersonId = sm.person_id AND DATEDIFF(day,aims.ActualEndDate+'00:01',sm.start_date ) BETWEEN 0 AND 4   
                          )                         
FROM @NewLearningAims aims
JOIN (SELECT [PersonId] =sm.person_id, [EndDate]=sm.end_date, [TotalClosures] = COUNT(*) 
      FROM sims.sims_member sm 
      JOIN sims.curr_course cc ON sm.base_group_id = cc.base_group_id 
       AND (cc.department_id = @curriculum_department_id OR cc.department_id = @alternative_department_id) 
      GROUP BY sm.person_id, sm.end_date) cl
  ON aims.PersonId = cl.PersonId AND aims.ActualEndDate = cl.EndDate



--h) and finally add in the remaining potential Learning Aim Records that we didnt create (they are "completed") 
INSERT INTO @NewLearningAims (PersonId, CourseId, StartDate, ActualEndDate, PlannedEndDate,closures,openers)  
  SELECT ocm.PersonId
        ,ocm.CourseId
        ,sm1.start_date
        ,sm1.end_date
        ,ocm.EndDate 
        ,-1 --closures
        ,-1 --openers
  FROM #OldCourseMembers ocm
  JOIN #Tolerances tol ON ocm.CourseId = tol.course_id
  JOIN sims.sims_member sm1 ON ocm.PersonId = sm1.person_id AND ocm.CourseId = sm1.base_group_id 
       AND ocm.StartDate < sm1.end_date AND sm1.start_date < ocm.EndDate
  --make sure there isnt a later course membership     
  LEFT JOIN sims.sims_member sm2 ON ocm.PersonId = sm2.person_id AND ocm.CourseId = sm2.base_group_id
       AND ocm.StartDate < sm2.end_date AND sm2.start_date < ocm.EndDate
       AND sm2.start_date > sm1.start_date
  --make sure there isn't an existing Learning Aim that would intersect this new one.
  LEFT JOIN sims.curr_p16_student_aim psa ON psa.person_id = sm1.person_id AND psa.course_id = sm1.base_group_id   
       AND sm1.start_date < psa.planned_end AND psa.start_date < sm1.end_date
  LEFT JOIN @NewLearningAims x ON ocm.PersonId = x.PersonId AND ocm.CourseId = x.CourseId AND ocm.StartDate = x.StartDate       
  WHERE sm2.person_id IS NULL 
    AND psa.person_id IS NULL
    AND sm1.end_date < ocm.EndDate  
    AND x.PersonId IS NULL  
  
 
  
-------------------------------------------------

--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-
IF (@DebugStudentId > 0) or (@DebugCourseId > 0)
BEGIN
SELECT 
 [DEBUG16] = 'Statistics'
,[TotalRecordsBefore]             = @TotalRecordsBefore
,[TotalRecordsAfterInitialMerge] = @TotalRecordsAfterInitialMerge
,[TotalRecordsAfterFullMerge]    = @TotalRecordsAfterFullMerge
END
--DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-DEBUG-

--==DROPTEMPTABLES================================================================================ 
-- Drop temporary tables and return data to caller

DROP TABLE #NewCourseMembersSorted
DROP TABLE #NewCourseMembers
DROP TABLE #MembershipBounds
DROP TABLE #OldCourseMembers
DROP TABLE #Students
DROP TABLE #Tolerances

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF @Diagnosis = 1
BEGIN
   DROP TABLE #ExcludedOldCourseMembers
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS

--Return TABLE0 : list of newly created Learning Aim records
SELECT [PersonId]        = PersonId
     , [CourseId]        = CourseId
     , [StartDate]       = StartDate
     , [ActualEndDate]   = ActualEndDate
     , [PlannedEndDate]  = PlannedEndDate
     , [StudentSurname]  = sp.surname
     , [StudentForename] = sp.forename
     , [CourseCode]      = sbg.short_name
     , [CurrRegGroup]    = reg.short_name
     , [openers]         = a.openers
     , [closures]        = a.closures
FROM @NewLearningAims a
JOIN sims.sims_base_group sbg ON a.CourseId = sbg.base_group_id
JOIN sims.sims_person sp ON a.PersonId = sp.person_id
LEFT JOIN sims.sims_member sm 
     JOIN sims.sims_base_group reg ON sm.base_group_id = reg.base_group_id
     JOIN sims.sims_base_group_type sbgt ON reg.base_group_type_id = sbgt.base_group_type_id AND sbgt.code = 'RegGrp'
  ON a.PersonId = sm.person_id AND GETDATE() between sm.start_date AND ISNULL(sm.end_date,@EndOfTime)

--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS
IF (@Diagnosis = 1)
BEGIN
SELECT 
  [Diagnosis] = 'Diagnosis'
  ,* FROM @CourseDiagnosis
END
--DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS-DIAGNOSIS

return 0
go

/* Test

exec sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2 @AcadYear_event_instance_id = 22108

exec sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2 @AcadYear_event_instance_id=22108, @DebugStudentID=6420

exec sims.curr_pia_CurrUpdateCourseMembership_ExecuteV2 @AcadYear_event_instance_id=22108, @PersonXml = '<data> <row PersonId=''6420'' /> <row PersonId=''6421'' /> </data>'

*/








-- END PATCH SCRIPT 2
GO


-- START PATCH SCRIPT 3

/*
Author: Arun Tiwari
Date  : 15 Oct 2014
Details: PRB8726 Fix - Removed space from the provider code '3670010E'.
*/

set nocount on

--Variable for storing course classification type id
DECLARE @regioncode NVARCHAR(10)

SELECT @regioncode = sr.region_code
	FROM sims.db_version_info ver
	INNER JOIN sims.sims_region sr
	ON sr.region_id = ver.region_id

IF @regioncode = 'WALES'
BEGIN

--Remove provider code space
   update sims.elwa_provider set code='3670010E', external_code='3670010E' where code='3670010E '

--Remove provider code space.
   update sims.curr_course_classification set code='3670010E' where code='3670010E '

	END

Go









-- END PATCH SCRIPT 3
GO


-- START PATCH SCRIPT 4

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2005                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : Schools Workforce Return                                                                        */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2014                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED				                                                                        */
/*--------------------------------------------------------------------------------------------------------------*/

SET NOCOUNT ON
GO

EXEC sims.db_p_drop 'sims.sre_pix_datapackage_swc_GetStaffsContractorServiceDetails', 'P'
GO

ALTER PROCEDURE sims.sre_pix_datapackage_swc_GetStaffsContractorServiceDetails
(
	@school_id     INT,			-- School id
    @return_id     INT,			-- Census return id 
	@xml_staff	   TEXT,		-- Eligible staff ids
    @show_contractDetails BIT,	-- Value from workforce setting file whether to show contract details or not 
    @show_serviceDetails BIT	-- Value from workforce setting file whether to show service details or not 
)

AS
/*---------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL                                                                                     */
/* Object     : sims.sre_pix_datapackage_swc_GetStaffsContractorServiceDetails                                   */
/* Purpose    : This stored procedure retrieves Contract or Service Details of the particular Staff in the school*/
/* Author     : Mitesh Shah                                                                                      */
/* Date       : 03/07/2007                                                                                       */
/* Reviewer   : Abhinav Pandey                                                                                  */
/* Date       : 26th July 2007                                                                                  */

/* Modified By: Mike Robinson                                                                                    */
/* Date       : 17/10/2007                                                                                       */
/* Date       : Round the values of FTE Hours, Hours per week and weeks per year.                                */
/*																												 */
/* Modified By: Ashwini Lote																					 */
/* Date       : 19/11/2008                                                                                       */
/* Details    : School Workforce Summer return 2009 changes														 */
/*				Check @term <> 'AUT' is replaced by @term = 'SPR'												 */
/*				as old open contract/service is not required in Autumn and Summer Term.		                     */
/* Date       : 27/11/2008                                                                                       */
/* Details    : Included those contracts or service agreements which starts from 31st Jan and ends ON 27th Feb	 */
/*				Added a column to recognise record is of contract or service agreement.							 */
/*				Column is added to regonise roles differently is there	exists same contract/service id			 */
/*				for the record																					 */
/*																												 */
/* Modified by: R Vijayavel																						 */
/* Date		  : 25/03/2009																						 */
/* Details	  : The condition, which checks the date in calendar month, is changed to a more simple calculation  */
/*              WHERE a contact or service agreement must be at least 28 days to be included in the return.      */
/*																												 */
/* Modified by: R Vijayavel																						 */
/* Date		  : 02/07/2009																						 */
/* Details	  : SWC Spring 2010 changes                                                                          */
/*              1. Changed the parameters of the function swfr_fnb_check_regular_service_duration.               */
/*              2. Fixed MIS reference no. 0809-384383:Pay Award,which falls ON census date, would be picked up. */
/*              3. Added more 'case' clauses for spine point. The new clauses include 'UT' & 'EX'.               */
/*              4. Modified the format of salary amount, the format is 9.99                                      */
/*              5. Fixed 0908-565997 - Pay item Daily Rate should not be present for historic SA                 */
/*              6. MIS 0908-566917 : SafeguardSalary should not be collected for historic contract               */
/*																												 */
/* Modified by: R Vijayavel                                                                                      */
/* Date		  : 29/01/2010																						 */
/* Details	  : SWC Autumn 2010:																				 */
/*				1. Requirement id:SWAUT2010-013 - Removed Spinr point 'ETMIN' for Excellent Teacher              */
/*				2. Requirement id:SWAUT2010-012 - Total pay calculation has been modified.                       */ 
/*				3. Requirement id:SWAUT2010-011 - Contracts/SAs, which falls in previous academic year and/or    */
/*                 which is current on census date, will be collected for the return.                            */
/*              4. Implemented to collect salary amount  for service agreement,default value is '0.00'           */
/*                                                                                                               */ 	
/* Modified by: R Vijayavel                                                                                      */
/* Date		  : 11/02/2011																						 */
/* Details	  : SWC Autumn 2011:																				 */
/*              1. SWAUT11_042 - Added 'is_contract_eligible' column to check whether the contract is            */ 
/*                 Service agreement - Teaching assistants or Service agreement - Support staff.                 */
/*                                                                                                               */ 	
/* Modified by: Sandeep Kharmate                                                                                 */
/* Date		  : 20/02/2011																						 */
/* Details	  : SWC AUT2012 3051:For Kent LA the calculation for 'Total Pay' must include the value of Kent      */
/*              specific additional allowances.  																 */
/*                                                                                                               */
/* Modified by: R Vijayavel                                                                                      */
/* Date       : 22/02/2012                                                                                       */
/* Details    : Historical contracts/SAs should not be collected for academy schools                             */
/*                                                                                                               */ 	
/* Modified by: Sandeep Kharmate                                                                                 */
/* Date		  : 19/03/2011																						 */
/* Details	  : SWC AUT2012 4794:For Kent table structure modified as per additional paymnet detail stored proc. */
/*                                                                                                               */
/*                                                                                                              */
/* Modified by: Ankit Shah                                                                                      */
/* Date       : 30 March 2012                                                                                   */
/* Details    : Updated code to incorporate new schema changes of sims.per_employment_history                   */
/*                                                                                                              */
/* Modified by: Ankit Shah                                                                                      */
/* Date       : 24 July 2012                                                                                   */
/* Details    : Updated code to set school arrival date to earliest employment start date                       */
/*                                                                                                               */ 	
/* Modified by: Sandeep Kharmate                                                                                 */
/* Date		  : 11/09/2012																						 */
/* Details	  : SWC AUT2012 BTR 17888:Changes done for SWC Base pay and Additional payment for service agreement. */
/*                                                                                                               */
/* Modified by: Jahir P                                                                                          */  
/* Date       : 16/11/2012                                                                                       */  
/* Details    : PRB2826-Unable to Create&Validate School Work Force Census if salary amount exceeds more than    */  
/*              8 digits.Increased size of the Salary amount field in the procedure.                             */  
/*                                                                                                               */  
/* Modified by: Deepak G Naik																					 */
/* Date		  : 19/12/2012																						 */
/* Details	  : PRB3248-Unable to Create&Validate School Work Force Census.										 */
/*				Column name or number of supplied values does not match table definition.						 */	
/*                                                                                                               */  
/* Modified by: R Vijayavel																					     */
/* Date		  : 19/04/2013																						 */
/* Details	  : SWC 2013 - User story 25779:Update base pay from new table, if edit paydetails is allowed.		 */
/*				Column name or number of supplied values does not match table definition.						 */
/*                                                                                                               */
/* Modified by: R Vijayavel                                                                                      */
/* Date       : 26/04/2013                                                                                       */
/* Details    : User story 25648 - Academy start date is calculated from date dfe number changed.                */	
/*                                                                                                               */
/* Modified by: R Vijayavel                                                                                      */
/* Date       : 09/05/2013                                                                                       */
/* Details    : User story 27924 - Retrieved original base pay.                                                  */	
/*                                                                                                               */
/* Modified by: R Vijayavel                                                                                      */
/* Date       : 22/05/2013                                                                                       */
/* Details    : Bug 30677 - Added collection start date to @academy_converted_date, if date_dfe_number_changed   */
/*                          is not exist                                                                         */	
/*                                                                                                               */
/* Modified by: R Vijayavel                                                                                      */
/* Date       : 13/06/2013                                                                                       */
/* Details    : Story 32501 - Employment start date should be collected when contract start date falls.          */
/*                                                                                                               */
/* Modified by: R Vijayavel                                                                                      */
/* Date       : 05/04/2014                                                                                       */
/* Details    : Story 47134 - Modified calculation of salary amount logic based on pay range implementation.     */
/*                                                                                                               */
/* Modified by: Vikas Belote                                                                                     */
/* Date       : 26/08/2015                                                                                       */
/* Details    : PRB11304 - Modified calculation of salary amount When fte hours and weeks worked year is zero.   */
/*---------------------------------------------------------------------------------------------------------------*/

SET NOCOUNT ON


---delcare the local variables
DECLARE @idoc INT
    ,@census_date SMALLDATETIME
    ,@continuous_contracts_start_date SMALLDATETIME
    ,@school_number VARCHAR(4)  
	,@max_date SMALLDATETIME				 -- To hold maximum date
	,@term VARCHAR(3)
	,@lea_numbar INT	
	,@school_type VARCHAR(10)			   -- To hold school type	
	,@academy_converted_date SMALLDATETIME -- To hold date when school converted to academy
	,@edit_paydetails BIT				   -- Value from workforce setting file whether to allow edit pay details or not				
	,@pre2014_framework VARCHAR(10)	
	,@2014_framework VARCHAR(10) 
--creating the temp table whcih will store the information for INSERT/UPDATE
DECLARE @tblContractorService TABLE
(                           
    person_id     INT
)

DECLARE @tblContractorServiceInfo TABLE(
                                            person_id     INT,
                                            contract_id INT,
                                            contract_Type    VARCHAR(4) COLLATE DATABASE_DEFAULT ,
                                            service_term_id INT,
                                            pay_scale_id INT,
                                            by_salary_range bit,  
                                            contract_StartDate  SMALLDATETIME ,
                                            contract_EndDate    SMALLDATETIME,
                                            post VARCHAR(6) COLLATE DATABASE_DEFAULT ,
                                            school_ArrivalDate SMALLDATETIME,
                                            daily_Rate CHAR(1) COLLATE DATABASE_DEFAULT,
                                            destination_Code VARCHAR(10) COLLATE DATABASE_DEFAULT,
                                            origin           VARCHAR(10) COLLATE DATABASE_DEFAULT, 
                                            lASchoolLevel CHAR(1) COLLATE DATABASE_DEFAULT,
                                            establishment VARCHAR(4)COLLATE DATABASE_DEFAULT,
                                            scale          VARCHAR(10) COLLATE DATABASE_DEFAULT, 
                                            region_Spine   VARCHAR(10) COLLATE DATABASE_DEFAULT, 
                                            spine_point    VARCHAR(10) COLLATE DATABASE_DEFAULT,
											salaried         BIT,											
                                            salary_amount    DECIMAL(13,4),     --PRB2826
                                            salary_amount_modified CHAR(1) DEFAULT 'F',
                                            original_base_pay DECIMAL(13,4), -- To hold amount from P7
                                            safeguard_salary    VARCHAR(5) COLLATE DATABASE_DEFAULT,
                                            fte_hours         DECIMAL(7,4),  -- values from SA or ' from service term for contracts'
                                            weeks_worked_year DECIMAL(7,5),  -- Values from service term 
                                            hours_per_week    DECIMAL(7,4),  -- values from SA/contract                                                    
                                            weeks_per_year    DECIMAL(7,4),  -- values from SA/contract                                         
											contract_service_flag VARCHAR(1) COLLATE DATABASE_DEFAULT  DEFAULT ('C'), --SWFSummer09:Added a column to recognise record is of contract or service agreement.	
											is_contract_eligible CHAR(1),  -- To check whether the collected contract is eligible or not based on SWC post as SA-TA and SA-SUP should not be collected for return.
											pay_review_date SMALLDATETIME,
											pay_framework VARCHAR(10),
											minimum_pay_range DECIMAL(12,2),
											maximum_pay_range DECIMAL(12,2)
                                        )
                                        
-- Declare temp table to store additional payment details 
DECLARE @tblAdditionalPaymentInfo TABLE(                           
                                            person_id				INT,
                                            contract_id				INT,
                                            payment_type			VARCHAR(10) COLLATE DATABASE_DEFAULT,
                                            payment_amount			DECIMAL(10,2),
                                            annual_amount			DECIMAL(10,2),
                                            allowance_start_date	SMALLDATETIME,
                                            allowance_end_date	    SMALLDATETIME,
                                            pay_factor				DECIMAL(7,4),
                                            applicable_month		SMALLINT,
                                            contract_service_flag VARCHAR(1) COLLATE DATABASE_DEFAULT                                            
			                           )                                       
 

-- main 

-- Assign term
SELECT @term = code FROM sims.swfr_census_term sct
INNER JOIN sims.swfr_return sr ON sr.census_term_id = sct.census_term_id
WHERE sr.return_id = @return_id

--Getting the school number to represent the establishment number
SELECT @school_number=school_number 
    FROM sims.sims_school 
WHERE status ='S'

--Fetch census date  for @term
SELECT @census_date = census_date
      ,@continuous_contracts_start_date = continuous_contracts_start_date      
FROM sims.swfr_census_term    
WHERE code = @term

-- Set the max date value
SET @max_date = '2079-06-06T00:00:00'

-- Set school type
SELECT @school_type = st.code from sims.school_type st 
INNER JOIN sims.sims_school ss ON ss.type_id = st.lookup_value_id
WHERE ss.status = 'S'

-- Set edit pay detail from workforce setting
SELECT @edit_paydetails = CASE WHEN edit_paydetail_details='T' THEN 1 ELSE 0 END
FROM sims.swc_configure_setting
-- Set academy transformed date
SELECT @academy_converted_date = ISNULL(date_dfe_number_changed,@continuous_contracts_start_date)
FROM sims.sims_school_previous_information

--Select framework code values
SELECT @pre2014_framework=code FROM sims.swc_pay_framework WHERE lookup_value_id=1
SELECT @2014_framework=code FROM sims.swc_pay_framework WHERE lookup_value_id=2

--Create an internal representation of the XML document.   
EXEC sp_xml_preparedocument @idoc OUTPUT, @xml_staff

INSERT INTO @tblContractorService
(
    person_id 
)
SELECT 
       	p

FROM 	OPENXML (@idoc, '/DataPackage/Row',1)  
WITH     
      	(
          	p	INT	'person_id'
      	)
EXEC sp_xml_removedocument @idoc 


-- MAIN PROCEDURE BODY GOES HERE!

IF (@show_contractDetails = 1)
BEGIN
--Inserting the Staff records for Contract in temp table
INSERT INTO @tblContractorServiceInfo(
                                            person_id
                                            ,contract_id
                                            ,contract_Type   
                                            ,service_term_id
                                            ,pay_scale_id
                                            ,by_salary_range
                                            ,contract_StartDate 
                                            ,contract_EndDate   
                                            ,post 
                                            ,school_ArrivalDate 
                                            ,destination_Code 
                                            ,origin           
                                            ,lASchoolLevel 
                                            ,establishment 
                                            ,scale          
                                            ,region_Spine   
                                            ,spine_point  
											,salaried         
                                            ,salary_amount    
                                            ,safeguard_salary   
                                            ,fte_hours 
                                            ,weeks_worked_year       
										    ,hours_per_week   
											,weeks_per_year   
											,pay_review_date
)
SELECT    [person_id]                      = spc.person_id
    ,[contract_id]                    = spc.contract_id
    ,[contract_Type]                 = spetcat.code
    ,[service_term_id]               = spc.service_term_id
    ,[pay_scale_id]                  = spcps.pay_scale_id
    ,[by_salary_range]               = sst.by_salary_range
    ,[contract_StartDate]            = spc.contract_start_date  
    ,[contract_EndDate]              = spc.contract_end_date  
    ,[post]                          = spsp.code
    ,[school_ArrivalDate]            = peh.employment_start_date
    ,[destination_Code]              = spcdcat.code
    ,[origin]                        = spcocat.code
    ,[lASchoolLevel]                 = 'S'
    ,[establishment]                 =  @school_number
    ,[scale]                         = sppss1.code
    ,[region_Spine]                  = sppsr.code
    ,[spine_Point]                   = CASE sst.by_salary_range WHEN  0  THEN   -- Spine point is retrieved for pay award only
										CASE sppss1.code                                           
                                           WHEN 'LD' THEN 'L' + CAST (CAST(spcps.point AS INT) AS VARCHAR (10))            								   
                                        END
                                       END
	,[salaried]						= sst.salaried
    ,[salary_Amount]                 = CASE WHEN sst.by_salary_range =0 THEN spa.scale_amount 
											ELSE spcps.salary_amount 
										END	
										
    ,[safeguarded_Salary]            = CASE spc.safeguarded_salary     
                                           WHEN 0 THEN 'false'
                                           WHEN 1 THEN 'true'
                                       END 
    
    ,[fte_Hours]	   = sst.hours_worked_per_week
    ,[weeks_worked_year] = sst.weeks_worked_year
    ,[hours_Per_Week]  = spc.hours_week 	
	,[weeks_Per_Year]  = CASE WHEN spc.weeks_year > 53 THEN 53
                              ELSE spc.weeks_year
                         END 
    ,[pay_review_date] = CASE WHEN spsp.code IN ('EXH', 'HDT', 'DHT', 'AHT', 'TCH', 'LDP', 'TCM', 'TCU')
                              THEN spc.latest_payreview_date
                              ELSE NULL
                         END
                                              
FROM  @tblContractorService tcs 
    INNER JOIN sims.per_contract spc 
		ON tcs.person_id = spc.person_id
    INNER JOIN sims.per_employee spe
        ON spc.person_id = spe.person_id       		            
    INNER JOIN sims.per_employment_type spet 
		ON spet.lookup_value_id = spc.employment_type_id
	INNER JOIN sims.per_employment_type_category spetcat
		ON spet.category_id= spetcat.category_id                     
    LEFT JOIN sims.per_employment_history peh
		ON peh.person_id = spe.person_id
		AND spc.contract_start_date BETWEEN peh.employment_start_date AND ISNULL(peh.date_of_leaving,@max_date)
    LEFT JOIN sims.per_post spp 
        ON spp.post_id = spc.post_id
    LEFT JOIN sims.per_swfr_post spsp 
        ON spp.swr_post_id = spsp.lookup_value_id  
    LEFT JOIN sims.per_contract_destination spcd 
		ON spcd.lookup_value_id = spc.destination_id 
	LEFT JOIN sims.per_contract_destination_category spcdcat 
		ON spcd.category_id = spcdcat.category_id
    LEFT JOIN sims.per_contract_origin spco 
		ON spco.lookup_value_id = spc.origin_id
	LEFT JOIN sims.per_contract_origin_category spcocat 
		ON spco.category_id = spcocat.category_id 
    LEFT JOIN sims.per_contract_pay_scale spcps 
		ON spcps.contract_id = spc.contract_id
		AND (spcps.start_date <= @census_date AND ( spcps.end_date IS NULL OR spcps.end_date>= @census_date)
		     AND (ISNULL(spc.contract_end_date,@max_date) >= @census_date))  
    LEFT JOIN sims.per_pay_scale spps 
		ON spcps.pay_scale_id = spps.pay_scale_id 
    LEFT JOIN sims.per_service_term sst
        ON spc.service_term_id = sst.service_term_id
    LEFT JOIN sims.per_award spa 
		ON spc.service_term_id = spa.service_term_id 
			AND  spcps.point = spa.scale_point
			AND (spcps.pay_scale_id = spa.pay_scale_id OR spa.pay_scale_id IS NULL)
			AND sst.by_salary_range = 0
			AND spa.award_date =(SELECT  MAX(paws.award_date) FROM    sims.per_award paws  
								WHERE paws.scale_point = spcps.point  
								AND spc.contract_id = spcps.contract_id   
								AND paws.award_date <= @census_date --0809-384383
								AND ((sst.single_pay_spine = 1 AND paws.pay_scale_id IS NULL AND paws.service_term_id = sst.service_term_id)  
									OR (sst.single_pay_spine = 0 AND paws.pay_scale_id = spcps.pay_scale_id)))
	 LEFT JOIN sims.per_pay_scale_scale sppss1 
		ON sppss1.lookup_value_id = spps.pay_scale_scale_id
	LEFT JOIN sims.per_pay_scale_region sppsr 
		ON sppsr.lookup_value_id = spps.regional_pay_spine_id
   
    WHERE
		sims.swfr_fnb_check_regular_service_duration(spetcat.code,spc.contract_start_date,spc.contract_end_date,@term) = 1

/* 
 SWC AUT 2012 - For Kent school only the calculation for 'Total Pay' must include the value of Kent specific additional allowances. 
 SWC AUT 2013 - Allowances with a code beginning with ?~? will be added (without any pro rata) to Basic Pay, while allowances with a code that does NOT begin with ?~? will be added (with pro rata) to Additional Payments
*/

	INSERT INTO @tblAdditionalPaymentInfo(                           
											   person_id,
											   contract_id,
											   payment_type,
											   payment_amount,
											   annual_amount,
											   allowance_start_date,
											   allowance_end_date,
											   pay_factor,
											   applicable_month,
											   contract_service_flag
											   
										  ) 
	EXEC sims.sre_pix_datapackage_swc_GetAdditionalPaymentsDetails @school_id ,@return_id,@xml_staff,@show_contractDetails,@show_serviceDetails, 1

    
END


-- Update Pay details Min and Max value for Scale point/Range.     
;WITH paydetails_cte
AS
(
 SELECT spa.service_term_id,spa.pay_scale_id,sst.by_salary_range,scale_point, scale_amount, minimum_amount, maximum_amount 
         FROM sims.per_service_term sst
         LEFT JOIN sims.per_award spa 
           ON sst.service_term_id = spa.service_term_id 
		  AND spa.award_date =(SELECT MAX(paws.award_date) FROM sims.per_award paws  
							    WHERE paws.award_date <= @census_date  
							      AND paws.service_term_id = spa.service_term_id 
							      AND (    (sst.by_salary_range = 0 AND sst.single_pay_spine = 1 AND paws.pay_scale_id IS NULL AND paws.service_term_id = sst.service_term_id)  
									    OR (sst.by_salary_range = 0 AND sst.single_pay_spine = 0 AND paws.pay_scale_id = spa.pay_scale_id)
									    OR (sst.by_salary_range = 1 AND paws.pay_scale_id = spa.pay_scale_id)
									  )
							   )
)

update tblInfo
   set minimum_pay_range = CASE WHEN tblInfo.by_salary_range = 1 THEN (select minimum_amount from paydetails_cte tmp Where tmp.pay_scale_id = tblInfo.pay_scale_id and tblInfo.service_term_id = tmp.service_term_id) 
                                WHEN tblInfo.by_salary_range = 0 THEN (select scale_amount from paydetails_cte tmp Where tblInfo.service_term_id = tmp.service_term_id and tmp.scale_point = spps.minimum_point and (tmp.pay_scale_id = tblInfo.pay_scale_id or tmp.pay_scale_id is null)) 
                           END,     
       maximum_pay_range = CASE WHEN tblInfo.by_salary_range = 1 THEN (select maximum_amount from paydetails_cte tmp Where tmp.pay_scale_id = tblInfo.pay_scale_id) 
                                WHEN tblInfo.by_salary_range = 0 THEN (select scale_amount from paydetails_cte tmp Where tblInfo.service_term_id = tmp.service_term_id and tmp.scale_point = spps.maximum_point and (tmp.pay_scale_id = tblInfo.pay_scale_id or tmp.pay_scale_id is null)) 
                           END,
       pay_framework     = CASE WHEN tblInfo.contract_StartDate < '2014-09-01' THEN @pre2014_framework
	                            ELSE @2014_framework
	                       END              
from @tblContractorServiceInfo tblInfo 
	 INNER JOIN sims.per_pay_scale spps
	 ON tblInfo.pay_scale_id = spps.pay_scale_id 
	 AND tblInfo.service_term_id = spps.service_term_id 
where tblInfo.post IN ('EXH','HDT','DHT','AHT') 	     


IF (@show_serviceDetails = 1)
BEGIN

--Inserting the Staff records for Service agreement in temp table
INSERT INTO @tblContractorServiceInfo(
                                            person_id 
                                            ,contract_id
                                            ,contract_Type   
                                            ,contract_StartDate 
                                            ,contract_EndDate   
                                            ,post 
                                            ,school_ArrivalDate 
                                            ,daily_Rate 
											,salary_amount
                                            ,destination_Code 
                                            ,origin           
                                            ,lASchoolLevel 
                                            ,establishment 
                                            ,hours_per_week    
                                            ,fte_hours         
                                            ,weeks_per_year    
											,contract_service_flag
											,pay_review_date			
                                        )    
SELECT 
    [person_id]                      = spsa.person_id
    ,[service_agreement_id]           = spsa.service_agreement_id  
    ,[contract_Type]                  = spsat.code
    ,[contract_StartDate]             = spsa.service_start_date  
    ,[contract_EndDate]               = spsa.service_end_date  
    ,[post]                          = psp.code
    ,[school_ArrivalDate]            = peh.employment_start_date 
    ,[daily_Rate]                    = CASE spsa.daily_rate
                                            WHEN 1 THEN 'Y'
                                            ELSE 'N'
                                       END
    ,[salary_amount]                 = spsa.swc_base_pay
    ,[destination_Code]              = 'NTKNWN'
    ,[origin]                        = 'NOTKNW'
    ,[lASchoolLevel]                 = 'S'
    ,[establishment]                 =  @school_number
    ,[hours_Per_Week]  = ROUND(spsa.agreement_hours_week, 2)
 	,[fte_Hours]	   = ROUND(spsa.fte_hours_week, 2)
	,[weeks_Per_Year]  = CASE WHEN ROUND(spsa.weeks_year, 0) > 53 THEN 53
                              ELSE ROUND(spsa.weeks_year, 0)
                         END 

    ,[contract_service_flag] = 'S'     --SWFSummer09: Setting flag to 'S' to recognise record is of type Service agreement.
    ,[pay_review_date] = CASE WHEN spsp.code IN ('EXH', 'HDT', 'DHT', 'AHT', 'TCH', 'LDP', 'TCM', 'TCU')
                              THEN spsa.latest_payreview_date
                              ELSE NULL
                         END  
FROM @tblContractorService tcs1 
   INNER JOIN sims.per_service_agreement spsa 
		ON spsa.person_id = tcs1.person_id
   LEFT JOIN sims.per_service_agreement_type spsat 
		ON spsat.lookup_value_id = spsa.service_type_id                  
   LEFT JOIN sims.per_employee spe1 
		ON spsa.person_id = spe1.person_id          
   LEFT JOIN sims.per_employment_history peh
		ON peh.person_id = spsa.person_id
		AND spsa.service_start_date BETWEEN peh.employment_start_date AND ISNULL(peh.date_of_leaving,@max_date)
    LEFT JOIN sims.per_swfr_post spsp
        ON spsa.swr_post_id = spsp.lookup_value_id   
    INNER JOIN sims.per_swfr_post psp 
		ON psp.lookup_value_id = spsa.swr_post_id
	WHERE
      (
			sims.swfr_fnb_check_regular_service_duration(spsat.code,spsa.service_start_date,spsa.service_end_date,@term) = 1
         )
END

-- Calculate salary amount for salaried service term
UPDATE  @tblContractorServiceInfo  
SET     salary_amount      = salary_amount * (hours_per_week/fte_hours) * (weeks_per_year/weeks_worked_year)            --salary_amount * FTE * pro_rata  
WHERE   salaried    = 1  and (fte_hours <> 0 and weeks_worked_year <> 0)

-- Set Salary amount to zero when fte hours and weeks worked year is zero
UPDATE  @tblContractorServiceInfo
SET		salary_amount  =  0
WHERE	salaried    = 1  and (fte_hours = 0 or weeks_worked_year = 0)

-- Calculate salary amount for non salaried service term
UPDATE  @tblContractorServiceInfo  
SET     salary_amount      = round(salary_amount * hours_Per_Week * weeks_Per_Year,2)
WHERE   salaried    = 0  

/*For kent school add allowances as a part of total pay.*/
UPDATE tcsi
   SET salary_amount = salary_amount + (
										SELECT ISNULL(SUM(tapi.payment_amount),0.00)
                                          FROM @tblAdditionalPaymentInfo tapi
                                         WHERE tcsi.person_id = tapi.person_id
                                           AND tcsi.contract_id = tapi.contract_id
                                        )
from @tblContractorServiceInfo  tcsi  
WHERE contract_service_flag = 'C' 

-- Assign 0.00 to salary amount service agreement, if it is null
UPDATE  @tblContractorServiceInfo  
SET     salary_amount      = 0.00
WHERE   salary_amount IS NULL 
		AND contract_service_flag = 'S'

--Contract or service agreement end date must be provided WHERE the contract is of type "FXT". 
UPDATE @tblContractorServiceInfo
SET contract_EndDate = NULL
WHERE contract_EndDate>=@census_date AND (contract_Type NOT IN('FXT','F') )

--Destination will be null if contract_EndDate>=@census_date
UPDATE @tblContractorServiceInfo
SET destination_Code = NULL
WHERE (contract_EndDate>=@census_date or contract_EndDate IS NULL)  AND  (contract_Type NOT IN('FXT','F'))

-- Update SafeguardSalary
UPDATE @tblContractorServiceInfo
SET safeguard_salary=NULL
-- MIS 0908-566917 : SafeguardSalary should not be included for historic contract
WHERE ISNULL(contract_EndDate,@max_date) < @census_date

-- Spring 2010: 0908-565997 - Pay item Daily Rate should not be present for historic SA
UPDATE @tblContractorServiceInfo
SET daily_Rate = NULL
WHERE ISNULL(contract_EndDate,@max_date) < @census_date 

-- Summer 2010: Total pay should not be collected for historic SA
UPDATE @tblContractorServiceInfo
SET salary_amount = NULL
WHERE ISNULL(contract_EndDate,@max_date) < @census_date 
	AND contract_service_flag = 'S'

-- Autumn 2011: SA-TA and SA-SUP should not be collected for the return
UPDATE @tblContractorServiceInfo
SET is_contract_eligible = CASE WHEN (contract_Type IS NULL OR contract_Type IN ('SLA','SAG','SOT')) AND 
								post IN ('SUP','AVT','TAS') THEN '0'
								ELSE '1'
                           END
-- Autumn 2012: Historical data should not be collected for academy schools (school type = 49)
IF @school_type = '49' -- Check for academy
BEGIN
	UPDATE @tblContractorServiceInfo
	SET is_contract_eligible = '0'
	WHERE contract_EndDate < @academy_converted_date
END

-- Update pay details 
IF (@edit_paydetails = 1)
BEGIN
	-- Update base salary for contracts
	UPDATE temp
		SET temp.salary_amount = spd.base_pay,
			temp.original_base_pay = spd.original_base_pay ,
			temp.salary_amount_modified = CASE WHEN spd.base_pay != spd.original_base_pay THEN 'T'
											ELSE 'F' END ,
			temp.pay_framework=CASE WHEN spd.pay_framework=1 THEN @pre2014_framework
								    WHEN spd.pay_framework=2 THEN @2014_framework
									ELSE NULL END,
			temp.maximum_pay_range=CASE WHEN spd.min_pay=0 AND spd.max_pay=0 THEN NULL ELSE spd.max_pay END,
			temp.minimum_pay_range=CASE WHEN spd.min_pay=0 AND spd.max_pay=0 THEN NULL ELSE spd.min_pay END										
	FROM @tblContractorServiceInfo temp
	JOIN sims.swc_pay_details spd 
		ON spd.person_id = temp.person_id		
        WHERE ((temp.contract_service_flag = 'C' and temp.contract_id = spd.contract_id)
               OR (temp.contract_service_flag = 'S' and temp.contract_id = spd.service_agreement_id))	                    
           AND spd.return_id = @return_id
               
END
-- Retrieve contract data
SELECT 
    DISTINCT person_id
    ,contract_id     
    ,contract_Type   
    ,contract_StartDate 
    ,contract_EndDate   
    ,post 
    ,school_ArrivalDate 
    ,daily_Rate 
    ,destination_Code 
    ,origin 
    ,lASchoolLevel 
    ,establishment 
    ,scale 
    ,region_Spine 
    ,spine_point 
    ,salary_amount = CAST(salary_amount AS DECIMAL(13,2))
    ,original_base_pay = CAST(original_base_pay AS DECIMAL(13,2))
    ,salary_amount_modified
    ,safeguard_salary 
    ,hours_per_week = CAST(hours_per_week AS DECIMAL(7,2))
    ,fte_hours = CAST(fte_hours AS DECIMAL(7,2))
    ,weeks_per_year = CAST(weeks_per_year AS DECIMAL(6,0))    
	,contract_service_flag			--SWFSummer09: Returning flag to recognise record is of contract or service agreement.	
	,is_contract_eligible
	,pay_review_date
	,pay_framework
	,minimum_pay_range = CAST(minimum_pay_range AS DECIMAL(13,2))
	,maximum_pay_range = CAST(maximum_pay_range AS DECIMAL(13,2))
FROM @tblContractorServiceInfo
ORDER BY person_id,contract_id


-- END OF PROCEDURE BODY!
-----------------------------------------------------------
RETURN 0 -- return value if all OK.
GO


-----------------------------------------------------------
-- START OF UNIT TEST SECTION

/*  TEST
Description : Test of functioning
Author      : R Vijayavel
Date        : 13/06/2013
Dataset     : English Secondary
 
-- Perform the test
declare @xml_staff varchar(8000)
set @xml_staff = '<DataPackage>
						<Row><person_id>12105</person_id></Row><Row><person_id>9287</person_id></Row><Row><person_id>15397</person_id></Row>						
					</DataPackage>'

--get the staff id for the Return,and pass it as parameter to Get Contract or Service SP
exec sims.sre_pix_datapackage_swc_GetStaffsContractorServiceDetails 1,19, @xml_staff,1,1

 
-- Result after execution of sp
  
 person_id   contract_id contract_Type contract_StartDate      contract_EndDate        post   school_ArrivalDate      daily_Rate destination_Code origin     lASchoolLevel establishment scale      region_Spine spine_point salary_amount                           original_base_pay                       salary_amount_modified safeguard_salary hours_per_week                          fte_hours                               weeks_per_year                          contract_service_flag is_contract_eligible
----------- ----------- ------------- ----------------------- ----------------------- ------ ----------------------- ---------- ---------------- ---------- ------------- ------------- ---------- ------------ ----------- --------------------------------------- --------------------------------------- ---------------------- ---------------- --------------------------------------- --------------------------------------- --------------------------------------- --------------------- --------------------
9287        416         PRM           2009-01-01 00:00:00     NULL                    SUP    2005-12-16 00:00:00     NULL       NULL             OTHEDU     S             4591          OT         EW           NULL        16830.00                                16830.00                                F                      false            37.00                                   37.00                                   52                                      C                     1
12105       671         PRM           2010-12-01 00:00:00     NULL                    TCH    2008-07-07 00:00:00     NULL       NULL             TCHLEA     S             4591          TE         EW           M4          27104.00                                27104.00                                F                      false            25.00                                   25.00                                   52                                      C                     1
15397       49          SLA           2012-09-01 00:00:00     2012-10-31 23:59:00     TCH    2012-08-25 00:00:00     NULL       NTKNWN           NOTKNW     S             4591          NULL       NULL         NULL        NULL                                    NULL                                    F                      NULL             NULL                                    NULL                                    NULL                                    S                     1
15397       50          SOT           2013-09-01 00:00:00     NULL                    TCH    2013-09-01 00:00:00     Y          NULL             NOTKNW     S             4591          NULL       NULL         NULL        0.00                                    NULL                                    F                      NULL             NULL                                    NULL                                    NULL                                    S                     1
15397       738         FXT           2012-09-01 00:00:00     2013-05-25 23:59:00     TCH    2012-08-25 00:00:00     NULL       LEAPRM           1STAFT     S             4591          NULL       NULL         NULL        NULL                                    NULL                                    F                      NULL             25.00                                   25.00                                   52                                      C                     1
15397       739         PRM           2013-09-01 00:00:00     NULL                    TCH    2013-09-01 00:00:00     NULL       NULL             1STAFT     S             4591          UT         EW           UQT5        23177.00                                NULL                                    F                      false            25.00                                   25.00                                   52                                      C                     1
*/
-- END OF UNIT TEST SECTION
-----------------------------------------------------------













-- END PATCH SCRIPT 4
GO


-- START PATCH SCRIPT 5

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2000                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : Course Manager                                                                                  */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2006                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : REVIEWED                                                                                        */
/*--------------------------------------------------------------------------------------------------------------*/


set nocount on
go
------------------------------------------------------------------------------------------------------------------

exec sims.db_p_drop 'sims.rpt_fnt_course_CourseDetails', 'if'
go

Alter function sims.rpt_fnt_course_CourseDetails
(
    @effective_date smalldatetime -- null for today
)

RETURNS TABLE 
as

/*--------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL                                                                                    */
/* Object     : sims.rpt_fnt_course_CourseDetails                                                               */
/*                                                                                                              */
/* Purpose    : the course details for selected students         			                        */
/* Comments   :                                                                                                 */
/*                                                                                                              */
/* Author     : Devin Deng                                                                                      */
/* Date       : 19/10/2006	                                                                                */
/* Reviewer   :   		                                                                                */
/*                                                                                                              */
/* Author     : Sooraj M.V                                                                                      */
/* Date       : 08/12/2006	                                                                                */
/* Comments   :  modified the expression to hold more size                                                       */
/*                                                                                                              */
/* Modified   : Mike Robinson                                                                                   */
/* Date       : 07/06/2007	                                                                                    */
/* Change     : Add QAN code to returned columns.                                                               */
/*                                                                                                              */
/* Modified   : Walter                                                                                          */
/* Date       : 31.01.2008	                                                                                */
/* Change     : KB76781 curr_course_classification_id, curr_course_classification_type_id -> not null           */
/*                                                                                                              */  
/* Modified   : Deepak R. Naik                                                                                  */  
/* Date       : 18.09.2008																						*/  
/* Change     : KB82709 Changes made in the function returning the QAN's for past and future dated courses		*/
/*				and students.																					*/		
/*				Changed the inner join to a left outer join for a inner Query getting the Course details.		*/		
/* Modified   : Jhabiulla Phatan                                                                                */  
/* Date       : 20.04.2009																						*/  
/* Change     : KB86843 To resolve issue Duplicates appear on Analysis Report                                   */	
/*																												*/
/* Modified   : Jeetendra Jaiswal                                                                               */
/* Date       : 03/11/2014	                                                                                    */
/* Change     : Story 61966: Report reason for withdrawal						                                */	
/*																												*/
/* Modified   : Ged                                                                                             */
/* Date       : 21/11/2014	                                                                                    */
/* Change     : Bug 64620 fix: duplicates when multiple qans                                                    */
/*																												*/
/*              Allow null @effective_date to represent Today in order to remove rpt_vix_course_CourseDetail    */
/*              which has exactly the same code as this     					                                */	
/*																												*/
/* Modified   : Abhishek Sudhir                                                                                 */  
/* Date       : 6/1/2015																						*/  
/* Change     : PRB9000 - To resolve issue of Incorrect figures showing when running the report                 */	
/*																												*/
/* Modified   : Vikas Belote                                                                                    */  
/* Date       : 01/09/2015																						*/  
/* Change     : PRB11316 - Set effective to today date when effective date is null.                             */	
/*--------------------------------------------------------------------------------------------------------------*/



return

select
 	  [ID]			    = ltrim(sm.base_group_id)+','+ltrim(sm.person_id)+','+ltrim(convert(int, sm.start_date))
	, [student_id]		= sm.person_id
	, [course_id] 		= cc.base_group_id
	, [course_code]		= sbg.code
	, [description]		= sbg.description
	, [study_hours]		= cc.study_hours
	, [tuition_hours]	= cc.tuition_hours
	, [qualification_hours]   = cc.planned_qualification_hours
	, [non_qualification_hours] = cc.planned_non_qualification_hours
	, [minutes_per_tt_cycle]= cc.minutes_per_tt_cycle
	, [start_date]		= sm.start_date 
	, [end_date]	 	= sm.end_date
	, [planned_end_date]= p16.planned_end
	, [comp_status]	 	= ccs.description
	, [withdrawal_reason] = cwr.[description]
	, [disability_status] 	= cccs.code
	, [learning_activity] 	= cccl.code
	, [learning_provider]	= cccp.code
	, [learning_medium]	= cccm.code
	, [uln]				= uln.unique_learner_number
	, [level]         	= cccv.code
    , [qan]             = cccq.code
    , [qanExpiry]       = cqm.certification_end
    , [genericqan]      = cccgq.code
    , [ppos]            = cccppos.code
 	, [subject]       	= star.code
	, [QCA_code] 		= qca.code
    , [core_aim]        = case when cla.course_id is null then 'F' else 'T' end

-- Course details
from sims.curr_course cc
inner join sims.sims_member sm			on sm.base_group_id = cc.base_group_id
AND 
(
sm.start_date <= isnull(@effective_date, getdate()) and isnull(sm.end_date,'2079-06-06 00:00:00') >= isnull(@effective_date, getdate()) 	--@effective_date will be provided by the date view 
)
--
inner join sims.sims_base_group sbg		on sbg.base_group_id = cc.base_group_id
inner join sims.curr_department sd		on sd.department_id = cc.department_id
	and sd.name = 'Curriculum'

	-- 'QCA'
left  join ( sims.curr_course_subject qcasub	
	inner join sims.curr_subject_code qca		on qca.subject_code_id = qcasub.subject_code_id 
	inner join sims.curr_subject_codeset codeset	on codeset.subj_codeset_id = qca.subj_codeset_id
		and codeset.owner = 'QCA'
	) on qcasub.base_group_id = cc.base_group_id

	-- 'Subject'
left  join ( sims.curr_course_subject starsub	
	inner join sims.curr_subject_code star			on star.subject_code_id = starsub.subject_code_id
	inner join sims.curr_subject_codeset starcodeset	on starcodeset.subj_codeset_id = star.subj_codeset_id
		and starcodeset.owner = 'STAR'
	) on starsub.base_group_id = cc.base_group_id

--KB86843
--When there is more than one classification link record for a course each section following will return a record for the classification type. 
--Hence changed sims.curr_course_classification_link to have a inner join with sims.curr_course_classification.This will get the code for 
--each type in a single record.

	-- 'Learning Medium'
left  join (sims.curr_course_classification_link mlink
	  inner join sims.curr_course_classification cccm     on cccm.curr_course_classification_id = mlink.curr_course_classification_id
		inner join sims.curr_course_classification_type mtype	on mtype.curr_course_classification_type_id = cccm.curr_course_classification_type_id
                                                          and mtype.title = 'DELLS Learning Medium'
		) on mlink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(mlink.start_date,'19000101') and isnull(mlink.end_date,'20790606')

	-- 'Level'
left  join (sims.curr_course_classification_link levellink
	  inner join sims.curr_course_classification cccv     on cccv.curr_course_classification_id = levellink.curr_course_classification_id
		inner join sims.curr_course_classification_type vtype	on vtype.curr_course_classification_type_id = cccv.curr_course_classification_type_id
                                                          and vtype.title = 'Level'
		) on levellink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(levellink.start_date,'19000101') and isnull(levellink.end_date,'20790606')

	-- 'QAN'
left  join (sims.curr_course_classification_link qanlink
	  inner join sims.curr_course_classification cccq     on cccq.curr_course_classification_id = qanlink.curr_course_classification_id
		inner join sims.curr_course_classification_type qtype	on qtype.curr_course_classification_type_id = cccq.curr_course_classification_type_id
                                                          and qtype.title = 'QAN'
		) on qanlink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(qanlink.start_date,'19000101') and isnull(qanlink.end_date,'20790606')

				-- 'QAN Expiry'
left  join (sims.curr_course_classification_link qanExpirylink
	  inner join sims.curr_course_classification cccqe     on cccqe.curr_course_classification_id = qanExpirylink.curr_course_classification_id
		inner join sims.curr_course_classification_type qetype	on qetype.curr_course_classification_type_id = cccqe.curr_course_classification_type_id
                                                          and qetype.title = 'QAN'
        inner join sims.census_qan_discount_code cdc on len(cccqe.code)>11  and substring(cccqe.code, 10, 4) = cdc.discount_code
        inner join sims.census_qan_main cqm on len(cccqe.code)>7 and cqm.qan = left(cccqe.code, 8)	and cqm.discount_code_id=cdc.discount_code_id
		) on qanExpirylink.course_id = cc.base_group_id	
	     and isnull(@effective_date, getdate()) between isnull(qanExpirylink.start_date,'19000101') and isnull(qanExpirylink.end_date,'20790606')
		 
			-- 'DAS Generic QAN'
left  join (sims.curr_course_classification_link genericqanlink
	  inner join sims.curr_course_classification cccgq     on cccgq.curr_course_classification_id = genericqanlink.curr_course_classification_id
		inner join sims.curr_course_classification_type gqtype	on gqtype.curr_course_classification_type_id = cccgq.curr_course_classification_type_id
                                                          and gqtype.title = 'DAS Generic QAN'
		) on genericqanlink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(genericqanlink.start_date,'19000101') and isnull(genericqanlink.end_date,'20790606')
		
			-- 'PPoS'
left  join (sims.curr_course_classification_link pposlink
	  inner join sims.curr_course_classification cccppos     on cccppos.curr_course_classification_id = pposlink.curr_course_classification_id
		inner join sims.curr_course_classification_type ppostype	on ppostype.curr_course_classification_type_id = cccppos.curr_course_classification_type_id
                                                          and ppostype.title = 'PPoS'
		) on pposlink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(pposlink.start_date,'19000101') and isnull(pposlink.end_date,'20790606')
		
		
   -- 'Learning Activity'  
left  join (sims.curr_course_classification_link lalink
	  inner join sims.curr_course_classification cccl     on cccl.curr_course_classification_id = lalink.curr_course_classification_id
		inner join sims.curr_course_classification_type ltype	on ltype.curr_course_classification_type_id = cccl.curr_course_classification_type_id
                                                          and ltype.title = 'DELLS Learning Activity'
		) on lalink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(lalink.start_date,'19000101') and isnull(lalink.end_date,'20790606')

  -- 'Provider'  
left  join (sims.curr_course_classification_link lplink
	  inner join sims.curr_course_classification cccp     on cccp.curr_course_classification_id = lplink.curr_course_classification_id
		inner join sims.curr_course_classification_type ptype	on ptype.curr_course_classification_type_id = cccp.curr_course_classification_type_id
                                                          and ptype.title = 'DELLS Learning Provider'
		) on lplink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(lplink.start_date,'19000101') and isnull(lplink.end_date,'20790606')

  -- 'Special Needs'  
left  join (sims.curr_course_classification_link lslink
	  inner join sims.curr_course_classification cccs     on cccs.curr_course_classification_id = lslink.curr_course_classification_id
		inner join sims.curr_course_classification_type stype	on stype.curr_course_classification_type_id = cccs.curr_course_classification_type_id
                                                          and stype.title = 'DELLS Disability Status'
		) on lslink.course_id = cc.base_group_id
	     and isnull(@effective_date, getdate()) between isnull(lslink.start_date,'19000101') and isnull(lslink.end_date,'20790606')

left join sims.curr_p16_student_aim p16 ON p16.person_id=sm.person_id AND p16.course_id=cc.base_group_id AND p16.start_date=sm.start_date
left join sims.curr_completion_status ccs on ccs.completion_status_id = p16.completion_status_id 
left join sims.sims_person_unique_learner_number uln on  uln.person_id=sm.person_id 
left join sims.curr_core_learning_aim cla 
       on cla.person_id = sm.person_id
      and cla.course_id = sm.base_group_id
      and cla.acad_year_id = (select ei.event_instance_id
                                from sims.sims_event_instance ei
                                join sims.sims_event e
                                  on e.event_id = ei.event_id
                                join sims.sims_event_type et
                                  on et.event_type_id = e.event_type_id
                                 and et.code = 'AcadYear'
                               where isnull(@effective_date, getdate()) between ei.event_start and ei.event_end)  
LEFT OUTER JOIN [sims].[curr_withdrawal_reason] cwr 
	ON cwr.[withdrawal_reason_id] = p16.[withdrawal_reason_id]						   


go

------------------------------------------------------------------------------------------------------------------
/*  TEST
Description: Unit Tests
     Author: Devin Deng
       Date: 19 October 2006

-- Perform the test

declare @effective_date smalldatetime
select @effective_date = getdate()
select * from sims.rpt_fnt_course_CourseDetails(@effective_date)

-- student_id  course_id   course_code description                      study_hours tuition_hours minutes_per_tt_cycle start_date                                             end_date                                               disability_status         learning_activity         learning_provider         learning_medium           level                     subject    QCA_code   ID          
-- ----------- ----------- ----------- -------------------------------- ----------- ------------- -------------------- ------------------------------------------------------ ------------------------------------------------------ ------------------------- ------------------------- ------------------------- ------------------------- ------------------------- ---------- ---------- ----------- 
-- 2875        1843        Bs/GCEA     Business Studies (GCEA)          NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Bs         NULL       198683557
-- 2875        1847        Ch/GCEA     Chemistry (GCEA)                 NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Ch         NULL       199083561
-- 2875        1874        En/GCSEF    English (GCSEF)                  NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCSEF                     NULL                      GCSEF                     GCSEF                     GCSEF                     En         NULL       201783593
-- 2875        1902        Gs/GCEA     General Studies (GCEA)           NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Gs         NULL       204583616
-- 2875        1924        Ma/GCSEF    Maths (GCSEF)                    NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCSEF                     NULL                      GCSEF                     GCSEF                     GCSEF                     Ma         NULL       206783643
-- 2875        3135        Le/GCEA     L & S (GCEA)                     NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Le         NULL       327884849
-- 2875        3145        Rc/GCEA     Recreation (GCEA)                NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Rc         NULL       328884859
-- 2875        3299        Ar/IB       Art (IB)                         NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    IB                        NULL                      IB                        IB                        IB                        Ar         NULL       344285022
-- 2879        1828        Ar/GCEA     Art (GCEA)                       NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Ar         NULL       197203542
-- 2879        1887        Fr/GCEA     French (GCEA)                    NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Fr         NULL       203103601
-- 2879        1902        Gs/GCEA     General Studies (GCEA)           NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Gs         NULL       204603616
-- 2879        1908        Hi/GCEA     History (GCEA)                   NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCEA                      NULL                      GCEA                      GCEA                      GCEA                      Hi         NULL       205203622
-- 2879        1924        Ma/GCSEF    Maths (GCSEF)                    NULL        NULL          NULL                 2006-09-02 00:00:00                                    2007-08-31 23:59:00                                    GCSEF                     NULL                      GCSEF                     GCSEF                     GCSEF                     Ma         NULL       206803643
-- ..........................

exec sims.db_p_grant
-- Display results of the successful test
-- Explain why these results are correct

*/

------------------------------------------------------------------------------------------------------------------







-- END PATCH SCRIPT 5
GO


-- START PATCH SCRIPT 6

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2012                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : School Census                                                                                   */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2015                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED                                                                                      */
/*--------------------------------------------------------------------------------------------------------------*/
set nocount on
go
exec sims.db_p_drop 'sims.census_pix_prior_attainment_Get_BestResult'
                    ,'p'
go
alter procedure sims.census_pix_prior_attainment_Get_BestResult

as

set nocount on

/*--------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL                                                                                    */
/* Object     : sims.census_pix_prior_attainment_Get_BestResult                                                 */
/*                                                                                                              */
/* Purpose    : This stored procedure retrieves the best exam result for English and Maths GCSEs for the each   */
/*              student on the prior attainment screen.                                                         */ 
/* Comments   :                                                                                                 */
/*                                                                                                              */
/* Author     : Mike Robinson                                                                                   */
/* Date       : April 2015                                                                                      */
/*                                                                                                             */
/* Author     : Manisha Ahire                                                                                   */
/* Date       : 28 Sept 2015                                                                                    */
/*            : PRB11535 - use reg shortname instead of desc , increase length of YTI							*/
/*--------------------------------------------------------------------------------------------------------------*/
declare @start_date smalldatetime
declare @end_date smalldatetime


/***********************************************************************************/
/* Get the Autumn census dates. We use this to ensure we only include students who */
/* are/were in YTI 12, 13, 14 on the census date.                                  */
/***********************************************************************************/

select  
        @start_date = '2000-01-01T00:00:00',                    --This is a fixed date well in the past to ensure we go back far enough to pick up their old results
        @end_date =  spct.learning_activities_collected_to
        from    sims.plasc_census_term spct
where   spct.code='AUT'

/***********************************************************************************/
/* Create required temporary tables.                                               */
/***********************************************************************************/

create table #prior_attainment_members
(
     person_id                          int
    ,surname                            varchar(100) collate database_default null
    ,forename                           varchar(100) collate database_default null
    ,uln                                varchar(10)  collate database_default null
    ,yti                                varchar(12)  collate database_default null
    ,regGroup                           varchar(12)  collate database_default null
    ,DOL                                datetime
    ,enrolment_status                   varchar(32)  collate database_default null          
    ,maths_gcse_hightest_grade_id       int
    ,maths_gcse_prior_attainment_id     int
    ,maths_gcse_funding_exception_id    int
    ,english_gcse_hightest_grade_id     int
    ,english_gcse_prior_attainment_id   int
    ,english_gcse_funding_exception_id  int
)

--This table is filled by the exam_pix_StudentExamResults_Get_External_Results procedure which expects it to exist when called.

Create table #members  
(
     person_id                          int
    ,course_id                          int
    ,start_date                         smalldatetime   --course_start_date 
    ,end_date                           smalldatetime   --course_end_date   
    ,surname                            varchar(100) collate database_default
    ,forename                           varchar(100) collate database_default
    ,uln                                varchar(10)  collate database_default
    ,enrolment_status                   varchar(32)  collate database_default
    ,yti                                varchar(12)  collate database_default
    ,dol                                smalldatetime
    ,regGroup                           varchar(12)  collate database_default
    ,maths_gcse_hightest_grade_id       int
    ,maths_gcse_prior_attainment_id     int
    ,maths_gcse_funding_exception_id    int
    ,english_gcse_hightest_grade_id     int
    ,english_gcse_prior_attainment_id   int
    ,english_gcse_funding_exception_id  int
)

--Table to temporarily hold the person_id of the students we are interested in.
Create table #temp_members  
(
    person_id int
)

--Table to hold the exams results prior to working out which ones are the best. This table is not
--populated in this procedure but by one of the procedures called from here.
create table #results
( 
    season_id           integer 
   ,course_id           integer 
   ,season_start_date   datetime -- stores season start date 
   ,season_end_date     datetime -- stores season end date   
   ,person_id           integer 
   ,subject_code        varchar(25)   collate database_default 
   ,subject_name        varchar(200)  collate database_default 
   ,qualification       varchar(150)  collate database_default
   ,level               varchar(50)   collate database_default
   ,season_name         varchar(150)  collate database_default 
   ,result              varchar(1000) collate database_default
   ,award_code          varchar(32)   collate database_default
   ,entry_code          varchar(12)   collate database_default
   ,board_abbreviation  varchar(10)   collate database_default
   ,qan                 varchar(8)    collate database_default
   ,points              float
   ,english_or_maths    char(1) collate database_default
   ,earliest_pass_date_english  smalldatetime
   ,earliest_pass_date_maths    smalldatetime
) 


--Holds the best result in English and Maths for each student.
create table #best_results
(
     row_id                      integer identity
    ,season_id                   integer
    ,course_id                   integer 
    ,season_start_date           datetime -- stores season start date 
    ,season_end_date             datetime -- stores season end date   
    ,person_id                   integer
    ,subject_code                varchar(20)   collate database_default
    ,result                      varchar(1000) collate database_default 
    ,original_result             varchar(1000) collate database_default 
    ,subject_name                varchar(200)  collate database_default 
    ,qualification               varchar(150)  collate database_default 
    ,level                       varchar(50)   collate database_default 
    ,season_name                 varchar(150)  collate database_default
    ,award_code                  varchar(32)   collate database_default
    ,entry_code                  varchar(12)   collate database_default
    ,board_abbreviation          varchar(10)   collate database_default
    ,qan                         varchar(8)    collate database_default
    ,points                      float
    ,english_or_maths            char(1)       collate database_default
    ,earliest_pass_date_english  smalldatetime
    ,earliest_pass_date_maths    smalldatetime
)

--Table of fixed discount codes for English and Maths. The only results we are interested in are ones
--which have one of the discount codes in this table.
create table #discount_codes
(
    code varchar(4) collate database_default,
    english_or_maths char(1) collate database_default
)

insert  #discount_codes (code, english_or_maths) values('5010', 'E')
insert  #discount_codes (code, english_or_maths) values('5030', 'E')
insert  #discount_codes (code, english_or_maths) values('5110', 'E')
insert  #discount_codes (code, english_or_maths) values('FC4',  'E')
insert  #discount_codes (code, english_or_maths) values('FK2A', 'E')
insert  #discount_codes (code, english_or_maths) values('FK2B', 'E')

insert  #discount_codes (code, english_or_maths) values('2210', 'M')
insert  #discount_codes (code, english_or_maths) values('2212', 'M')
insert  #discount_codes (code, english_or_maths) values('2214', 'M')
insert  #discount_codes (code, english_or_maths) values('RB1',  'M')
insert  #discount_codes (code, english_or_maths) values('RB1G', 'M')
insert  #discount_codes (code, english_or_maths) values('RB7J', 'M')


--This is the final table which holds the data to be returned to SIMS to populate the screen
create table #final_table
(
     person_id                      int
    ,course_id                      int
    ,result                         varchar(1000) collate database_default
    ,date_english_result_achieved   smalldatetime
    ,date_maths_result_achieved     smalldatetime
    ,year_english_result_achieved   varchar(10)   collate database_default
    ,year_maths_result_achieved     varchar(10)   collate database_default
    ,subject_name                   varchar(40)   collate database_default
    ,result_subject                 varchar(1)    collate database_default
    ,points                         int
    ,result_date                    smalldatetime
)

/**************************************************************************************************/
/* Call the existing Stored Procedure to get the list of students on the Prior Attainment screen. */
/**************************************************************************************************/

insert #prior_attainment_members
exec sims.census_pix_prior_attainment_GetDetails

--We only need the student id so copy it out and discard everything else by dropping the table
insert into #temp_members(person_id)
select distinct person_id 
from #prior_attainment_members
where yti in ('12','13','14','(12)','(13)','(14)') or yti is null

drop table #prior_attainment_members


/**************************************************************************************************/
/* Create in the #members table a row for each student whose data we want. Then call the exams    */
/* procedure which will read this members table and retrieve all the data for each member.        */
/**************************************************************************************************/

insert into #members(person_id, course_id, start_date, end_date)
    select distinct cm.person_id, null, @start_date, @end_date
    from   #temp_members cm
    join   sims.sims_member mem on cm.person_id = mem.person_id

exec sims.curr_pia_ExaminationResults_get_best_result_ignoring_course_manager @start_date,@end_date, @do_mapping=0


/****************************************************************************************************/
/* For a lot of the results we will have a result value but not the points. In this case update     */
/* the points with the standard values from the DfE table here:                                     */
/* http://www.education.gov.uk/schools/performance/secondary_13/Average_Grade_Per_Qualification.pdf */
/****************************************************************************************************/

update #best_results
set points = case result                    
                      when 'A*' then 58
                      when 'A*-' then 56
                      when 'A+' then 54
                      when 'A'  then 52
                      when 'A-' then 50
                      when 'B+' then 48
                      when 'B'  then 46
                      when 'B-' then 44
                      when 'C+' then 42
                      when 'C'  then 40
                      when 'C-' then 38
                      when 'D+' then 36
                      when 'D'  then 34
                      when 'D-' then 32
                      when 'E+' then 30
                      when 'E'  then 28
                      when 'E-' then 26
                      when 'F+' then 24
                      when 'F'  then 22
                      when 'F-' then 20
                      when 'G+' then 18
                      when 'G'  then 16
                      when 'G-' then 8
                      when 'U'  then 7.9
                      else null
                    end
where ISNULL(points, 0) = 0


/*************************************************************************************************************************************/
/* Because we are cutting this down to results for English and results for Maths then at this point there may be duplicate results   */
/* for one of the subjects because the discount codes are different but are still for the same subject. A further removal of         */
/* duplicates is therefore required here.                                                                                            */
/*************************************************************************************************************************************/

delete o
from #best_results o
where o.points < (select max(i.points) from #best_results i where i.person_id = o.person_id and i.english_or_maths = o.english_or_maths)
or o.row_id < (select max(i.row_id) from #best_results i where i.person_id = o.person_id and o.points = i.points and i.english_or_maths = o.english_or_maths)
and o.english_or_maths = 'E'


delete o
from #best_results o
where o.points < (select max(i.points) from #best_results i where i.person_id = o.person_id and i.english_or_maths = o.english_or_maths)
or o.row_id < (select max(i.row_id) from #best_results i where i.person_id = o.person_id and o.points = i.points and i.english_or_maths = o.english_or_maths)
and o.english_or_maths = 'M'


/***************************************************************************************************************************/
/* Now that we have the final set of results we can insert them into the final table. There are still some gaps to fill in */
/* so we'll do them next.                                                                                                  */
/***************************************************************************************************************************/

insert #final_table
select per.person_id, course_id, br.result, br.earliest_pass_date_english, br.earliest_pass_date_maths, null, null, br.subject_name, english_or_maths, points, br.season_start_date
    from #best_results br
join sims.sims_person per on per.person_id = br.person_id
where (qualification = 'GCSE' or qualification is null) 
order by 1


/*************************************************************************************************************************/
/* We have the result date (or the start date of the exams season) for the english and maths pass result. Turn this into */
/* an NCYear by finding the NCYear the student was in on that date. For external results it is possible that we don't    */
/* know this, in which case it'll just remain blank.                                                                     */
/*************************************************************************************************************************/

update ft
set ft.year_english_result_achieved = isnull(bg.code,'')
from #final_table ft 
join sims.sims_member mem on ft.person_id = mem.person_id
join sims.sims_base_group bg on bg.base_group_id = mem.base_group_id
join sims.sims_base_group_type bgt on bgt.base_group_type_id = bg.base_group_type_id and bgt.code = 'NCYear' and ft.date_english_result_achieved between mem.start_date and mem.end_date
where result_subject = 'E'

update ft
set ft.year_maths_result_achieved = isnull(bg.code,'')
from #final_table ft 
join sims.sims_member mem on ft.person_id = mem.person_id
join sims.sims_base_group bg on bg.base_group_id = mem.base_group_id
join sims.sims_base_group_type bgt on bgt.base_group_type_id = bg.base_group_type_id and bgt.code = 'NCYear' and ft.date_maths_result_achieved between mem.start_date and mem.end_date
where result_subject = 'M'


/**********************************************************************************/
/* The final step is to retrieve the data needed by the screen.                   */
/**********************************************************************************/

select person_id, course_id, result, year_english_result_achieved, year_maths_result_achieved, subject_name, result_subject
 from #final_table


GO




-- END PATCH SCRIPT 6
GO


-- START PATCH SCRIPT 7

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2008                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : School Census                                                                                   */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2013                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED                                                                                      */
/*--------------------------------------------------------------------------------------------------------------*/
set nocount on
go
exec sims.db_p_drop 'sims.census_pix_prior_attainment_GetDetails'
                    ,'p'
go
alter procedure sims.census_pix_prior_attainment_GetDetails

as

set nocount on

/*--------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL                                                                                    */
/* Object     : sims.census_pix_prior_attainment_GetDetails                               		            */
/*                                                                                                              */
/* Purpose    : This stored procedure is used to get the prior attainments Details					            */
/*				from table census_prior_attainment        														*/
/* Comments   :                                                                                                 */
/*                                                                                                              */
/* Author     : Sachin salvekar  																	 			*/
/* Date       : 2nd Apr 2015                                                                                    */
/*																												*/
/* Author     : Mike Robinson                                                                                   */
/* Date       : 23 April 2015                                                                                   */
/*            : Retrieve the YTI and Reg group for leavers and get the current reg group on today's date when   */
/*              it isn't correctly retrieving it on census day.													*/
/* Author     : Manisha Ahire                                                                                   */
/* Date       : 28 Sept 2015                                                                                    */
/*            : PRB11535 - use reg shortname instead of desc , increase length of YTI							*/
/*																												*/
/* Author     : Abhishek Sudhir                                                                                 */
/* Date       : 1 Oct 2015																						*/
/* Desc       : PRB11499 - To avoid crash on save by listing only students and not applicants					*/
/*--------------------------------------------------------------------------------------------------------------*/

-- Get surname, forename, yti, reg group, uln, dol and prior attainment data

--  get all pupils where yti is 12+, onroll status 'C' or 'M' and membership exists in learning aim collection period in a temp table
-- join temp table with prior attainment table to get prior attainment data
-- return the result

declare @start_date datetime
declare @end_date	datetime
declare @census_date datetime
declare @funding_exemption_id int
declare @this_school_id int

select  
		@start_date =spct.learning_activities_collected_from ,
	    @end_date =  spct.learning_activities_collected_to,
		@census_date = spct.census_date
from    sims.plasc_census_term spct
where   spct.code='AUT'

select @funding_exemption_id = funding_exemption_id
from sims.census_pa_funding_exemption
where [description] = 'No exemption'

select @this_school_id = school_id from sims.sims_school where status = 'S'

create table #members
(
  person_id int
  ,surname   VARCHAR(100) 		COLLATE DATABASE_DEFAULT NULL
	,forename VARCHAR(100) 		COLLATE DATABASE_DEFAULT NULL
	,uln varchar(10)  		COLLATE DATABASE_DEFAULT NULL
	,yti varchar(12)  		COLLATE DATABASE_DEFAULT NULL
	,regGroup varchar(12)  		COLLATE DATABASE_DEFAULT NULL
	,DOL datetime
	,enrolment_status VARCHAR(32) COLLATE DATABASE_DEFAULT NULL
	,[maths_gcse_hightest_grade_id] INT
	,[maths_gcse_prior_attainment_id] INT
	,[maths_gcse_funding_exception_id] INT
	,[english_gcse_hightest_grade_id] INT
	,[english_gcse_prior_attainment_id] INT
	,[english_gcse_funding_exception_id] INT
)

-- below query gives me list of pupils who are in YTI 12, 13 and 14
insert into #members(person_id, surname, forename, uln)
select
distinct
	sp.person_id,
	sp.surname,
	sp.forename,
	uln.unique_learner_number	
from
sims.sims_person sp
left join sims.stud_school_history hist on sp.person_id = hist.person_id and hist.school_id = @this_school_id and @census_date between hist.start_date and (isnull(hist.end_date, @census_date))
inner join sims.sims_member sm 
	on sp.person_id = sm.person_id
inner join sims.stud_student st
	on sp.person_id = st.person_id
inner join sims.sims_base_group sbg 
	on sm.base_group_id = sbg.base_group_id
inner join sims.sims_base_group_type sbgt
	on sbg.base_group_type_id = sbgt.base_group_type_id
left join sims.sims_person_unique_learner_number uln
	on uln.person_id = sp.person_id
Where sbgt.code = 'NCYear' and sbg.code in ('12','13','14')
and sm.start_date <= @end_date and isnull(sm.end_date,@start_date) >= @start_date

update m set yti = sbg.code
from #members m
inner join sims.sims_member sm on m.person_id = sm.person_id
inner join sims.sims_base_group sbg 
	on sm.base_group_id = sbg.base_group_id
inner join sims.sims_base_group_type sbgt
	on sbg.base_group_type_id = sbgt.base_group_type_id
Where sbgt.code = 'NCYear' and sbg.code in ('12','13','14')
and (@census_date between sm.start_date and isnull(sm.end_date, @census_date))

-- update enrolment status
 UPDATE m
 SET enrolment_status = som.description
 FROM #members m
 JOIN sims.stud_school_history ssh ON ssh.person_id = m.person_id
	AND ssh.school_id = @this_school_id
	AND ssh.start_date = (SELECT MAX(start_date) FROM sims.stud_school_history 
												  WHERE person_id = m.person_id
												  AND	school_id = @this_school_id
												  AND	start_date <= @census_date)												  
 JOIN sims.stud_on_roll_mode som ON som.on_roll_mode_id = ssh.on_roll_mode_id	

 --update DOL
update mem set dol = hist.end_date
from #members mem inner join 
sims.stud_school_history hist
on      hist.person_id = mem.person_id
inner join sims.sims_school ss
      on ss.school_id=hist.school_id and ss.status = 'S'
inner join sims.stud_on_roll_mode sorm
      on hist.on_roll_mode_id = sorm.on_roll_mode_id 
where   hist.end_date is not null
and     hist.end_date between @start_date and @end_date
and     hist.end_date = (select  MAX(end_date)
                         from    sims.stud_school_history ssh1
                         where   ssh1.person_id = hist.person_id
                          and    ssh1.school_id = hist.school_id)


-- updating reg group
update mem set regGroup = sbg.short_name
from #members mem 
inner join sims.sims_person sp
	on mem.person_id = sp.person_id
inner join sims.sims_member sm
	on mem.person_id = sm.person_id
inner join sims.sims_base_group sbg
	on sbg.base_group_id = sm.base_group_id
inner join sims.sims_base_group_type sbgt
	on sbgt.base_group_type_id = sbg.base_group_type_id
where sbgt.code = 'RegGrp' 
and ((sm.end_date is not null and mem.dol is not null and sm.end_date = mem.dol) or (@census_date between sm.start_date and isnull(sm.end_date, @census_date)) or (@census_date between sm.start_date and isnull(sm.end_date, @census_date)))


-- If student is a leaver get their Reg Group and their YTI to the values they had on DOL
update mem set regGroup = '(' + sbg.short_name + ')'
from #members mem 
inner join sims.sims_person sp
	on mem.person_id = sp.person_id
inner join sims.sims_member sm
	on mem.person_id = sm.person_id
inner join sims.sims_base_group sbg
	on sbg.base_group_id = sm.base_group_id
inner join sims.sims_base_group_type sbgt
	on sbgt.base_group_type_id = sbg.base_group_type_id
where sbgt.code = 'RegGrp' 
and mem.DOL between sm.start_date and sm.end_date

update m set yti = '(' + sbg.code + ')'
from #members m
inner join sims.sims_member sm on m.person_id = sm.person_id
inner join sims.sims_base_group sbg 
	on sm.base_group_id = sbg.base_group_id
inner join sims.sims_base_group_type sbgt
	on sbg.base_group_type_id = sbgt.base_group_type_id
Where sbgt.code = 'NCYear' and sbg.code in ('12','13','14')
and m.DOL between sm.start_date and isnull(sm.end_date, m.DOL)

-- updating prior attainment data
update mem set 
	 [maths_gcse_hightest_grade_id] = cpa.maths_gcse_hightest_grade_id
	,[maths_gcse_prior_attainment_id] = cpa.maths_gcse_prior_attainment_id
	,[maths_gcse_funding_exception_id] = isnull(cpa.maths_gcse_funding_exception_id,@funding_exemption_id) --Default value for funding exemption No Exemption
	,[english_gcse_hightest_grade_id] = cpa.english_gcse_hightest_grade_id
	,[english_gcse_prior_attainment_id] = cpa.english_gcse_prior_attainment_id
	,[english_gcse_funding_exception_id] = isnull(cpa.english_gcse_funding_exception_id,@funding_exemption_id)  ----Default value for funding exemption No Exemption
from #members mem
left join sims.census_prior_attainment cpa
	on mem.person_id = cpa.person_id 
 
select  person_id ,surname   ,forename ,uln ,yti ,regGroup ,DOL ,enrolment_status ,maths_gcse_hightest_grade_id ,maths_gcse_prior_attainment_id ,maths_gcse_funding_exception_id,
		english_gcse_hightest_grade_id ,english_gcse_prior_attainment_id ,english_gcse_funding_exception_id 

from #members
order by surname

drop table #members

GO

------------------------------------------------------------------------------------------------------------------
/*  TEST
Description: Unit Tests
     Author: Sachin Salvekar                                                                                   
       Date: 02nd April 2015   
    Dataset: English Secondary

-- Perform the test
begin tran

exec sims.census_pix_prior_attainment_GetDetails

*/



-- END PATCH SCRIPT 7
GO


-- START PATCH SCRIPT 8

----------------------------------------------------------------------------------------------------------------
--
-- Module     : SIMS Database SQL Server 2014
-- Platform   : Pulsar
-- Area       : STAR/CTF												
-- Copyright  : (c) Capita Business Services Ltd 1984-2015
--            : This code has been written by Capita Education Services for use within its own SIMS systems.
--            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any
--            : means or used without the express written permission of Capita Education Services.
-- Status     : 
-- Purpose    : Update Welsh Programmes of Study codes
-- Author     : Jhabiulla Phatan
--              Oct 2015 Revised for v4.0 (LearningActivityList1415 v4.0.xlsx)
--------------------------------------------------------------------------------------------------------------

-- WALES ONLY
declare @region VARCHAR(10)
        
select @region = sr.region_code
  from sims.sims_region sr
  join sims.db_version_info vr
    on sr.region_id=vr.region_id

if @region = 'WALES'
begin

	-- Create a temporary table to hold all the Programmes
	declare @programmes table
	(
	   code varchar(10) collate database_default,
	   subject varchar(100) collate database_default,
	   description varchar(100) collate database_default
	)

	-- Now load the temporary table
	insert into @Programmes values ('0012A03B','General education','2 AS')
	insert into @Programmes values ('0012B03B','General education','2 AS equivalent')
	
	--commented this as not required 
	-- Set to Inactive any that no longer exist in the list
	--   update sims.plasc_p16_programme
	--   set active = 0
    --    where code not in (select code from @programmes)

	-- Update any that need updating
	update p16
	   set p16.subject = p.subject
	     , p16.description = p.description
		 , p16.active = 1
	  from sims.plasc_p16_programme p16
	  join @programmes p
	    on p.code = p16.code
     where (p16.subject != p.subject or p16.description != p.description or p16.active = 0)

	-- insert any new ones
	insert into sims.plasc_p16_programme (code, subject, description, active)
	select p.code, p.subject, p.description, 1
	  from @programmes p
	  left join sims.plasc_p16_programme pp -- check not already there
		on pp.code = p.code
	where pp.programme_id is null


	--In version 6 of consolidated patch this code was inserted as 99999999
	--so now changing this to 9999999B.
	update sims.plasc_p16_programme
	set code = '9999999B'
	where code = '99999999'
	
	--insert if not exists
	if not exists(select 1 from sims.plasc_p16_programme where code = '9999999B')
	begin 
		insert into sims.plasc_p16_programme (code, subject, description, active)
		select '9999999B','General education','Non-funded Programme',1
	end

	    
end  

GO



-- END PATCH SCRIPT 8
GO


-- START PATCH SCRIPT 9

----------------------------------------------------------------------------------------------------------------
--                                                                                                              
-- Module     : SIMS Database SQL Server 2008                                                                   
-- Platform   : Pulsar                                                                                          
-- Area       : Student Course Details
-- Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2014
--            : This code has been written by Capita Education Services for use within its own SIMS systems.    
--            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    
--            : means or used without the express written permission of Capita Education Services.              
--
-- Purpose    : Function to map new (2014) discount codes to the old ones if found in the mapping table exam_pi_qan_indicator_mapping
-- Author     : Ged April 2014
----------------------------------------------------------------------------------------------------------------
-- Modified By: Arun Sutar
-- Date		  : 28/09/2015
-- Details    : PRB11360 - Fixed - Added Year check when multiple QANs found            
----------------------------------------------------------------------------------------------------------------

exec sims.db_p_drop 'sims.census_fnc_map_discount_code','FN'
go

alter function sims.census_fnc_map_discount_code
(
   	@qan varchar(8) -- exam qan
   ,@disc_code varchar(4) -- exam discount code
   ,@Year	   varchar(4)	
)
returns varchar(4)
as
begin

    declare @matches int
    select @matches = count(1) from sims.exam_pi_qan_indicator_mapping map where map.QAN = @qan and map.new_discount_code = @disc_code

    if @matches = 1 -- single mapping found so use it
    begin
        select @disc_code = map.current_discount_code 
          from sims.exam_pi_qan_indicator_mapping map 
         where map.QAN = @qan 
           and map.new_discount_code = @disc_code
    end
    else if @matches > 1 -- multiple matches, there is more than one mapping found then check for the year  
    begin
       select @disc_code = map.new_discount_code -- Exam Discount code 
       from sims.exam_pi_qan_indicator_mapping map 
       where map.QAN = @qan and [year] =@Year
       and map.new_discount_code = @disc_code 
    end
	else
	  set @disc_code = NULL -- set to null and the calling proc will revert it to QWS QAN/Disc
    
    -- else not found so leave it alone   

    return @disc_code

end
go


------------------------------------------------------------------------------------------------------------------
/*  UNIT TEST

select sims.census_fnc_map_discount_code('10006758','RB55') -- should return 2440
union all 
select sims.census_fnc_map_discount_code('50027761','RH3') -- should return 1010
union all 
select sims.census_fnc_map_discount_code('50025077','FK2B') -- should return 5030
union all 
select sims.census_fnc_map_discount_code('50026410','JA2') -- should return null because it maps to multiple old codes
union all 
select sims.census_fnc_map_discount_code('50027761','RH4') -- should return RH4 because no match found


*/




-- END PATCH SCRIPT 9
GO


-- START PATCH SCRIPT 10

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2008                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : CourseManager                                                                                   */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2012                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED                                                                                      */
/*--------------------------------------------------------------------------------------------------------------*/
    
set nocount on
go

exec sims.db_p_drop 'sims.curr_pia_ExaminationResults_get_best_result_ignoring_course_manager', 'P'
go
ALTER  procedure sims.curr_pia_ExaminationResults_get_best_result_ignoring_course_manager    
(
    @start_date      smalldatetime,
    @end_date        smalldatetime,
    @do_mapping bit = 1 -- if this is set, the QAN/Disc code is mapped in exam_pi_qan_indicator_mapping
)
   
AS    
/*--------------------------------------------------------------------------------------------------------------*/      
/*                                                                                                              */      
/* Format     : Transact-SQL                                                                                    */
/* Object     : curr_pia_ExaminationResults_get_best_result_ignoring_course_manager                             */
/*              Based on sims.curr_pia_ExaminationResults_matching                                              */
/*                                                                                                              */
/* Purpose    : Retrieves the best exam result for each of the students and Discount Codes passed in.           */
/*                                                                                                              */
/* Author     : Mike Robinson                                                                                   */
/* Date       : April 2015                                                                                      */
/*                                                                                                              */      
/*                                                                                                              */      
/*--------------------------------------------------------------------------------------------------------------*/      
/* Modified By: Arun Sutar																						*/
/* Date		  : 28/09/2015																						*/
/* Details    : PRB11360 - Fixed - To display Exam QAN/Discount Code in AO QAN/Disc in membership				*/
/* Modified By: Manisha Ahire                                                                                   */
/* Date       : 28 Sept 2015                                                                                    */
/*            : PRB11535 - Increase length of title																*/
/*--------------------------------------------------------------------------------------------------------------*/
set nocount on    
    
BEGIN    

declare  @result_category_id integer 

/**********************************************************************************/
/* Create the required temporary tables.                                          */
/**********************************************************************************/
 
-- Temporary table to store the seasons data which we will need later.

create table #seasons 
( 
     index1                    integer identity not null
    ,season_id                 integer
    ,season_start_date         datetime
    ,season_end_date           datetime
    ,embargo_id int
) 
    

-- Temporary table to store the best results only of the candidates.

create table #best_cand_grp
(
     person_id      integer
    ,course_id      integer
    ,subject_code   varchar(20) collate database_default
    ,points         float
)
   

-- Temporary tables to store the candidate results with same outcome

create table #duplicateQans
(
     person_id  integer
    ,QAN varchar(20) collate database_default
    ,points float
)


-- Create  a couple of temp tables to improve performance 

create table #result_category_link 
(       
     aspect_id      int 
    ,category_id    int
)   


create table #temp_results          
(
     student_id      int 
    ,resultset_id    int 
    ,aspect_id       int
    ,result          varchar(1000)  collate database_default
    ,gradeset_id     int
)  

--Temporary table to hold external results (entered via the Tools | External Results | Manual Entry menu in Exams Organiser)
create table #external_results
(
     student_id      int
    ,[date]          varchar(10) collate database_default
    ,[level]         varchar(40) collate database_default
    ,board           varchar(40) collate database_default
    ,code            varchar(10) collate database_default
    ,title           varchar(60) collate database_default
    ,result          varchar(15) collate database_default
    ,result_date     smalldatetime
    ,points          float
)

-- Set a flag to indicate whether the exam results calculations in Course Manager and School Census will include Discounting.
-- This is set on the Course Manager settings screen.

declare @discountedFlag bit

select @discountedFlag = default_discount_results  
  from sims.course_manager_setting 
    

-- Get the category id for results to use later
select @result_category_id = ec.category_id 
  from sims.asm_category ec 
  join sims.asm_category_type   ect   on  ec.category_type_id = ect.category_type_id 
                                      and ect.title = 'Process' 
                                      and ec.category_name = 'Result' 
  join sims.sims_agency         sea   on  ect.module_id = sea.agency_id 
                                      and sea.system = 'T' 


insert into #result_category_link (aspect_id, category_id)
select aspect_id
      ,category_id 
 from  sims.asm_aspect_category_link 
where  category_id = @result_category_id
              

/**********************************************************************************/
/* Retrieve all the exams seasons data for both EDI and non-EDI results.          */
/**********************************************************************************/

-- Insert EDI season(s) 

insert into #seasons 
    select distinct seas.season_id,
                    svei.event_start, 
                    svei.event_end,
                    ere.resultset_id 
     from sims.exam_season                          seas 
     join sims.sims_event_instance                  svei on  seas.event_instance_id = svei.event_instance_id 
     join sims.sims_event                           sve  on  svei.event_id = sve.event_id 
     join sims.exam_season_pattern                  vsp  on  sve.event_id = vsp.event_id 
     join sims.exam_lookup                          lku  on  lku.code = seas.season_status 
     join sims.asm_result                           ser  on  seas.resultset_id = ser.resultset_id      
     join sims.asm_aspect_category_link             acl  on  ser.aspect_id = acl.aspect_id 
     join sims.asm_category                         ec   on  ec.category_id = acl.category_id 
                                                         and ec.category_name = 'Result' 
     join sims.asm_category_type                    ect  on  ec.category_type_id = ect.category_type_id 
                                                         and ect.title = 'Process' 
     join sims.sims_agency                          sea  on  ect.module_id = sea.agency_id 
                                                         and sea.system = 'T' 
    left join sims.exam_result_embargo              ere  on  ere.resultset_id = seas.resultset_id 
                                                         and ere.start_date <= getdate() 
                                                         and ere.end_date >= getdate() 
    left join sims.exam_result_embargo_exception    eree on  eree.result_embargo_id = ere.result_embargo_id 

    where   lku.tablename = 'exam_season' 
    and     lku.columnname = 'season_status' 
    and     ( 
                ( ere.result_embargo_id is not null and ( isnull(ec.category_name, 'OK') not in ( 'Result', 'Equivalent', 'Endorsement' ) ) ) or ere.result_embargo_id is null 
            ) 
    and     (
                (svei.event_start>=@start_date and  svei.event_end <=@end_date) or (svei.event_start<=@start_date and  svei.event_end >=@start_date)
            )
                
-- Insert Non EDI season(s) 
    
insert into #seasons 

select distinct seas.season_id,
              svei.event_start, 
              svei.event_end,
              ere.resultset_id 
  from   sims.exam_season seas 
         join sims.sims_event_instance                  svei on  seas.event_instance_id = svei.event_instance_id 
         join sims.sims_event                           sve  on  svei.event_id = sve.event_id 
         join sims.exam_season_pattern                  vsp  on  sve.event_id = vsp.event_id 
         join sims.exam_lookup                          lku  on  lku.code = seas.season_status 
         join sims.exam_pi_result                       ser  on  seas.resultset_id = ser.resultset_id          
         left join sims.exam_result_embargo             ere  on  ere.resultset_id = seas.resultset_id 
                                                             and ere.start_date <= getdate() 
                                                             and ere.end_date >= getdate() 
         left join sims.exam_result_embargo_exception   eree on  eree.result_embargo_id = ere.result_embargo_id 

  where  lku.tablename = 'exam_season' 
    and  lku.columnname = 'season_status' 
    and  ( ( ere.result_embargo_id is not null ) or ere.result_embargo_id is null ) 
    and  ( ( svei.event_start>=@start_date and  svei.event_end <=@end_date) or (svei.event_start<=@start_date and  svei.event_end >=@start_date) )
    and  seas.season_id not in(select season_id from #seasons)


/*************************************************/
/* Insert EDI results                            */
/*************************************************/
 
insert into #temp_results (res.student_id ,   evs.resultset_id ,   aa.aspect_id ,   res.result  ,   aa.gradeset_id  ) 
    select   res.student_id
            ,evs.resultset_id 
            ,aa.aspect_id    
            ,res.result
            ,aa.gradeset_id 
    from sims.asm_result        res 
    join sims.exam_via_season   evs on res.resultset_id = evs.resultset_id
    join #seasons               ss  on ss.season_id = evs.season_id 
    join sims.asm_aspect        aa  on res.aspect_id = aa.aspect_id
    join #result_category_link  acl on aa.aspect_id = acl.aspect_id  

            
if not exists(select 1 from #seasons where embargo_id is not null)
begin 
insert into #results 
select distinct season_id           = ses.season_id,
                course_id           = null,
                season_start_date   = s.season_start_date,
                season_end_date     = s.season_end_date,
                person_id           = sr.student_id,
                subject_code        = csc.code,
                subject_name        = csc.title,
                qualification       = seq.jc_qualification_code, 
                [level]             = sel.jc_level_code,   
                season_name         = (select esp.season_pattern_code 
                                         from sims.sims_event e 
                                         join sims.sims_event_instance ei on e.event_id = ei.event_id 
                                         join sims.exam_season_pattern as esp on esp.event_id = e.event_id 
                                         where  ei.event_instance_id = ses.event_instance_id
                                      ) + ' ' + ses.calendar_year, 
                result              = sr.result,
                award_code          = ea.award_code,
                entry_code          = vae.element_code,
                board_abbreviation  = evb.board_abbreviation,
                qan                 = vae.qual_accr_number,
                points              = isnull(epv.points,0),
                english_or_maths    = disc.english_or_maths,
				null,
				null

 from sims.exam_season                 ses 
 join sims.exam_series                 es   on  ses.season_id = es.season_id 
 join sims.exam_vit_exam_board         evb  on  es.board_id = evb.board_id
 join #seasons                         s    on  ses.season_id = s.season_id 
 join sims.exam_element                see  on  es.series_id = see.series_id 
 join sims.exam_base_element           ebe  on  ebe.assessment_id = see.assessment_id 
                                             and ebe.element_type = 'C' 
 join sims.curr_subject_code           csc  on  ebe.qca_code_id = csc.subject_code_id 
 join #discount_codes                  disc on  disc.code = csc.code
 join sims.exam_level                  sel  on  ebe.level_id = sel.level_id 
 join sims.exam_qualification          seq  on  sel.qualification_id = seq.qualification_id 
 join sims.exam_assessment_aspect      esa  on  see.assessment_id = esa.assessment_id 
                                            and es.series_id = esa.series_id 
 join sims.exam_vit_assessment_element vae  on  es.series_id = vae.series_id  
                                             and see.assessment_id = vae.assessment_id 
 join sims.exam_vit_award_element evea on  vae.element_id = evea.element_id
 left join sims.exam_award_history     evh  on  evea.assessment_id = evh.assessment_id
 left join sims.exam_award             ea   on  evh.assessment_id=ea.assessment_id
 join #temp_results                    sr   on  esa.aspect_id = sr.aspect_id
                                            and sr.resultset_id = ses.resultset_id                            
 join sims.asm_gradeset                ags  on  ags.gradeset_id = sr.gradeset_id
 left join (sims.exam_pi_value epv join sims.exam_grade_map egm on epv.grade = egm.a2c_grade)  on  vae.qual_accr_number = epv.qan  
                                                                                                and egm.edi_grade = sr.result   
                                                                                                and egm.gradeset_code = rtrim(substring(ags.default_name, 4, 2))
where seq.jc_qualification_code = 'GCSE'        


/*******************************************************************************************************************************/
/* Insert External results. These are retrieved by calling another stored procedure to fill the *external_results temp table   */
/* and then taking from the temp table only the data we are interested in.                                                     */
/*******************************************************************************************************************************/

insert #external_results
  exec sims.exam_pix_StudentExamResults_Get_External_Results


insert into #results
    select season_id           = null,
           course_id           = null,
           season_start_date   = null,
           season_end_date     = null,
           person_id           = er.student_id,
           subject_code        = er.code,
           subject_name        = er.title,
           qualification       = null, 
           [level]             = er.level,   
           season_name         = null,
           result              = er.result,
           award_code          = '',
           entry_code          = '',
           board_abbreviation  = null,
           qan                 = 'EXTERNAL',
           points              = er.points,
           english_or_maths    = disc.english_or_maths,
		   null,
		   null

      from      #external_results   er
      join      #discount_codes     disc  on disc.code = er.code
      left join #seasons            s     on er.result_date between s.season_start_date and s.season_end_date


/*******************************************************************************************************************************/
/* Insert non EDI results                                                                                                      */
/*******************************************************************************************************************************/

 insert into #results 
  select distinct 
         season_id          = ses.season_id,
         course_id          = null,
         season_start_date  = s.season_start_date,
         season_end_date    = s.season_end_date,
         person_id          = EP.person_id,
         subject_code       = csc.code,
         subject_name       = csc.title,
         qualification      = seq.jc_qualification_code, 
         [level]            = sel.jc_level_code,   
         season_name        = '',
         result             = ep.result,
         award_code         = '',
         entry_code         = '',
         board_abbreviation = eb.board_abbreviation,
         qan                = ep.QAN,
         points             = epv.points,
         english_or_maths   = disc.english_or_maths,
		 null,
		 null

    from #seasons s
    join sims.exam_season         ses  on ses.season_id = s.season_id
    join sims.exam_series         es   on ses.season_id = es.season_id 
    join sims.exam_pi_result      ep   on ep.season_id = s.season_id
    join sims.curr_subject_code   csc  on ep.subject_code_id = csc.subject_code_id 
    join #discount_codes          disc on csc.code = disc.code
    join sims.exam_level          sel  on ep.level_id = sel.level_id 
    join sims.exam_qualification  seq  on sel.qualification_id = seq.qualification_id 
    join sims.exam_board          eb   on ep.board_id = eb.board_id
    left join sims.exam_pi_value  epv  on ep.QAN = epv.qan and epv.grade = ep.result         

   where seq.jc_qualification_code = 'GCSE'
 end

/*************************************************************************************************/
/* Ensure that we have points for every one so that we can determine which ones are passes       */
/*************************************************************************************************/
update #results
set points = case result                    
                      when 'A*' then 58
                      when 'A*-' then 56
                      when 'A+' then 54
                      when 'A'  then 52
                      when 'A-' then 50
                      when 'B+' then 48
                      when 'B'  then 46
                      when 'B-' then 44
                      when 'C+' then 42
                      when 'C'  then 40
                      when 'C-' then 38
                      when 'D+' then 36
                      when 'D'  then 34
                      when 'D-' then 32
                      when 'E+' then 30
                      when 'E'  then 28
                      when 'E-' then 26
                      when 'F+' then 24
                      when 'F'  then 22
                      when 'F-' then 20
                      when 'G+' then 18
                      when 'G'  then 16
                      when 'G-' then 8
                      when 'U'  then 7.9
                      else null
                    end
where ISNULL(points, 0) = 0


/**************************************************************************************************************************************************************/
/* Update the earliest pass date in English by getting the earliest season start date of a result where the points are >= 38 (i.e. a C- or better). This will */
/* update all the rows (including duplicates) but this isn't an issue as the duplicate rows will be discarded later.                                          */
/**************************************************************************************************************************************************************/

update r
set r.earliest_pass_date_english = (select min(r2.season_start_date)
                                       from #results r2 
                                      where r2.person_id = r.person_id 
                                        and r2.english_or_maths = r.english_or_maths 
                                        and r2.points >= 38
										and r2.english_or_maths = 'E'
                                    )
from #results r
where r.earliest_pass_date_english is null
  and r.english_or_maths = 'E'


--Now do the same for Maths. We have to do this separately to stop the English contaminating the Maths and vice-versa
update r
set r.earliest_pass_date_maths = (select min(r2.season_start_date)
                                     from #results r2 
                                    where r2.person_id = r.person_id 
                                      and r2.english_or_maths = r.english_or_maths 
                                      and r2.points >= 38
									  and r2.english_or_maths = 'M'
                                  )
from #results r
where r.earliest_pass_date_maths is null
  and r.english_or_maths = 'M'


/*************************************************************************************************/
/* Map any new subject (discount) codes to current ones                                          */
/*************************************************************************************************/
if @do_mapping = 1
begin
    update #results 
       set subject_code = sims.census_fnc_map_discount_code(qan, subject_code,YEAR(season_start_date))
    
    -- if the mapping did not work, the subject_code will be set to null, in which case we need to use the QWS QAN and Discount code (from the course)
    update #results
       set qan          = left(cc.code,8)
         , subject_code = substring(cc.code,10,4)
      from #results                         r
      join sims.curr_course_classification  cc on cc.curr_course_classification_id = (
                                                        select top 1 csc.curr_course_classification_id
                                                         from sims.curr_course_classification_link  cccl
                                                         join sims.curr_course_classification       csc on cccl.curr_course_classification_id = csc.curr_course_classification_id   
                                                         join sims.curr_course_classification_type  sct on sct.curr_course_classification_type_id = csc.curr_course_classification_type_id
                                                        where cccl.course_id = r.course_id
                                                          and isnull(cccl.start_date, @start_date) <= @end_date
                                                          and isnull(cccl.end_date, @end_date) >= @start_date
                                                          and sct.title='QAN'
                                                          and isnull(cccl.start_date, @start_date) <= r.season_end_date
                                                          and isnull(cccl.end_date, @end_date) >=  r.season_start_date
                                                     order by cccl.start_date desc
                                                ) -- get most recent one if there is more than one   
     where subject_code is null
end
  
/*************************************************************************************************/
/* Insert the the bests results for each candidate per subject                                   */
/*************************************************************************************************/

insert into #best_cand_grp
    select  person_id,
            course_id,
            subject_code,
            MAX(points)
      from #results
  group by person_id, course_id, subject_code

     -- Get only the entries of best results
insert into #best_results (
        season_id,
        course_id,
        person_id, 
        subject_code,
        subject_name,
        qualification,
        level,
        result,
        original_result,
        season_name,
        season_start_date,
        season_end_date,
        award_code,
        entry_code,
        board_abbreviation,
        qan,
        points,
        english_or_maths,
		earliest_pass_date_english,
		earliest_pass_date_maths)
  select  distinct 
        season_id,
        r.course_id,
        r.person_id, 
        r.subject_code, 
        r.subject_name,
        r.qualification,
        r.level,
        result,
        result,
        r.season_name,
        r.season_start_date,
        r.season_end_date,
        r.award_code,
        r.entry_code,
        r.board_abbreviation,
        r.qan,
        r.points,
        r.english_or_maths,
		r.earliest_pass_date_english,
		r.earliest_pass_date_maths
    from #results         as  r 
    join #best_cand_grp   as  bcg  on r.person_id = bcg.person_id and r.subject_code = bcg.subject_code
    join #discount_codes      disc on r.subject_code = disc.code and (r.points = bcg.points or r.points is null)   
      

    --Where there is more than one result with the same QAN for a student insert the highest result into the duplicates
    --table for use later in discarding lower-level duplicates

insert into #duplicateQans
  select    person_id,
            qan,
            max(points)
    from #best_results 
    group by person_id, qan
    having count(*) > 1
    
    insert into #best_results (season_id,
                                course_id,
                                  person_id, 
                                  subject_code,
                                  subject_name,
                                  qualification,
                                  level,
                                  result,
                                  original_result,
                                  season_name,
                                  season_start_date,
                                  season_end_date,
                                  award_code,
                                  entry_code,
                                  board_abbreviation,
                                  qan,
                                  points)
      select  distinct season_id,
              r.course_id,
              r.person_id, 
              r.subject_code, 
              r.subject_name,
              r.qualification,
              r.level,
              result,
              result,
              r.season_name,
              r.season_start_date,
              r.season_end_date,
              r.award_code,
              r.entry_code,
              r.board_abbreviation,
              r.qan,
              r.points
        from  #results as r 
          where r.result='Q'
         --update result=? if QAN and discount code confilicts 
       update  b
       set b.result='?'
       from #best_results  b
       inner join #results r
       on b.person_id=r.person_id
       and b.qan=r.qan
       where exists (select 1 from #results r1 where r1.person_id=r.person_id and r1.qan<>r.qan and r1.subject_code<>r.subject_code and r1.course_id=r.course_id)
      
       --update result=? if more than 1 different results with same points
        update  b
       set b.result='?'
       from #best_results  b
       inner join #results r
       on b.person_id=r.person_id
       and b.qan=r.qan
       where exists (select 1 from #results r1 where r1.person_id=r.person_id and r1.result<>r.result and r1.course_id=r.course_id and isnull(r1.points,0)=isnull(r.points,0))
    
        -- update result =? if diff QAN+Level with same subject    
       if @discountedFlag=0 
       begin       
       update  b   
       set b.result='?'  
       from #best_results  b  
       inner join #results r  
       on b.person_id=r.person_id  
       and b.qan=r.qan  
       where exists (select 1 from #results r1 where r1.person_id=r.person_id and r1.qan<>r.qan and r1.subject_code=r.subject_code and r1.course_id=r.course_id
                    and r1.level <> r.level)
       end
       
 --update for Q
       update  b
       set b.result='Q'
       from #best_results  b
       inner join #results r
       on b.person_id=r.person_id
       and b.qan=r.qan
       and b.result is null and r.result='Q'

   drop table #seasons    
   drop table #best_cand_grp    
   drop table #result_category_link
   drop table #temp_results

END

-----------------------------------------------------------    
return 0 -- return value if all OK.    




-- END PATCH SCRIPT 10
GO


-- START PATCH SCRIPT 11

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2008                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : CourseManager   				                                                                */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2012                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED                                                                                      */
/*--------------------------------------------------------------------------------------------------------------*/
    
set nocount on
go

exec sims.db_p_drop 'sims.curr_pia_ExaminationResults_matching', 'P'
go
ALTER  procedure sims.curr_pia_ExaminationResults_matching    
(
    @start_date	     smalldatetime,
    @end_date        smalldatetime,
    @do_mapping bit = 1 -- if this is set, the QAN/Disc code is mapped in exam_pi_qan_indicator_mapping
)
   
AS    
/*--------------------------------------------------------------------------------------------------------------*/      
/*                                                                                                              */      
/* Format     : Transact-SQL                                                                                    */
/* Object     : sims.curr_pia_ExaminationResults_matching										                */
/*                                                                                                              */
/* Purpose    : This SP used to populate the results for the selected students								    */
/* Comments   : Please Note                                                                                     */
/*  This Stored procedure is referenced in                                                                      */
/*  (1) sims.census_pix_census_GetLearningAimsQans						:School census Learning Aim Calculation */
/*  (2) sims.curr_pix_CourseManagerDetail_CourseMember_Populate			:Course manager course member populate  */
/*  (3) sims.curr_pix_CourseManagerDetail_ResultDataForReport_Populate  :Course manager Result Report (no mapping) */
/*  (4) sims.curr_pix_StudentCoursesDetail_Populate						:Student course populate                */
/*  (5) sims.curr_pix_Result_Populate_forMissingOutcomes			    :missing outcome bulk update (unused?)  */
/*  (6) sims.sre_pix_datapackage_census_display_learningAims_Coursemanager_Queries :learning aims report        */
/* Please check the impacts for any change in this stored procedure                                             */
/* #results  and #best_results schema should be the same			                                            */
/*                                                                                                              */
/* Author     : Sooraj M.V																						*/
/* Date       : 25th Feb 2012                                                                                   */
/* Modified BY: Arun Sutar																						*/
/* Date		  : 28th Sept 2012																					*/
/* Description:	update result=? if more than 1 different results with same points								*/
/* Modified BY: Arun Sutar																						*/
/* Date		  : 24/09/2013																						*/
/* Description:[PRB5346]- Update result='?' if diff QAN+Level with same subject(QCA code) in a course			*/
/* Modified BY: Jahir P                                                                                         */  
/* Date       : 01st Oct 2013                                                                                   */  
/* Description: PRB5519 Census are not showing exams results if student on two courses with same QANS           */ 
/*--------------------------------------------------------------------------------------------------------------*/
/* Modified by	: Arun Sutar																														*/
/* Date			: 30/09/2014																														*/
/* Details		: PRB8550 - fixed - The students who completed last year should show the QAN which was active for */
/*					  their completion date in Learning Aim Panel																				*/
/*--------------------------------------------------------------------------------------------------------------*/
-- Modified by	: Arun Sutar
-- Date			: 11-Nov-2014
-- Details		: PRB8978 - Fixed - Re-structure code to improve the performance while opening any courses in course manager.
/*--------------------------------------------------------------------------------------------------------------*/
-- Modified by	: Arun Sutar
-- Date			: 16/11/2014
-- Details		: PRB8731 - Edi result not pulling in course manager when QAN is not part of performance measure.
/*--------------------------------------------------------------------------------------------------------------*/
-- Modified By: Arun Sutar
-- Date		  : 28/09/2015
-- Details    : PRB11360 - Fixed - To display Exam QAN/Discount Code in AO QAN/Disc in membership panel on Course 
--              Manager. 
/*--------------------------------------------------------------------------------------------------------------*/
 
set nocount on    
-----------------------------------------------------------------------------------------------------------------    
-- MAIN PROCEDURE BODY GOES HERE!    
    
BEGIN    
declare  @result_category_id integer 
 -- Temporary table to store the seasons 
	  create table #seasons 
		( 
		   index1                    integer identity not null,--Idenity column 
		   season_id                 integer,-- stores season id 
		   season_start_date         datetime,-- stores season start date 
           season_end_date           datetime,-- stores season end date 
           embargo_id int
		) 
    
  -- Temporary table to store the best results only of the candidates.
  create table #best_cand_grp(
    person_id  integer,
    course_id  integer,
    subject_code varchar(20) collate database_default,
    points float
  )
   
   -- Temporary table to store the candidate results with same outcome
  create table #duplicates(
    person_id  integer,
    subject_code varchar(20) collate database_default,
    points float,
    season_id integer)
    
    create table #duplicateQans(
    person_id  integer,
    QAN varchar(20) collate database_default,
    points float
   )
   -- Created temp tables to improve performance 
   create table #result_category_link (		aspect_id		int 
													,	category_id		int
												  )	

  
   create table #temp_results			(		student_id		int 
												   ,	resultset_id		int 
												   ,	aspect_id		int
												   ,	result				varchar(1000)  collate database_default
												   ,	gradeset_id		int
												)  
    declare @descountedFlag bit
    
    select @descountedFlag=default_discount_results  from sims.course_manager_setting 
    
      select @result_category_id = ec.category_id 
		from   sims.asm_category ec 
         inner join sims.asm_category_type ect 
           on ec.category_type_id = ect.category_type_id 
              and ect.title = 'Process' 
              and ec.category_name = 'Result' 
         inner join sims.sims_agency sea 
           on ect.module_id = sea.agency_id 
              and sea.system = 'T' 

  insert into #result_category_link  
						(	aspect_id
						,	category_id )
		select			aspect_id
						,	category_id 
		from		sims.asm_aspect_category_link 
		where	category_id = @result_category_id
              
    -- Insert EDI season(s) 
  insert into #seasons 
  select distinct seas.season_id,
				  svei.event_start, 
                  svei.event_end,
                  ere.resultset_id 
  from   sims.exam_season seas 
         inner join sims.sims_event_instance svei 
           on seas.event_instance_id = svei.event_instance_id 
         inner join sims.sims_event sve 
           on svei.event_id = sve.event_id 
         inner join sims.exam_season_pattern vsp 
           on sve.event_id = vsp.event_id 
         inner join sims.exam_lookup lku 
           on lku.code = seas.season_status 
         inner join sims.asm_result ser 
           on seas.resultset_id = ser.resultset_id 		
         inner join sims.asm_aspect_category_link acl 
           on ser.aspect_id = acl.aspect_id 
         inner join sims.asm_category ec 
           on ec.category_id = acl.category_id 
              and ec.category_name = 'Result' --'Result' 
         inner join sims.asm_category_type ect 
           on ec.category_type_id = ect.category_type_id 
              and ect.title = 'Process' 
         inner join sims.sims_agency sea 
           on ect.module_id = sea.agency_id 
              and sea.system = 'T' 
         left join sims.exam_result_embargo ere 
           on ere.resultset_id = seas.resultset_id 
              and ere.start_date <= getdate() 
              and ere.end_date >= getdate() 
         left join sims.exam_result_embargo_exception eree 
           on eree.result_embargo_id = ere.result_embargo_id 
  where  lku.tablename = 'exam_season' 
         and lku.columnname = 'season_status' 
         and ( ( ere.result_embargo_id is not null 
                 and ( isnull(ec.category_name, 'OK') not in ( 'Result', 'Equivalent', 'Endorsement' ) ) ) 
                or ere.result_embargo_id is null ) 
            and    ((svei.event_start>=@start_date and  svei.event_end <=@end_date)
            or (svei.event_start<=@start_date and  svei.event_end >=@start_date)
				)
				
   -- Insert Non EDI season(s) 
	
	  insert into #seasons 
 
	select distinct seas.season_id,
				  svei.event_start, 
                  svei.event_end,
                  ere.resultset_id 
  from   sims.exam_season seas 
         inner join sims.sims_event_instance svei 
           on seas.event_instance_id = svei.event_instance_id 
         inner join sims.sims_event sve 
           on svei.event_id = sve.event_id 
         inner join sims.exam_season_pattern vsp 
           on sve.event_id = vsp.event_id 
         inner join sims.exam_lookup lku 
           on lku.code = seas.season_status 
         inner join sims.exam_pi_result ser 
           on seas.resultset_id = ser.resultset_id          
         left join sims.exam_result_embargo ere 
           on ere.resultset_id = seas.resultset_id 
              and ere.start_date <= getdate() 
              and ere.end_date >= getdate() 
         left join sims.exam_result_embargo_exception eree 
           on eree.result_embargo_id = ere.result_embargo_id 
  where  lku.tablename = 'exam_season' 
         and lku.columnname = 'season_status' 
         and ( ( ere.result_embargo_id is not null 
                  ) 
                or ere.result_embargo_id is null ) 
            and    ((svei.event_start>=@start_date and  svei.event_end <=@end_date)
            or (svei.event_start<=@start_date and  svei.event_end >=@start_date)
				)
  and seas.season_id not in(select season_id from #seasons)


insert into #temp_results (		res.student_id
									,	evs.resultset_id 
									,	aa.aspect_id
									,	res.result	
									,	aa.gradeset_id	) 

		select						res.student_id
									,	evs.resultset_id 
									,	aa.aspect_id	
									,	res.result
									,	aa.gradeset_id 
		from	sims.asm_result res 
		   inner join sims.exam_via_season evs 
				on res.resultset_id = evs.resultset_id
			inner join #seasons ss 
				on ss.season_id = evs.season_id 
			inner join sims.asm_aspect aa 
				on res.aspect_id = aa.aspect_id
			inner join #result_category_link acl 
				on aa.aspect_id = acl.aspect_id  
            
	 -- Query to get all the data for the result broadsheet report 
if not exists(select 1 from #seasons where embargo_id is not null)
	 begin 
  insert into #results 
  select distinct [season_id]   = ses.season_id,
		 [course_id]   = mR.course_id,
		 [season_start_date]=s.season_start_date,
		 [season_end_date]=s.season_end_date,
         [person_id] = mR.person_id,
         [subject_code] = csc.code,
         [subject_name] = csc.title,
         [qualification] = seq.jc_qualification_code, 
         [level] = sel.jc_level_code,   
         [season_name] = (select esp.season_pattern_code 
                          from   sims.sims_event e 
                                 join sims.sims_event_instance ei 
                                   on e.event_id = ei.event_id 
                                 join sims.exam_season_pattern as esp 
                                   on esp.event_id = e.event_id 
                          where  ei.event_instance_id = ses.event_instance_id) + ' ' + ses.calendar_year, 
         [result] = sr.result,
         [award_code]=ea.award_code,
         [entry_code]=vae.element_code,
         [board_abbreviation]=evb.board_abbreviation,
         [qan]=vae.qual_accr_number,
         [points] = isnull(epv.points,0)
  from   sims.exam_season ses 
         inner join sims.exam_series es 
           on ses.season_id = es.season_id 
            join sims.exam_vit_exam_board evb
            on es.board_id=evb.board_id
         inner join #seasons s 
           on ses.season_id = s.season_id 
         inner join sims.exam_element see 
           on es.series_id = see.series_id 
         inner join sims.exam_base_element ebe 
           on ebe.assessment_id = see.assessment_id 
              and ebe.element_type = 'C' 
         inner join sims.curr_subject_code csc 
           on ebe.qca_code_id = csc.subject_code_id 
         inner join sims.exam_level sel 
           on ebe.level_id = sel.level_id 
            inner join sims.exam_qualification seq 
           on sel.qualification_id = seq.qualification_id 
         inner join sims.exam_assessment_aspect esa 
           on see.assessment_id = esa.assessment_id 
              and es.series_id = esa.series_id 
            join sims.exam_vit_assessment_element vae
			on es.series_id = vae.series_id  and see.assessment_id = vae.assessment_id 
			join sims.exam_vit_award_element evea
			on vae.element_id=evea.element_id
        join  sims.exam_award_history evh
      on  evea.assessment_id=evh.assessment_id
      join sims.exam_award ea
      on evh.assessment_id=ea.assessment_id
		 inner join #temp_results sr on esa.aspect_id = sr.aspect_id 
              and sr.resultset_id = ses.resultset_id 
         inner join #members mR 
           on mR.person_id = sr.student_id  and evh.course_id=mR.course_id        		                    
			join sims.asm_gradeset             ags on ags.gradeset_id = sr.gradeset_id
			left join (sims.exam_pi_value epv join sims.exam_grade_map egm on   epv.grade = egm.a2c_grade)
			on  vae.qual_accr_number = epv.qan	
			and egm.edi_grade = sr.result	
			and egm.gradeset_code = rtrim(substring(ags.default_name, 4, 2))

			
 
 --inserting non EDI results
 insert into #results 
  select distinct [season_id]   = ses.season_id,
		 [course_id]   = mR.course_id,
		 [season_start_date]=s.season_start_date,
		 [season_end_date]=s.season_end_date,
         [person_id] = mR.person_id,
         [subject_code] = csc.code,
         [subject_name] = csc.title,
         [qualification] = seq.jc_qualification_code, 
         [level] = sel.jc_level_code,   
         [season_name] = (select esp.season_pattern_code 
                          from   sims.sims_event e 
                                 join sims.sims_event_instance ei 
                                   on e.event_id = ei.event_id 
                                 join sims.exam_season_pattern as esp 
                                   on esp.event_id = e.event_id 
                          where  ei.event_instance_id = ses.event_instance_id) + ' ' + ses.calendar_year, 
         [result] = ep.result,
         [award_code]='',
         [entry_code]='',
         [board_abbreviation]=eb.board_abbreviation,
         [qan]=ep.QAN,
         [points] = epv.points
  from    #seasons s
		join sims.exam_season ses 
		on ses.season_id=s.season_id
         inner join sims.exam_series es 
           on ses.season_id = es.season_id 
         join sims.exam_pi_result ep
         on ep.season_id=s.season_id
         inner join sims.curr_subject_code csc 
           on ep.subject_code_id = csc.subject_code_id 
         inner join sims.exam_level sel 
           on ep.level_id = sel.level_id 
            inner join sims.exam_qualification seq 
           on sel.qualification_id = seq.qualification_id 
           inner join sims.exam_board eb
           on ep.board_id=eb.board_id
         inner join #members mR 
           on mR.person_id = EP.person_id  and ep.course_id=mR.course_id
         left join   sims.exam_pi_value epv on ep.QAN = epv.qan
        and epv.grade = ep.result
          
 end
 
 -- GED: TFS 47169 - map any new subject (discount) codes to current ones
 if @do_mapping = 1
 begin
     update #results 
        set subject_code = sims.census_fnc_map_discount_code(qan, subject_code,YEAR(season_start_date))
     
     -- if the mapping did not work, the subject_code will be set to null, in which case we need to use the QWS QAN and Discount code (from the course)
     update #results
        set qan = left(cc.code,8)
          , subject_code = substring(cc.code,10,4)
       from #results r
       join sims.curr_course_classification cc
         on cc.curr_course_classification_id = (select top 1 csc.curr_course_classification_id
                                                  from sims.curr_course_classification_link cccl
	                                              join sims.curr_course_classification csc 
	                                                on cccl.curr_course_classification_id = csc.curr_course_classification_id	
	                                              join sims.curr_course_classification_type sct
	                                                on sct.curr_course_classification_type_id = csc.curr_course_classification_type_id
                                                 where cccl.course_id = r.course_id
                                                   and isnull(cccl.start_date, @start_date) <= @end_date
                                                   and isnull(cccl.end_date, @end_date) >= @start_date
                                                   and sct.title='QAN'
												   and isnull(cccl.start_date, @start_date) <= r.season_end_date
												   and isnull(cccl.end_date, @end_date) >=  r.season_start_date
                                              order by cccl.start_date desc) -- get most recent one if there is more than one   
      where subject_code is null
  end
  
      -- Insert the the bests results for each candidate per subject
    insert into #best_cand_grp
    select person_id,
			course_id,
           subject_code,
           MAX(points)
      from #results
  group by person_id,
			course_id,
           subject_code
      
     -- Get only the entries of best results
      insert into #best_results (season_id,
								course_id,
                                  person_id, 
                                  subject_code,
                                  subject_name,
                                  qualification,
                                  level,
                                  result,
                                  original_result,
                                  season_name,
                                  season_start_date,
                                  season_end_date,
                                  award_code,
                                  entry_code,
                                  board_abbreviation,
                                  qan,
                                  points)
      select  distinct season_id,
			  r.course_id,
              r.person_id, 
              r.subject_code, 
              r.subject_name,
              r.qualification,
              r.level,
              result,
              result,
              r.season_name,
              r.season_start_date,
              r.season_end_date,
              r.award_code,
              r.entry_code,
              r.board_abbreviation,
              r.qan,
              r.points
        from  #results as r 
              inner join #best_cand_grp as bcg
           on r.person_id = bcg.person_id
           and r.course_id=bcg.course_id
          and r.subject_code = bcg.subject_code
          and r.points = bcg.points   
        
      
     insert into #duplicateQans
    select person_id,
           qan,
           max(points)
      from #best_results 
  group by person_id,
           qan
    having count(*) > 1
  
--PRB5519 commented below check, census is not showing exam results for duplicate QANS 
/*    
      delete r
  from #best_results as r
       inner join #duplicateQans as d
    on r.person_id = d.person_id
   and r.qan = d.qan
   and r.points <> d.points
*/   
    insert into #best_results (season_id,
								course_id,
                                  person_id, 
                                  subject_code,
                                  subject_name,
                                  qualification,
                                  level,
                                  result,
                                  original_result,
                                  season_name,
                                  season_start_date,
                                  season_end_date,
                                  award_code,
                                  entry_code,
                                  board_abbreviation,
                                  qan,
                                  points)
      select  distinct season_id,
			  r.course_id,
              r.person_id, 
              r.subject_code, 
              r.subject_name,
              r.qualification,
              r.level,
              result,
              result,
              r.season_name,
              r.season_start_date,
              r.season_end_date,
              r.award_code,
              r.entry_code,
              r.board_abbreviation,
              r.qan,
              r.points
        from  #results as r 
          where r.result='Q'
         --update result=? if QAN and discount code confilicts 
       update  b
       set b.result='?'
       from #best_results  b
       inner join #results r
       on b.person_id=r.person_id
       and b.qan=r.qan
       where exists (select 1 from #results r1 where r1.person_id=r.person_id and r1.qan<>r.qan and r1.subject_code<>r.subject_code and r1.course_id=r.course_id)
      
       --update result=? if more than 1 different results with same points
        update  b
       set b.result='?'
       from #best_results  b
       inner join #results r
       on b.person_id=r.person_id
       and b.qan=r.qan
       where exists (select 1 from #results r1 where r1.person_id=r.person_id and r1.result<>r.result and r1.course_id=r.course_id and isnull(r1.points,0)=isnull(r.points,0))
    
		-- update result =? if diff QAN+Level with same subject    
	   if @descountedFlag=0 
	   begin	   
	   update  b   
       set b.result='?'  
       from #best_results  b  
       inner join #results r  
       on b.person_id=r.person_id  
       and b.qan=r.qan  
       where exists (select 1 from #results r1 where r1.person_id=r.person_id and r1.qan<>r.qan and r1.subject_code=r.subject_code and r1.course_id=r.course_id
					and r1.level <> r.level)
	   end
			
       --update result=? if no QAN
       update  b
       set b.result='?'
       from #best_results b where ISNULL(b.qan,'')=''
       
 --update for Q
       update  b
       set b.result='Q'
       from #best_results  b
       inner join #results r
       on b.person_id=r.person_id
       and b.qan=r.qan
       and b.result is null and r.result='Q'
       if @descountedFlag=1
       begin
     -- There could be scenarios where students end up with same grade in same subject again in different seasons.
    -- In that case, only consider the one in latest season.
    insert into #duplicates
    select person_id,
           subject_code,
           points,
           max(season_id)
      from #best_results 
  group by person_id,
           subject_code,
           points
    having count(*) > 1
	
  -- delete the duplicates entries once identified
  delete r
  from #best_results as r
       inner join #duplicates as d
    on r.person_id = d.person_id
   and r.subject_code = d.subject_code
   and r.points = d.points
   and r.season_id <> d.season_id -- except the one from latest season   
   end
   drop table #seasons    
   drop table #best_cand_grp    
   drop table #result_category_link
   drop table #temp_results
END

    
-- END OF PROCEDURE BODY!    
-----------------------------------------------------------    
return 0 -- return value if all OK.    
    
-----------------------------------------------------------
-- START OF UNIT TEST SECTION


---------------------------------------------
-- END OF UNIT TEST SECTION
-----------------------------------------------------------






-- END PATCH SCRIPT 11
GO


-- START PATCH SCRIPT 12

/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2005                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : School Census                                                                                   */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2010                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED                                                                                      */
/*--------------------------------------------------------------------------------------------------------------*/
/* Modified By: Sooraj V                                                                                        */
/* Modified   : 15/08/2013                                                                                      */
/* Detail     : PRB5245 Error occurs when the school have a reg group short name of greater than 8 characters   */
/*              even though SIMS allows a maximum of 10 characters                                              */
/*																												*/
/* Modified By: Sandeep Kharmate                                                                                */
/* Modified   : 05/04/2014                                                                                      */
/* Detail     : Story 51496-Calculate All  option to NOT use results.											*/
/*																												*/
/* Modified By: Sandeep Kharmate                                                                                */
/* Modified   : 14/04/2014                                                                                      */
/* Detail     : Story 51852-Add Core Aim column.																*/
/*--------------------------------------------------------------------------------------------------------------*/
set nocount on
go


exec sims.db_p_drop 'sims.census_pix_census_GetLearningAimsQans','P'
go

alter procedure sims.census_pix_census_GetLearningAimsQans
(
           
	 @census_date datetime,
	 @previousReturnID int = 0,
     @currentReturnID  int = 0,
     @overwrite_edits  int = 0,
     @flagSourceforPlannedEndDate bit,  
     @useExamResultToDetermineQANDiscount bit  
)
as
/*------------------------------------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL																												*/
/* Object     : sims.census_pix_census_GetLearningAimsQans   																		        */
/*																																			*/
/* Purpose    : This procedure gets the Learning Aims Qans																					*/
/* Comments   :																																*/
/*																																			*/
/*                                                                                                                                          */
/* Author     : Sooraj M.V																													*/
/* Date       : 10th Nov 2010                                                                                                               */
/* Modified By: Sooraj M.V                                                                                                                  */
/* Date       : 04/10/2012                                                                                                                  */
/* Detail     : PRB2467 - Leavers not appearing in Learning aims panel                                                                      */
/* Modified By: Jahir                                                                                                                       */
/* Date       :19/10/2012                                                                                                                   */
/* Detail     : PRB2589 Performance issue when calculating Census Autumn Return.                                                            */ 
/* Modified By: Mohanish  S.                                                                                                                */
/* Date       : 11/03/2015                                                                                                                  */
/* Detail     : 68232 Record Learning Aims information   Edit/Add/Hide.                                                                     */
/* Modified By: Jahir P                                                                                                                     */
/* Modified   : 05/10/2015                                                                                                                  */
/* Detail     : PRB11605 : Schools using Year taught in 12, 13,14 in previous acdemic year and not using in                                 */
/*              current academic year in this scenario  Learning aims data is not calculating  for the students                             */
/*				who was in  year taught in  12 and above in the previous academic year                                                      */ 
/*------------------------------------------------------------------------------------------------------------------------------------------*/


set nocount on

--Variable for filtering LAs 
declare @start_date datetime
declare @end_date	datetime
declare @summer_census_date	datetime
declare @prev_acad_start_date datetime
declare @prev_acad_end_date datetime 
declare @acad_start_date datetime
declare @acad_end_date datetime
declare @defualt_course_manager_normal_duration_code varchar(10)
declare @ENDOFTIME smalldatetime  
declare @STARTTIME smalldatetime  
declare @home_school_ukprn_id int 
set @ENDOFTIME = '20790606'
set @STARTTIME='19000101'


--- declare local variables
--
declare @idoc int
declare @completion_status_id_continuing int
declare @completion_status_id_completed int


DECLARE @AlternDeptId int
SELECT @AlternDeptId = department_id FROM sims.curr_department WHERE name = 'Alternative Curriculum'

select @completion_status_id_continuing=completion_status_id from sims.curr_completion_status where description='Continuing'
select @completion_status_id_completed=completion_status_id from sims.curr_completion_status where description='Completed'

--Select ukprn number for current school 
SELECT @home_school_ukprn_id=cur.curr_course_classification_id
FROM sims.sims_School sch
INNER JOIN sims.curr_course_classification cur on sch.ukprn_number=cur.code
WHERE sch.STATUS='S' and cur.active_status='A'

--temporary table for pupil
 
declare @tblPupil table 
(
            person_id         int primary key 
)

declare @tblCoreAimAcadYearID table 
(
            acad_year_id      int primary key 
)


select  
		@start_date =spct.learning_activities_collected_from ,
	    @end_date =  spct.learning_activities_collected_to  
from    sims.plasc_census_term spct
where   spct.code='AUT'

--PRB2589
DECLARE @maxDate SMALLDATETIME    
SELECT  @maxDate='20790606'

declare @basegroup_ncyear as table
(
 base_group_id int
,code varchar(10)
)
insert into @basegroup_ncyear (base_group_id,code)
select grp.base_group_id,grp.code from
sims.sims_base_group grp   
inner join sims.sims_base_group_type bgt  ON grp.base_group_type_id = bgt.base_group_type_id      
and bgt.code = 'NCYear' 
--PRB2589

--Insert Acad years available: the year of the census date and the previous year.
insert into @tblCoreAimAcadYearID (acad_year_id)
select tei.event_instance_id
  from sims.sims_event_instance tei
  join sims.sims_event te
    on te.event_id = tei.event_id
  join sims.sims_event_type tet
    on tet.event_type_id = te.event_type_id
   and tet.code = 'AcadYear'
 where tei.event_start <= @census_date
   and tei.event_end >= dateadd(year, -1, @census_date)

--
--- main process
 
--Create an internal representation of the XML document.  
insert into @tblPupil
(
            person_id
)

Select distinct per.person_id
From    sims.sims_person per
Join    sims.stud_school_history ssh
on      ssh.person_id=per.person_id
and     isnull(ssh.start_date,'1900-01-01 00:00:00') <= @census_date 
and     isnull(ssh.end_date, @start_date) >= @start_date
Join    sims.sims_school ss 
on      ss.school_id=ssh.school_id 
and     ss.status='S'
Join    sims.stud_on_roll_mode sorm 
on      ssh.on_roll_mode_id = sorm.on_roll_mode_id
and     sorm.external_code in ('C','M')
--getting term 'SPR' or 'SUM' or 'AUT' 
--Learning Aim collection Start date
--Leatning Aim collection End date


 
-- set summer census Date
 
select @summer_census_date=census_date from sims.plasc_census_term where code='SUM'
create table #courses
(
course_id int,
courseDurationCode int
)
--temporary table for learning aim memberships
create table #members
(
  person_id int,
  course_id int,
  qan varchar(8) collate database_default,
  disccode varchar(4) collate database_default,
  start_date smalldatetime,
  actual_end smalldatetime,
  planned_end smalldatetime,
  completion_status varchar(250) collate database_default,
  completion_status_id char(1) collate database_default,
  withdrawal_reason_id int,
  ukprn_classification_id int
)

create table #previousYearMembers
(
  person_id int,
  course_id int,
  qan varchar(8) collate database_default,
  start_date smalldatetime,
  actual_end smalldatetime,
  planned_end smalldatetime,
  completion_status varchar(250) collate database_default,
)
-- Temporary table to store the results details 
  create table #results
    ( 
       season_id     integer, 
        course_id     integer, 
       season_start_date         datetime,-- stores season start date 
       season_end_date           datetime,-- stores season end date   
       person_id     integer, 
       subject_code  varchar(25) collate database_default, 
       subject_name  varchar(200) collate database_default, 
       qualification varchar(150) collate database_default, 
       level         varchar(50) collate database_default, 
       season_name   varchar(150) collate database_default, 
       result        varchar(1000) collate database_default,
       award_code   varchar(32)  collate database_default,
       entry_code varchar(12)  collate database_default,
	   board_abbreviation   varchar(10)  collate database_default,
	   qan varchar(8)  collate database_default,
       points            float,
    ) 
  
  -- Temporary table to store all the best results from the result table
  create table #best_results(
    season_id         integer,
     course_id     integer, 
    season_start_date         datetime,-- stores season start date 
    season_end_date           datetime,-- stores season end date   
    person_id         integer, 
    subject_code      varchar(20) collate database_default, 
    result            varchar(1000) collate database_default, 
    original_result            varchar(1000) collate database_default, 
    subject_name	  varchar(200) collate database_default, 
    qualification	  varchar(150) collate database_default, 
    level             varchar(50) collate database_default, 
    season_name       varchar(150) collate database_default,
    award_code   varchar(32)  collate database_default,
    entry_code varchar(12)  collate database_default,
    board_abbreviation   varchar(10)  collate database_default,
	qan varchar(8)  collate database_default, 
    points            float
  )
-- Declare temp table for storing student's course detail current
create TABLE #pupil_course_detail_for_current_return   
(  
     person_id INT	--person_id
    ,course_id INT	 --course_id						--  
    ,surname   VARCHAR(100) 		COLLATE DATABASE_DEFAULT NULL
    ,forename VARCHAR(100) 		COLLATE DATABASE_DEFAULT NULL
    ,nc_year  VARCHAR(10)  COLLATE DATABASE_DEFAULT NULL
    ,qan VARCHAR(50)COLLATE DATABASE_DEFAULT NULL
    ,discount_code VARCHAR(50)COLLATE DATABASE_DEFAULT NULL
    ,qan_description VARCHAR(200)COLLATE DATABASE_DEFAULT NULL
	,start_date SMALLDATETIME		    -- Course start date
	,actual_end_date SMALLDATETIME				-- Course END date
	,planned_end_date SMALLDATETIME		-- Course planned END date    
	,completion_status VARCHAR(200) COLLATE DATABASE_DEFAULT NULL	-- Course completion status	
	,completion_status_id INT NULL	-- Course completion status	Id
	,result varchar(1000) collate database_default
    ,outcome int
    ,exam_qan varchar(20) collate database_default
	,withdrawal_reason_id INT
	,ukprn_classification_id INT
    ,user_original_qan varchar(20) collate database_default
    ,user_original_discount_code VARCHAR(50)COLLATE DATABASE_DEFAULT NULL
    ,user_original_start_date smalldatetime null
    ,user_original_actual_end  smalldatetime null
    ,user_original_planned_end   smalldatetime null
    ,user_original_completion_status_id int null
    ,user_added smallint default 0
    ,deleted smallint default 0 
    ,prev_return_planned_end smalldatetime
    ,core_aim varchar(1)
    ,user_original_core_aim varchar(1)
	,user_original_withdrawal_reason_id INT
	,user_original_ukprn_classification_id INT
	) 


	--populating Acad start date for Autumn term
	Select @acad_start_date = eventInst.event_start, @acad_end_date =  eventInst.event_end 
	from sims.sims_event_instance eventInst    
	inner join sims.sims_event event on event.event_id = eventInst.event_id    
	inner join sims.sims_event_type eventType on eventType.event_type_id = event.event_type_id and eventType.code = 'AcadYear'    
	where @census_date between eventInst.event_start and eventInst.event_end 

--selecting previous Academic Year
Select @prev_acad_start_date = eventInst.event_start 
      ,@prev_acad_end_date = eventInst.event_end
	from sims.sims_event_instance eventInst    
	inner join sims.sims_event event on event.event_id = eventInst.event_id    
	inner join sims.sims_event_type eventType on eventType.event_type_id = event.event_type_id and eventType.code = 'AcadYear'    
	where @acad_start_date-1 between eventInst.event_start and eventInst.event_end 
-- Get the QAN and the student aims data for the pupils passed

declare @previousAcademicYear varchar(10)
declare @shouldLAcalculate int
set @shouldLAcalculate=@previousReturnID
set @previousAcademicYear=YEAR(@census_date)-1
 if not exists(	select 1
				from sims.plasc_return pr
				inner join sims.plasc_census_term pct
				on pr.census_term_id = pct.census_term_id
				where pct.code = 'AUT'
				AND pr.academic_year =SUBSTRING( @previousAcademicYear,3,2)
				and pr.authorised_date is not null)
	begin
			set @shouldLAcalculate=1
    end

select  @defualt_course_manager_normal_duration_code=code from sims.course_manager_duration where description='One Year' 

--PRB11605 Check whether the School having Year taughts 12 ,13,14 for previous acdemic year.
if @shouldlacalculate>=0 and (exists(select 1 from sims.sims_base_group_set_link bsl
                                        join sims.sims_base_group_set bs on bs.base_group_set_id = bsl.base_group_set_id
										join sims.sims_base_group_type bt on bt.base_group_type_id = bs.base_group_type_id
										join sims.sims_base_group bg on bg.base_group_id = bsl.base_group_id and bg.base_group_type_id = bt.base_group_type_id
										where bt.code ='ncyear' and bg.code in('12','13','14')
										and isnull(bs.start_date , '19000101') <= @prev_acad_end_date
										and isnull(bs.end_date , '20790606') >= @prev_acad_start_date
        							)
    							or

							  exists(select 1 from sims.sims_school where status = 's' and top_ncyear in ('12','13','14')) -- check whether the school having 6th form
                             )
Begin

if exists (select 1 from sims.course_manager_setting)
	select  @defualt_course_manager_normal_duration_code=isnull(cmd.code,@defualt_course_manager_normal_duration_code) from 
	sims.course_manager_setting cms 
	join sims.course_manager_duration cmd
	on cms.course_manager_normal_duration_id= cmd.id
else
	select  @defualt_course_manager_normal_duration_code=code from sims.course_manager_duration where description='One Year' 


insert into #courses(course_id,courseDurationCode)
select grpCourse.base_group_id,cast (isnull(cmd.code,ISNULL(cmd1.code,@defualt_course_manager_normal_duration_code)) as INT) from sims.sims_base_group grpCourse			   
inner join sims.sims_base_group_type typeCourse		on grpCourse.base_group_type_id = typeCourse.base_group_type_id  
                        													and typeCourse.code = 'Course'  
inner join sims.curr_course cc                    on cc.base_group_id = grpCourse.base_group_id
													                        and cc.department_id <> @AlternDeptId
left join --course setting
(sims.course_manager_setting_course_config cmscc
join sims.course_manager_duration cmd
on cmd.id=cmscc.normal_duration_id
)
on 	grpCourse.base_group_id=cmscc.course_id	
left join --level setting
( 
sims.curr_course_classification_link cccl
join sims.curr_course_classification ccc 
on  cccl.curr_course_classification_id=ccc.curr_course_classification_id
inner join sims.curr_course_classification_type ct  on ct.curr_course_classification_type_id  = ccc.curr_course_classification_type_id  
and ct.title = 'Level'  and ccc.code!='Altern'  
inner join sims.course_manager_setting_level_config config
on ccc.curr_course_classification_id=config.level_id
join sims.course_manager_duration cmd1
on cmd1.id=config.normal_duration_id
)     
on 	grpCourse.base_group_id=cccl.course_id		
left join sims.course_manager_setting ccs
on ccs.setting_id>0										                        
where grpCourse.active_state<>'O'--We do not consider obsolute courses



insert into #members 
(
  person_id,
  course_id,
  qan,
  disccode,
  start_date,
  actual_end,
  planned_end,
  completion_status,
  completion_status_id,
  withdrawal_reason_id,
  ukprn_classification_id
)
select distinct
            [Person_Id]             =		p.person_id,
			[course_id]				=		c.course_id,
            [QAN]                   =		left(ccc.code, 8),
			[DiscCode]              =		substring(ccc.code, 10, 4),
            [Start_Date]			=		case when p16.person_id is null then sm.start_date-- no explicit record exists in p16_student_aim table
													  else -- explicit record exists in p16_student_aim
                                                        
																  case when ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then sm.start_date
																			when sm.end_date >= @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then sm.start_date -- <CB> Couldn't this be combined with the above?
                                                                            when sm.end_date < @census_date and ( ccs.description is null ) then sm.start_date
                                                                            when sm.end_date >= @census_date and ( ccs.description is null ) then sm.start_date
                                                                  end
                                                            end
                                                      ,
 
            [actual_end]			=         case when p16.person_id is null then -- no explicit record exists in p16_student_aim table
                                                    case when sm.end_date > @census_date then null -- <CB> This should be >, not >=, because memberships end at 23:59, so on the day of the census it hasn't finished.
                                                         else sm.end_date
                                                         end
                                                    else -- explicit record exists in p16_student_aim table
                                                         case 
                                                              when sm.end_date < @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then sm.end_date
                                                              when sm.end_date >= @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then null
                                                              when sm.end_date < @census_date and ( ccs.description is null ) then sm.end_date
															  when sm.end_date >= @census_date and ( ccs.description is null ) then null--Sooraj M.V  If End Date in Course Manager (the membership end date) is after the Census Date and not re-sit then null.
                                                         end
                                                     end,
            [Planned_End]			=          case when p16.person_id is null then sm.end_date -- No explicit record exists in p16_student_aim
                                                    else  -- explicit record exists in p16_student_aim
														case 
															
                                                             when sm.end_date < @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then p16.planned_end
															 when sm.end_date >= @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then p16.planned_end --<CB> Could be combined with above - end date is irrelevant
                                                             when sm.end_date < @census_date and ( ccs.description is null ) then p16.planned_end
															 when sm.end_date >= @census_date and ( ccs.description is null ) then p16.planned_end --<CB> Could be combined with above - end date is irrelevant
                                                         end
                                                    end,
            
            [Completion_Status]     =			case when p16.person_id is null then -- no explicit record exists in p16_student_aim table
                                                     case when sm.end_date >= @census_date then 'Continuing'
                                                          else 'Completed'
                                                          end
                                                          else -- explicit record exists in p16_student_aim table
                                                          case 
                                                               when sm.end_date < @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then ccs.description
															   when sm.end_date < @census_date and ( sm.end_date=p16.planned_end) and ( ccs.description is null ) then 'Completed'
															   when sm.end_date >= @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then 'Continuing'
															   when sm.end_date < @census_date and sm.end_date < p16.planned_end and ( ccs.description is null ) then null -- <CB> This will, correctly, trigger an error, but we must allow them to fill in planned completion manually in CM.
															   when sm.end_date >= @census_date and ( ccs.description is null ) then 'Continuing'--Sooraj M.V  If End Date in Course Manager (the membership end date) is after the Census Date and not re-sit then Continuing.
                                                          end
                                                      end,
		   [Completion_Status_Id]    =		     case when p16.person_id is null then -- no explicit record exists in p16_student_aim table
                                                      case when sm.end_date > @census_date then @completion_status_id_continuing --'Continuing'
                                                           else @completion_status_id_completed --'Completed'
                                                           end
                                                           else -- explicit record exists in p16_student_aim table
                                                           case 
                                                                when sm.end_date < @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then ccs.completion_status_id
                                                                when sm.end_date < @census_date and ( sm.end_date=p16.planned_end) and ( ccs.description is null ) then @completion_status_id_completed
                                                                when sm.end_date >= @census_date and ( ccs.description = 'Transferred' or ccs.description = 'Withdrawn' ) then @completion_status_id_continuing
																when sm.end_date < @census_date and sm.end_date < p16.planned_end and ( ccs.description is null ) then null   -- <CB> This will, correctly, trigger an error, but we must allow them to fill in planned completion manually in CM.
																when sm.end_date >= @census_date and ( ccs.description is null ) then @completion_status_id_continuing--Sooraj M.V  If End Date in Course Manager (the membership end date) is after the Census Date and not re-sit then 1.
                                                           end
                                
                                                       end,
		withdrawal_reason_id       =			case when (sm.end_date is not null and sm.end_date >= @acad_start_date and sm.end_date <= @acad_end_date and sm.end_date < @census_date ) 
															then p16.withdrawal_reason_id
															else NULL
												end,
		ukprn_classification_id    =				   p16.ukprn_classification_id
    
from @tblPupil p
inner join sims.sims_member sm on sm.person_id = p.person_id 
	
inner join #courses c			    on c.course_id = sm.base_group_id  												                      												 
left join sims.curr_p16_student_aim p16				on p16.person_id = p.person_id and sm.base_group_id = p16.course_id and sm.start_date=p16.start_date
left join sims.curr_completion_status ccs			on ccs.completion_status_id = p16.completion_status_id
left join 
(sims.curr_course_classification_link cccl 
 join sims.curr_course_classification ccc		on ccc.curr_course_classification_id = cccl.curr_course_classification_id
 join sims.curr_course_classification_type ccct on ccct.curr_course_classification_type_id = ccc.curr_course_classification_type_id
													and ccct.title = 'QAN'
)
on cccl.course_id = sm.base_group_id
and
(
-- if membership ended before census date then QAN will be checked on membership end date
(isnull(cccl.start_date,@STARTTIME) <= isnull(sm.end_date,@census_date) and isnull(cccl.end_date,@ENDOFTIME) >= isnull(sm.end_date,@census_date) and isnull(sm.end_date,@census_date)<@census_date)
or
-- if membership ended after census date then QAN will be checked on census date
 (isnull(cccl.start_date,@STARTTIME) <= isnull(@census_date,@ENDOFTIME) and isnull(cccl.end_date,@ENDOFTIME) >= @census_date and isnull(sm.end_date,@census_date)>=@census_date)
 )
--PRB2589 
left join ( sims.sims_member memncyear   
join @basegroup_ncyear bgn on bgn.base_group_id = memncyear.base_group_id)
on   memncyear.person_id = p.person_id  
and  sm.start_date between memncyear.start_date and isnull(memncyear.end_date,@maxDate)
--PRB2589
													where 
	 (
			(
				sm.start_date <= @end_date and sm.end_date >= @start_date and sm.end_date <= @end_date 
				and ((MONTH(isnull(sm.end_date,@maxDate))=8  and bgn.code in ('12','13','14'))or MONTH(isnull(sm.end_date,@maxDate)) <> 8 )      --PRB2589
			) 
	or		( 
				sm.start_date < @end_date and sm.end_date > @end_date  
			)
	
	) 
and p.person_id in (select person_id from sims.census_fnt_get_on_roll_student_ids_forLA(sm.start_date, sm.end_date))
and  (ccs.description <> 'Re-sit' or ccs.description is null)
order by person_id

-- BTR25791 we have to exclude any one-day memberships which have an exception OUT record
delete m
  from #members m
  join sims.curr_course_exception cce
    on cce.course_id = m.course_id
   and cce.person_id = m.person_id
   and cce.exception_type = 'O'
    -- there is an acad year ID as well but I don't think we should check it, as it isn't really handled properly for courses over 1 year
 where datediff(day, m.start_date, m.actual_end) = 0 -- one day membership will start and end on the same day


update m
set m.ukprn_classification_id = isnull(m.ukprn_classification_id,clc.curr_course_classification_id)
from #members m left join 
(
select clink.course_id, ccc.curr_course_classification_id,clink.start_date,clink.end_date
from sims.curr_course_classification_link clink
INNER JOIN sims.curr_course_classification ccc
on ccc.curr_course_classification_id = clink.curr_course_classification_id
INNER JOIN sims.curr_course_classification_type ctype
on ctype.curr_course_classification_type_id = ccc.curr_course_classification_type_id
where ctype.title = 'Provider UKPRN' 
) clc --courses with their classification ids.
on m.course_id = clc.course_id
and 
(
 (clc.start_date is null or clc.end_date is null)
 or
 (
   (m.completion_status = 'Continuing' and @census_date between clc.start_date and clc.end_date) 
   or
   (m.completion_status != 'Continuing' and m.actual_end between clc.start_date and clc.end_date)
 )
)							


--Update ukprn_classification_id to null if it is same as home school ukprn
--update default ukprn_classification_id for a course in classification to null if its is same as home _school_ukprn 
update m
set m.ukprn_classification_id = NULL
from #members m
left join sims.curr_p16_student_aim p16				on p16.person_id = m.person_id 
left join sims.curr_course_classification_link cccl on p16.ukprn_classification_id=cccl.curr_course_classification_id
left join sims.curr_course_classification ccc		on ccc.curr_course_classification_id = cccl.curr_course_classification_id  
where m.ukprn_classification_id=@home_school_ukprn_id OR cccl.curr_course_classification_id=@home_school_ukprn_id
                                               
insert into #previousYearMembers 
(
  person_id,
  course_id,
  qan,
  start_date,
  actual_end,
  planned_end,
  completion_status
)
select clq.person_id,clq.course_id,clq.qan,clq.start_date,clq.actual_end,clq.planned_end,ccs.description from sims.census_learningAims_qans clq
join sims.curr_completion_status ccs
ON clq.completion_status_id=ccs.completion_status_id
where clq.return_id=@previousReturnID


		

--Populate the results
--for testing 
--set @start_date='2010-08-01 00:00:00'
exec sims.curr_pia_ExaminationResults_matching @start_date,@end_date

      
		
       insert into #pupil_course_detail_for_current_return(person_id,course_id,surname,forename,nc_year,qan,discount_code,qan_description,start_date,actual_end_date,planned_end_date,completion_status,completion_status_id, user_original_planned_end, core_aim, withdrawal_reason_id,ukprn_classification_id)
		select  distinct  
		  p.person_id, 
		  p.course_id,
		  isnull(per.legal_surname, per.surname),
		  per.forename,
		  grp.code,
		  p.qan,  
		  p.disccode, 
		  [QANDisc]=	p.qan + '/' + p.disccode ,
		  p.start_date ,  
		  p.actual_end,  
		  p.planned_end,    
		  p.completion_status,
		  p.completion_status_id,
		  p.planned_end, -- put the course planned end date into the original_planned_end so that it can be highlighted if different
		  case when ccla.person_id is null then '0' else '1' end,
		  p.withdrawal_reason_id,
		  p.ukprn_classification_id
		 from #members p
		-- filter condition in DUR 1.8 > section 4.4 > BR 2  - pupils in year 12 and above, regardless of their age; 
		-- and pupils in year 11 or below who are studying for a Level 3 qualification.)
		--Gaurav Kakirwar 26/02/2007
		-- Added to avoid NC Year membership record
		inner join sims.sims_person per
		 on p.person_id=per.person_id
		 join sims.stud_student stud
		on p.person_id=stud.person_id
		inner join sims.sims_member sm1 on sm1.person_id = p.person_id 
		and sm1.start_date = (select max(sm2.start_date) from sims.sims_member sm2
									 inner join sims.sims_base_group grp2 on grp2.base_group_id = sm2.base_group_id  
									 inner join sims.sims_base_group_type type2  on grp2.base_group_type_id = type2.base_group_type_id  
									 and type2.code = 'NCYear'
									 and sm2.start_date <= @census_date
									 where sm2.person_id = sm1.person_id)
		--end
		inner join sims.sims_base_group grp on grp.base_group_id = sm1.base_group_id  
		inner join sims.sims_base_group_type type  on grp.base_group_type_id = type.base_group_type_id  
		  and type.code = 'NCYear'  
		  and planned_end is not null
		left join sims.curr_completion_status ccs			on ccs.code = p.completion_status_id
		left join sims.curr_core_learning_aim ccla on ccla.person_id = p.person_id and ccla.course_id = p.course_id  and ccla.acad_year_id in (select acad_year_id from @tblCoreAimAcadYearID) 
		where
		(
					(ltrim(rtrim(grp.code))) IN ('13','14')  

			OR
		((ltrim(rtrim(grp.code))) = '12')
			--Course records of student in NC Year 12 last academic year should be populated
		and  
		(
			(  
			sims.census_fnc_check_NCYear11(@prev_acad_start_date,@acad_start_date,p.person_id)='T' 
			and (p.actual_end>= @prev_acad_start_date and p.actual_end> @acad_start_date/*@acad_start_date*/)
			)
			or 
			( 
			sims.census_fnc_check_NCYear11(@prev_acad_start_date,@acad_start_date,p.person_id)='F'
			and (p.start_date >= @start_date and p.start_date <= @end_date /*@census_date*/) 
			)
			or  (p.start_date <= @start_date and isnull(p.actual_end,@acad_start_date) >= @acad_start_date/*@acad_start_date*/)
			or ( isnull(p.actual_end,@acad_start_date)  >= @acad_start_date and isnull(p.actual_end,@end_date) <= @end_date/*@census_date*/)
			or ( p.start_date  <= @acad_start_date and isnull(p.actual_end,@end_date) >= @end_date/*@census_date*/)
			--For Resit
		)
		--						OR 
		--			( 
		--(ltrim(rtrim(grp.code))) NOT IN ('12','13','14') and p.national_qualification_framework_level in ('3','4','5','6','7','8','9') )

		)

-- TFS 16900 Remove any courses where:
--   student in YTI Y12 on census date and courses not studied in Y12
--   student in YTI Y13 on census date and courses not studied in Y12 or Y13
--   student in YTI Y14 on census date and courses not studied in Y12 or Y13 or Y14
delete cs
from #pupil_course_detail_for_current_return cs
where not exists (select 1 from sims.sims_member m
                           join sims.sims_base_group yti
                             on yti.base_group_id = m.base_group_id
                           join sims.sims_base_group_type bgt
                             on bgt.base_group_type_id = yti.base_group_type_id
                            and bgt.code = 'NCYear' 
                          where m.person_id = cs.person_id
                            and m.start_date <= isnull(cs.actual_end_date, @ENDOFTIME) 
                            and cs.start_date <= isnull(m.end_date, @ENDOFTIME)
                            and (  (cs.nc_year = '12' and yti.code = '12') 
                                or (cs.nc_year = '13' and yti.code in ('12','13') 
                                or (cs.nc_year = '14' and yti.code in ('12','13','14')) ) ) ) 

--updating result and qan
update #pupil_course_detail_for_current_return
set result = (select top 1 result from #best_results where person_id=sm.person_id and sm.course_id=course_id
 and (sm.start_date <=season_end_date and (isnull(sm.actual_end_date,@ENDOFTIME) >= season_start_date))
)
    ,exam_qan  =   (select top 1 qan from #best_results where person_id=sm.person_id  and sm.course_id=course_id
    and (sm.start_date <=season_end_date and (isnull(sm.actual_end_date,@ENDOFTIME) >= season_start_date))
    )
from #pupil_course_detail_for_current_return sm

--updating outcome
update #pupil_course_detail_for_current_return
set outcome = (select top 1 co.id from sims.curr_outcome_main m 
				join sims.course_manager_outcomes co
				on m.original_outcome=co.description
				where m.result=sm.result and qan =sm.exam_qan   
				and (isnull(start_date,@STARTTIME) <= isnull(@end_date,@ENDOFTIME) and isnull(end_date,@ENDOFTIME) >= @start_date)

               )
from #pupil_course_detail_for_current_return sm

/*
    qan column indicate QWS QAN if result is null otherwise it indicate exam qan. (same applicable for discount code)
*/
if @useExamResultToDetermineQANDiscount = 1
Begin    
	update #pupil_course_detail_for_current_return 
	   set discount_code = (select top 1 subject_code from #best_results where person_id=sm.person_id  and sm.course_id=course_id
								and (sm.start_date <=season_end_date and (isnull(sm.actual_end_date,@ENDOFTIME) >= season_start_date))
							)
		  ,qan          = exam_qan 				    
	  from #pupil_course_detail_for_current_return sm
	  where result is not null
		and ltrim(rtrim(result)) not in ('','?')
End    
					    
-- Updating planned end date from previous year
--replace the 2012 Autumn Planned End Date value with the 2011 Autumn Planned End Date value where all of the following conditions are met:
--a)        the 2011 Autumn learning aim was continuing and 
--b)       the 2011 Autumn planned end date is not in August and 
--c)        the 2012 Autumn actual end date does not exist or is on or before 2011 Autumn planned end date and  
--d)       the 2011 Autumn learning planned end date is not equal to the 2012 Autumn learning planned end date


if (@flagSourceforPlannedEndDate=1)
begin
--matching with exam QAN if exam QAN is exists
        update #pupil_course_detail_for_current_return
		set planned_end_date=pm.planned_end
		from #pupil_course_detail_for_current_return m
		inner join #previousYearMembers pm
		on m.person_id=pm.person_id and m.course_id=pm.course_id and m.exam_qan=pm.qan and m.start_date=pm.start_date
		where pm.completion_status='Continuing' and m.start_date IS NOT NULL and m.exam_qan IS NOT NULL
		and
		(( pm.planned_end IS NOT NULL  AND month(pm.planned_end)!=8 and m.actual_end_date IS NULL  and m.start_date IS NOT NULL  
		and pm.planned_end<>m.planned_end_date)
		or
		( pm.planned_end IS NOT NULL  AND month(pm.planned_end)!=8 and m.actual_end_date IS NOT NULL  and m.actual_end_date< pm.planned_end and m.start_date IS NOT NULL  
		and pm.planned_end<>m.planned_end_date)
		)
		
--matching with QWS QAN if exam QAN is not exists	
	
		update #pupil_course_detail_for_current_return
		set planned_end_date=pm.planned_end
		from #pupil_course_detail_for_current_return m
		inner join #previousYearMembers pm
		on m.person_id=pm.person_id and m.course_id=pm.course_id and m.qan=pm.qan and m.start_date=pm.start_date
		where pm.completion_status='Continuing' and m.start_date IS NOT NULL and m.exam_qan IS  NULL and m.qan IS NOT NULL
		and
		(( pm.planned_end IS NOT NULL  AND month(pm.planned_end)!=8 and m.actual_end_date IS NULL  and m.start_date IS NOT NULL  
		and pm.planned_end<>m.planned_end_date)
		or
		( pm.planned_end IS NOT NULL  AND month(pm.planned_end)!=8 and m.actual_end_date IS NOT NULL  and m.actual_end_date< pm.planned_end and m.start_date IS NOT NULL  
		and pm.planned_end<>m.planned_end_date)
		)
		
--matching with QWS QAN if exam QAN is exists and QWS QAN is not equal to exam QAN
		
		update #pupil_course_detail_for_current_return
		set planned_end_date=pm.planned_end
		from #pupil_course_detail_for_current_return m
		inner join #previousYearMembers pm
		on m.person_id=pm.person_id and m.course_id=pm.course_id and m.qan=pm.qan and m.exam_qan <> pm.qan and m.start_date=pm.start_date
		where pm.completion_status='Continuing' and m.start_date IS NOT NULL and m.exam_qan IS NOT NULL and m.qan IS NOT NULL
		and
		(( pm.planned_end IS NOT NULL  AND month(pm.planned_end)!=8 and m.actual_end_date IS NULL  and m.start_date IS NOT NULL  
		and pm.planned_end<>m.planned_end_date)
		or
		( pm.planned_end IS NOT NULL  AND month(pm.planned_end)!=8 and m.actual_end_date IS NOT NULL  and m.actual_end_date< pm.planned_end and m.start_date IS NOT NULL  
		and pm.planned_end<>m.planned_end_date)
		)
		
		--Update planned end dates from previous return 
		UPDATE #pupil_course_detail_for_current_return      
        SET planned_end_date= m.planned_end
        FROM #pupil_course_detail_for_current_return curr 
		INNER JOIN #previousYearMembers m on curr.person_id=m.person_id and curr.course_id=m.course_id and curr.start_date=m.start_date
		INNER JOIN sims.census_LearningAims_Qans clq    on clq.person_id=m.person_id and clq.course_id =m.course_id and m.start_date = clq.user_original_start_date 
		WHERE clq.return_id = @currentReturnID 
		
     -- Update prev_return_planned_end date 
	   UPDATE pcd  
       SET prev_return_planned_end = planned_end_date
       FROM #pupil_course_detail_for_current_return pcd  
		
	
END -- Check whether the school having 6th form
ELSE
BEGIN 	
--To Update planned end dat from course manager Autumn return
	UPDATE #pupil_course_detail_for_current_return 
	SET planned_end_date= m.planned_end   
    FROM #pupil_course_detail_for_current_return curr 
    INNER JOIN #members m on curr.person_id=m.person_id and curr.course_id=m.course_id and curr.start_date=m.start_date     
    INNER JOIN sims.census_LearningAims_Qans clq    on clq.person_id=m.person_id and clq.course_id =m.course_id and m.start_date = clq.user_original_start_date  
    WHERE clq.return_id = @currentReturnID 
    
    --update user_original_planned_end date 
    UPDATE pcd  
    SET   user_original_planned_end = pcd.planned_end_date
    FROM #pupil_course_detail_for_current_return pcd   
    
END
END

-- Update calculated details in original columns values  
UPDATE pcd  
 SET 
	 user_original_qan = pcd.qan  
    ,user_original_discount_code = pcd.discount_code
    ,user_original_start_date = pcd.start_date 
	,user_original_actual_end  = pcd.actual_end_date
    ,user_original_completion_status_id = pcd.completion_status_id
    ,user_original_core_aim = pcd.core_aim  
	,user_original_withdrawal_reason_id = pcd.withdrawal_reason_id
	,user_original_ukprn_classification_id = pcd.ukprn_classification_id
FROM #pupil_course_detail_for_current_return pcd  

if @overwrite_edits = 0 -- we need to preserve items modified by the user
begin
    -- Update user modified details in current columns values  
    update #pupil_course_detail_for_current_return  
       set qan					=  case when clq.qan <> clq.user_original_qan then clq.qan else pcd.qan end,  
           discount_code		=  case when clq.discount_code <> clq.user_original_discount_code then clq.discount_code else pcd.discount_code end,  
           start_date			=  case when clq.start_date <> clq.user_original_start_date then clq.start_date else pcd.start_date end,
	       actual_end_date		=  case when isnull(clq.actual_end,'') <> isnull(clq.user_original_actual_end,'') then clq.actual_end else pcd.actual_end_date end,
           planned_end_date		=  case when (isnull(clq.planned_end,'') <> isnull(clq.user_original_planned_end,'') and (isnull(clq.planned_end,'')<> isnull(clq.previous_return_planned_end,'') or clq.previous_return_planned_end is null)) then isnull(clq.planned_end,'')
										else isnull(pcd.planned_end_date,'')end,
           completion_status_id =  case when clq.completion_status_id <> clq.user_original_completion_status_id then clq.completion_status_id else pcd.completion_status_id end,  
           deleted              = clq.deleted,
           core_aim             = case when clq.core_aim <> clq.user_original_core_aim then clq.core_aim else pcd.core_aim end,
		   withdrawal_reason_id = case when isnull(clq.withdrawal_reason_id,0) <> isnull(clq.user_original_withdrawal_reason_id,0) then clq.withdrawal_reason_id else pcd.withdrawal_reason_id end,
		   ukprn_classification_id= case when isnull(clq.ukprn_classification_id,0) <> isnull(clq.user_original_ukprn_classification_id,0) then clq.ukprn_classification_id else pcd.ukprn_classification_id end
     from #pupil_course_detail_for_current_return pcd  
    inner join sims.census_LearningAims_Qans clq  
       on pcd.person_id = clq.person_id  
      and pcd.course_id = clq.course_id   
      and pcd.start_date = clq.user_original_start_date 
    where clq.return_id = @currentReturnID   
end

select distinct
	[Person Id]=pcd.person_id 
	,pcd.course_id
    ,pcd.surname   
    ,pcd.forename
    ,[YTI]         = pcd.nc_year 
    ,[reg] 		   =  substring(ss.current_reg,1,10) 		
    ,[adno]        = ss.latest_adno
    ,[ULN]		   = uln.unique_learner_number
    ,[UCI]         = exc.uci
    ,[exam_no]     = exc.exam_number
    ,[QAN]=pcd.qan
    ,[Disc Code]=pcd.discount_code 
    ,[QWS QAN/Disc]=	case 
				when pcd.qan_description is null or pcd.qan_description='No description found' then pcd.qan + '/' + pcd.discount_code +'*'
			    when isnull(pcd.qan,'')<>''   then pcd.qan + '/' + pcd.discount_code 
					else '' end
    ,[qan_description]=isnull(pcd.qan_description,'No description found')
	,[Start Date] = pcd.start_date
	,[End Date]=pcd.actual_end_date
	,pcd.planned_end_date
	,pcd.completion_status
	,pcd.completion_status_id
	,[subject]    =  isnull(csc.title,'')  
    ,[level]     =  isnull(sclLevel.description,'')  
    ,[result]				= pcd.result
    ,[Exam QAN]			    = pcd.exam_qan	
    ,[outcome_id]			= pcd.outcome
    ,[deleted]			    = isnull(pcd.deleted,0)	
	,[withdrawal_reason_id] = pcd.withdrawal_reason_id	
	,[ukprn_classification_id]=pcd.ukprn_classification_id 	
    ,[user_added]           = pcd.user_added 
    ,[user_original_qan]	= pcd.user_original_qan
    ,[user_original_discount_code]  = pcd.user_original_discount_code
    ,[user_original_start_date]			= pcd.user_original_start_date
    ,[user_original_actual_end]				= pcd.user_original_actual_end
    ,[user_original_planned_end]			= pcd.user_original_planned_end
    ,[user_original_completion_status_id]	= pcd.user_original_completion_status_id
    ,[previous_return_planned_end] =pcd.prev_return_planned_end   
    ,[Core Aim]               = cast(isnull(pcd.core_aim,'0') as char(1))COLLATE DATABASE_DEFAULT
    ,[user_original_core_aim] = cast(isnull(pcd.user_original_core_aim,'0') as char(1))COLLATE DATABASE_DEFAULT
	,[user_original_withdrawal_reason_id]=pcd.user_original_withdrawal_reason_id	
	,[user_original_ukprn_classification_id]=pcd.user_original_ukprn_classification_id 
	from #pupil_course_detail_for_current_return pcd
	inner join sims.curr_course cc       on cc.base_group_id = pcd.course_id  
   inner join sims.curr_department cd      on cd.department_id = cc.department_id  and (cd.name != 'Pastoral' and cd.name !='Alternative Curriculum')  
   inner join sims.curr_course_subject sccs     on sccs.base_group_id = cc.base_group_id    
   inner join sims.curr_subject_code csc     on csc.subject_code_id = sccs.subject_code_id    
   inner join sims.curr_subject_codeset codeset    on csc.subj_codeset_id = codeset.subj_codeset_id   
   and codeset.owner = 'Star'    
      left join ( sims.curr_course_classification_link ccclLevel         
                 inner join sims.curr_course_classification sclLevel   on sclLevel.curr_course_classification_id = ccclLevel.curr_course_classification_id    
                 inner join sims.curr_course_classification_type ctLevel  on sclLevel.curr_course_classification_type_id = ctLevel.curr_course_classification_type_id    
                    and ctLevel.title = 'Level'      
               ) on ccclLevel.course_id = cc.base_group_id    
    left join sims.sims_person_unique_learner_number uln on pcd.person_id = uln.person_id 
	left join sims.stud_student ss on ss.person_id = pcd.person_id
	left join sims.exam_candidate exc on exc.person_id = pcd.person_id
	
    where (pcd.qan is not null or pcd.exam_qan is not null)
    
union all /*Added rows added by user*/

select distinct 
	[Person Id]= curr.person_id 
	,curr.course_id
    ,p.surname   
    ,p.forename   
    ,[YTI]         = curr.year 
    ,[reg] 		   = substring(ss.current_reg,1,10) 	
    ,[adno]        = ss.latest_adno
    ,[ULN]		   = uln.unique_learner_number
    ,[UCI]         = exc.uci
    ,[exam_no]     = exc.exam_number
    ,[QAN]=curr.qan
    ,[Disc Code]  = curr.discount_code
     ,[QWS QAN/Disc]=	 case 
			    when isnull(curr.qan,'')<>'' then curr.qan + '/' + curr.discount_code 
					else '' end
    ,[qan_description]	=	 case 
			    when isnull(curr.qan,'')<>'' then curr.qan + '/' + curr.discount_code 
					else '' end
	,[Start Date]=curr.start_date	
	,[End Date]=curr.actual_end
	,[planned_end_date]=curr.planned_end
	,[completion_status] = ''
	,curr.completion_status_id
	,[subject]    =  isnull(csc.title,'')  
    ,[level]      =  isnull(sclLevel.description,'') 
    ,[result]				=curr.result
    ,[Exam QAN]			    =curr.examQAN		   			   
    ,[outcome_id]			=outcome.id	 
    ,[deleted]			    =curr.deleted
	,[withdrawal_reason_id] = curr.withdrawal_reason_id
	,[ukprn_classification_id]=curr.ukprn_classification_id		
    ,[user_added]			= isnull(curr.user_added,0)
    ,[user_original_qan]					= curr.user_original_qan
    ,[user_original_discount_code]			= curr.user_original_discount_code
    ,[user_original_start_date]			    = curr.user_original_start_date
    ,[user_original_actual_end]				= curr.user_original_actual_end
    ,[user_original_planned_end]			= curr.user_original_planned_end
    ,[user_original_completion_status_id]	= curr.user_original_completion_status_id
    ,[previous_return_planned_end]          = curr.previous_return_planned_end   
    ,[Core Aim]								= cast(isnull(curr.core_aim,'0') as char(1))COLLATE DATABASE_DEFAULT
    ,[user_original_core_aim]				= cast(isnull(curr.user_original_core_aim,'0') as char(1))COLLATE DATABASE_DEFAULT
	,[user_original_withdrawal_reason_id]	=curr.user_original_withdrawal_reason_id
	,[user_original_ukprn_classification_id]=curr.user_original_ukprn_classification_id
	from sims.census_LearningAims_Qans curr 
	inner join sims.sims_person p
	on curr.person_id=p.person_id
	 left join sims.sims_person_unique_learner_number uln on curr.person_id = uln.person_id 
	
	 left join
	 (
	 sims.curr_course cc       
	 inner join sims.curr_department cd      on cd.department_id = cc.department_id  and (cd.name != 'Pastoral' and cd.name !='Alternative Curriculum') 
	 inner join sims.curr_course_subject sccs     on sccs.base_group_id = cc.base_group_id    
     inner join sims.curr_subject_code csc     on csc.subject_code_id = sccs.subject_code_id    
     inner join sims.curr_subject_codeset codeset    on csc.subj_codeset_id = codeset.subj_codeset_id   
     and codeset.owner = 'Star'     
     ) on cc.base_group_id = curr.course_id  
   
      left join ( sims.curr_course_classification_link ccclLevel         
                 inner join sims.curr_course_classification sclLevel   on sclLevel.curr_course_classification_id = ccclLevel.curr_course_classification_id    
                 inner join sims.curr_course_classification_type ctLevel  on sclLevel.curr_course_classification_type_id = ctLevel.curr_course_classification_type_id    
                    and ctLevel.title = 'Level'      
               ) on ccclLevel.course_id = cc.base_group_id   
    left join sims.course_manager_outcomes outcome
           on outcome.code = curr.outcome   
	left join sims.stud_student ss on ss.person_id = curr.person_id
	left join sims.exam_candidate exc on exc.person_id = curr.person_id
    where curr.return_id=@currentReturnID
     and curr.user_added = 1
	 and curr.deleted = 0 
	 and not exists (  
					 select 1  
					   from #pupil_course_detail_for_current_return pcd  
					  where pcd.person_id  = curr.person_id
						and pcd.course_id  = isnull(curr.course_id,0)   
						and pcd.start_date = curr.start_date  
				   )  
	 and not exists (  
					 select 1  
					   from #pupil_course_detail_for_current_return pcd  /*If status is withdrawn then dont add it*/
					  where pcd.person_id  = curr.person_id
						and pcd.course_id  = curr.course_id   
						and pcd.start_date = curr.start_date  
						and pcd.completion_status_id = (select completion_status_id from sims.curr_completion_status where description='Withdrawn')
				   )              
	ORDER BY surname, forename,[subject],pcd.qan,pcd.start_date
	
	
drop table #pupil_course_detail_for_current_return
drop table #results
drop table #members
drop table #best_results	
drop table #previousYearMembers
drop table #courses
-------
return 0
 
go

------------------------------------------------------------------------------------------------------------------
/*  TEST
Description: Unit Tests
     Author: Manish T
       Date: Feb 08, 2007
    Dataset: English Secondary

-- Perform the test

      exec sims.census_pix_census_GetLearningAimsQans '2014-10-02 00:00:00',0,65,0,0,0

Resultset:
person_id   qan      disccode start_date              actual_end              planned_end             completion_status                                                                                                                                                                                                                                          completion_status_id
----------- -------- -------- ----------------------- ----------------------- ----------------------- ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- --------------------
3384        10000902 3650     2008-08-30 23:59:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Re-sit                                                                                                                                                                                                                                                     0
3384        10001001 4410     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3388        10001025 5030     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3388        10001116 3910     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3390        10001025 5030     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3391        10001025 5030     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3391        10003502 4010     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3393        10000902 3650     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3393        10001116 3910     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2
3393        10003502 4010     2007-09-01 00:00:00     2008-08-30 23:59:00     2008-08-30 23:59:00     Completed                                                                                                                                                                                                                                                  2

---------------------------------------------
*/






-- END PATCH SCRIPT 12
GO


-- START PATCH SCRIPT 13



/*--------------------------------------------------------------------------------------------------------------*/
/* Module     : SIMS Database SQL Server 2005                                                                   */
/* Platform   : Pulsar                                                                                          */
/* Area       : School Census                                                                                   */
/* Copyright  : LegalCopyright (c) Capita Business Services Ltd 1984-2009                                       */
/*            : This code has been written by Capita Business Services for use within its own SIMS systems.     */
/*            : It is unpublished proprietary work, and as such it is illegal for it to be reproduced by any    */
/*            : means or used without the express written permission of Capita Education Services.              */
/* Status     : UNREVIEWED                                                                                      */
/*--------------------------------------------------------------------------------------------------------------*/

set nocount on
go
------------------------------------------------------------------------------------------------------------------


exec sims.db_p_drop 'sims.sre_pix_datapackage_census_GetLeaverIDsForReturn'
                    ,'p'
go


alter procedure sims.sre_pix_datapackage_census_GetLeaverIDsForReturn
(
   @pupilXML text,
   @return_id int,
   @school_id    int
)

as
/*--------------------------------------------------------------------------------------------------------------------------*/
/* Format     : Transact-SQL																								*/
/* Object     : sims.sre_pix_datapackage_census_GetLeaverIDsForReturn   								                    */
/*																															*/
/* Purpose    : This procedure gets the Leaver IDs For Return																*/
/* Comments   :																												*/
/*																															*/
/*                                                                                                                          */
/* Author     : Sooraj M.V																									*/
/* Date       : 12th Nov 2009                                                                                               */
/* Comments   : Modified the stored procedure for SR Engine																	*/                                                                                             
/*                                                                                                                          */
/* Updated    : AJD (for Sooraj) 17/01/2011   KB106460 delete from @studentOnAttendance now depends on PRU status           */
/* Updated    : AJD KB106900 to deal with PRU attendance data when SIMS Attendance module is NOT used                       */
/* Modified By: Chandra S Neelam                                                                                            */
/* Date       : 03 Feb 2011                                                                                                 */
/* Details    : KB107004 Fix - Exclusions with 'R - Reinstatement' or 'O - Reinstatement appropriate but not in best        */
/*            : interest of child' should not be collected if the exlcusion is in appeal.                                   */
/* Modified By: Jhabiulla Phatan                                                                                            */
/* Date       : 01 Oct 2013                                                                                                 */
/* Details    : PRB5457 added a check include Leaner support details only for SUM and SPR terms								*/
/*--------------------------------------------------------------------------------------------------------------------------*/
/* Modified by : Arun Sutar																																		 */
/* Date			: 15/10/2014																																		*/
/* Details		: PRB8751 - fixed - when school is type of a SPECIAL - Deemed PRU - Collected attendance base on term			*/
/*					  and not entire year.																															*/
/*--------------------------------------------------------------------------------------------------------------------------*/
/* Modified By: Jahir P                                                                                                     */
/* Date       : 06/10/2015                                                                                                  */
/* Details    : PRB11605 Fix - Leavers data not showing on report if school not using Year taught in 12,13,14 for           */
/*              currentyear and were school previous academic year.                                                         */
/*--------------------------------------------------------------------------------------------------------------------------*/

set nocount on

--main body
--

--- declare local variables
--
declare @idoc			int
declare @census_date datetime
declare @term_code char(3)
declare @exclusion_start_date smalldatetime
declare @exclusion_end_date smalldatetime
declare @annual_exclusion_start_date smalldatetime
declare @annual_exclusion_end_date smalldatetime
declare @flavour varchar(10)
declare @learning_aim_start_date datetime
declare @learning_aim_end_date	datetime
declare @summer_census_date	datetime
declare @prev_acad_start_date datetime
declare @acad_start_date datetime
declare @acad_end_date datetime
declare @prev_acad_end_date datetime
declare  @maxdate smalldatetime
declare @homeschool_id int
declare @age_as_at_date smalldatetime
declare @Min_Date datetime
declare @Max_Date datetime
declare @boarderID int
declare @attendance_data_editable bit
declare @dfes_code_compatible bit
declare @attendance_panel_used bit
declare @minDateYear int
declare @Att_start_date datetime
declare @Min_Date_learnerSupport			smalldatetime

			,@Att_end_date datetime
			--,@Annual_Att_start_date datetime
			--,@Annual_Att_end_date datetime
			,@Term varchar(3)
,@schoolType nvarchar(5)
,@year_structure_codes nvarchar(20)
,@isPRU BIT
DECLARE @LearnerSupport_start smalldatetime

-- load person_ids into learner support table
DECLARE @studentLearnerSupport TABLE (person_id int)
-- Temporary table for on roll pupils
Declare @studentOnRoll table
(                           
	person_id int primary key
) 

-- Temporary table for pupils having attendance data
Declare @studentOnAttendance table
(                           
	 person_id int primary key
) 

-- Temporary table for pupils having exclusion data
Declare @studentExclusion table
(                           
	person_id int primary key
) 

-- Temporary table for Not on roll pupils
Declare @studentNotOnRoll table
(                           
	person_id int primary key
) 

-- Temporary table to hold all the attendance and exclusion pupils, to give a single dataset
Declare @studentAttEcxl table
(                           
	person_id int primary key
) 

-- Temporary table for leavers having p16 data  
Declare @studentP16Leaver table  
(                             
 person_id int primary key  
) 
  
declare @attendanceTable table
(                             
 person_id int ,
 session_start smalldatetime 
) 	

-----Table to track the border period of the pupil-----
declare @boarderTable table
(
	person_id	    int,
	start_date      datetime,
	end_date	    datetime
)
-- Check if school is an academy...
declare @isAcademy bit = 0

-- Date when school was converted to an academy
declare @school_number_changed_date smalldatetime

set @maxdate = '20790606'

select @schoolType = code ,  @homeschool_id = ss.school_id from sims.school_phase_type spt
	inner join sims.sims_school ss on ss.phase_id = spt.lookup_value_id
	where ss.status = 'S'

select @term = pct.code from sims.plasc_census_term pct 
	inner join sims.plasc_return pr
	on pct.census_term_id=pr.census_term_id where pr.return_id = @return_id
	
DECLARE @AlternDeptId int
SELECT @AlternDeptId = department_id FROM sims.curr_department WHERE name = 'Alternative Curriculum'	
-- Set value for variable @isPRU
select @isPRU = sims.census_fnb_check_school_type_for_PRU()

-- set minimum age of pupils on age as at date
set @minDateYear=4

/*-------------------------------------------End -- Declaration for the changes for School Census Spring 2009 ----------------*/

--initialize variables
	
-- Check if the school is an academy (code 49) . We can trust the "code" here because it is a statutory lookup.
select  @isAcademy = case st.code when '49' then 1 else 0 end
from    sims.sims_school ss
join    sims.school_type st
on      ss.type_id = st.lookup_value_id
where   ss.school_id = @homeschool_id


if (@isAcademy = 1)
begin
    select  @school_number_changed_date = spi.date_number_changed
    from    sims.sims_school_previous_information spi
    where   spi.school_id = @homeschool_id
end

select @school_number_changed_date = ISNULL(@school_number_changed_date, '1900-01-01')	
	
select @flavour = region.region_code + phase.external_code  
 from sims.db_version_info ver  
 inner join sims.sims_region region  
 on ver.region_id = region.region_id  
 inner join sims.sims_school sch  
 on sch.status = 'S'  
 inner join sims.school_phase_type phase  
 on sch.phase_id = phase.lookup_value_id
 

--
--- initialise local variables
--

-- get all the dates 
-- note: PRU age_as_at_date is the one for attendance collection which is a year earlier, so add a year to it for Learner Support calculation	
select  @census_date            =   spr.survey_date
,       @exclusion_start_date   =   case when @school_number_changed_date > spct.exclusion_start_date and @school_number_changed_date<= spct.census_date then @school_number_changed_date else spct.exclusion_start_date end
,       @exclusion_end_date     =   spct.exclusion_end_date
,       @age_as_at_date         =   spr.age_as_at_date
,       @attendance_data_editable=  spr.attendance_data_editable
,       @dfes_code_compatible    =  spr.dfes_code_compatible
,	    @annual_exclusion_start_date = case when @school_number_changed_date > spct.annual_exclusion_start_date and @school_number_changed_date<= spct.census_date then @school_number_changed_date else spct.annual_exclusion_start_date end
,	    @annual_exclusion_end_date = spct.annual_exclusion_end_date
,	    @learning_aim_start_date =case when @school_number_changed_date > spct.learning_activities_collected_from and @school_number_changed_date<= spct.census_date then @school_number_changed_date else spct.learning_activities_collected_from end
,	    @learning_aim_end_date = spct.learning_activities_collected_to
,       @term_code              =   spct.code
from    sims.plasc_return spr
join    sims.plasc_census_term spct
on      spr.census_term_id = spct.census_term_id
where   spr.return_id = @return_id

--PRB11605 get Current academic start and end, previous academic start and end dates.
	--populating Acad start date for Autumn term
	Select @acad_start_date = eventInst.event_start, @acad_end_date =  eventInst.event_end 
	from sims.sims_event_instance eventInst    
	inner join sims.sims_event event on event.event_id = eventInst.event_id    
	inner join sims.sims_event_type eventType on eventType.event_type_id = event.event_type_id and eventType.code = 'AcadYear'    
	where @census_date between eventInst.event_start and eventInst.event_end 

--selecting previous Academic Year
Select @prev_acad_start_date = eventInst.event_start 
      ,@prev_acad_end_date = eventInst.event_end
	from sims.sims_event_instance eventInst    
	inner join sims.sims_event event on event.event_id = eventInst.event_id    
	inner join sims.sims_event_type eventType on eventType.event_type_id = event.event_type_id and eventType.code = 'AcadYear'    
	where @acad_start_date-1 between eventInst.event_start and eventInst.event_end

-- set summer census Date
 
select @summer_census_date=census_date from sims.plasc_census_term where code = 'SUM'


set @attendance_panel_used=1

if (@attendance_data_editable=1 and @dfes_code_compatible=1)
begin
set @attendance_panel_used=0
end

	SELECT @Min_Date= Cast(DatePart(yyyy,DATEADD(yyyy, -1, DATEADD(yyyy, -15, @age_as_at_date))) as varchar(4))+ '/' + 
					  Cast(DatePart(mm,DATEADD(mm, 1, DATEADD(yyyy, -15, @age_as_at_date))) as varchar(2)) + '/' + 
							  Cast(DatePart(dd,DATEADD(dd, 1, DATEADD(yyyy, -15, @age_as_at_date))) as varchar(2))

	SELECT @Max_Date=Cast(DatePart(yyyy,DATEADD(yyyy, -@minDateYear, @age_as_at_date)) as varchar(4))+ '/' + 
					  Cast(DatePart(mm,@age_as_at_date) as varchar(2)) + '/' + 
					  Cast(DatePart(dd,@age_as_at_date) as varchar(2)) 

--
--- main process
--

-- Insert on roll pupils
EXEC sp_xml_preparedocument @idoc OUTPUT, @pupilXML         
insert into @studentOnRoll
(
            person_id
)
Select 
            p
 
from openxml (@idoc, '/DataPackage/Row',1)  
WITH     
            (
            p          int         'person_id'
            )
EXEC sp_xml_removedocument @idoc

select @year_structure_codes = 'XY#*'
-- Pupils having attenance data

if (@attendance_panel_used=1)
begin
    Insert  into @studentOnAttendance
    Select  distinct pcas.person_id
    From    sims.plasc_census_attendance_summary pcas
    where   return_id = @return_id 
    and     (termly_sessions_possible > 0 or annual_sessions_possible > 0)  -- KB106655
end
else
begin
--collecting the boarder ID for boarder status check
    select @boarderID = base_group_type_id 
    from   sims.sims_base_group_type
    where  code = 'Boarder'

	select  @Att_start_date = attendance_termly_startdate,
	        @Att_end_date = attendance_second_termly_enddate 
	from    sims.plasc_census_term 
	where   code = @term
	
    if (@Term='AUT'  or (@isPRU = 1 and @Term='SPR') )
    begin
	    --Based on school type, assign the value to the variable 		
        if (@schoolType = 'SP' and @isPRU =0)
        begin
            select  @att_start_date = attendance_termly_startdate 
	        from    sims.plasc_census_term 
	        where   code = 'SPR' 
        end        
        

    end
    
    if (@school_number_changed_date > @att_start_date and @school_number_changed_date<= @census_date)
    begin
        select @att_start_date = @school_number_changed_date 
    end

	--inserting students eligible for attendance
	insert  @studentonattendance ( person_id )
	select  [person_id] = stud.person_id 
	from    sims.sims_person per
	join    sims.stud_school_history stud
	on      per.person_id=stud.person_id
	and     stud.school_id = @homeschool_id      
	and 	isnull(stud.start_date,'1900-01-01 00:00:00') <= @census_date
    and     isnull(stud.end_date,'2079-06-06 23:59:00')>= @att_start_date
	--added the condition for filtering readmissions
	and     stud.start_date = (select  max(stud1.start_date) 
	                            from    sims.stud_school_history stud1
  							    where   stud1.person_id = per.person_id  
							    and     stud1.school_id = @homeschool_id  
							    and     isnull(stud1.start_date,'1900-01-01 00:00:00')  <= @census_date)
	and not exists(select  1 
					from    sims.sims_member sm
				    join    sims.sims_base_group sbg
					on      sm.base_group_id = sbg.base_group_id
				    and     sm.person_id = stud.person_id
				    and     sbg.base_group_type_id = @boarderID
				    and     sbg.Code != 'N' 
				    and     (
				                (sm.Start_date <= @census_date  and  isnull(sm.end_date, @census_date) >= @census_date)
							OR
								(sm.Start_date <= isnull(stud.end_date,'2079-06-06 23:59:00')  and  (sm.end_date >= isnull(stud.end_date,'2079-06-06 23:59:00')))
					        )
		        )
			
    where   per.dob between @min_date and @max_date
    and     per.person_id in (select person_id from sims.census_fnt_get_on_roll_student_ids(@att_start_date, @att_end_date)) 	
	
    --selecting boarder sessions
    insert  @boarderTable(person_id, start_date, end_date)
    select  studAtt.person_id, sm.start_date, sm.end_date  
    from    @studentOnAttendance studAtt
    join    sims.sims_member sm  
    on      sm.person_id = studAtt.person_id
    join    sims.sims_base_group sbg  
    on      sm.base_group_id = sbg.base_group_id  
    and     sbg.base_group_type_id = @boarderID  
    and     sbg.Code != 'N'   
    and     sm.Start_date <= @Att_end_date   
	and     isnull(sm.end_date, @Att_start_date) >= @Att_start_date

    --selecting attendance for studentOnAttendance
    insert  @attendanceTable (person_id,session_start)
    select  studAtt.person_id,att.session_start
    from    @studentOnAttendance studAtt
    join    sims.att_mark_normalised att
    on      studAtt.person_id = att.person_id
    where   att.session_start >= @Att_start_date
    and     att.session_start <= @Att_end_date
    and     charindex(mark,@year_structure_codes)=0

					
	-- Delete the border session of the pupil from attendence table
	delete  att
	from 	@attendanceTable att
	join 	@boarderTable bt
	on	    bt.person_id = att.person_id 
	where	att.session_start >= bt.start_date
	and 	att.session_start <= isnull(bt.end_date, att.session_start)

    -- deleting studentOnAttendance where there is no attendance found
    delete  studAtt
    from    @studentOnAttendance studAtt
    where   not exists(select 1 from @attendanceTable where person_id=studAtt.person_id)

end

if @schoolType = 'NS' OR (@schoolType = 'SP' AND @TERM IN ('SUM','SPR') and @isPRU=0)  -- KB106460
begin   
	delete from @studentonattendance
end  


/*----------------------------------------------------------------------------------------------------------------*/
-- Pupils having exclusion data
-- 18-01-2007 RI KB65172 Students with Fixed Term did not always appear on the Census
if  @flavour='ENGSS' or @flavour='ENGMS' or @flavour='ENGSP' or @flavour='ENGPS' or @flavour='ENGMP' or @flavour='ENGPR' or @flavour='ENGAT' 
begin
    insert  @studentexclusion
    select  distinct exn.person_id
    from    sims.stud_exclusion exn
    join    sims.stud_exclusion_type exnt
	on      exnt.lookup_value_id = exn.exclusion_type_id
    and     exnt.external_code in ('FIXD','LNCH')
    left join sims.stud_exclusion_disciplinary_committee sedc          
	            join    sims.stud_exclusion_appeal_result sear  
		        on      sedc.outcome_id = sear.lookup_value_id
	            and     sear.active = 'T'
    on      sedc.exclusion_id = exn.exclusion_id    
    where   exn.deleted = 'F'  
    and     (sear.code is null  or not ( sear.code in ('R','O')))
    and     ((@exclusion_start_date >= @school_number_changed_date and exn.start_date between @exclusion_start_date and  @exclusion_end_date and @isAcademy = 1) 
	         or
	(exn.start_date between @exclusion_start_date and  @exclusion_end_date and @isAcademy = 0)
			)
	
    insert  into @studentexclusion
    select  distinct exn.person_id
    from    sims.stud_exclusion exn
    join    sims.stud_exclusion_type exnt
	on      exnt.lookup_value_id = exn.exclusion_type_id
    and     exnt.external_code in ('PERM')
    left join sims.stud_exclusion_appeal_result sear         
    on      exn.appeal_result_id = sear.lookup_value_id  and sear.active = 'T'
    where   exn.deleted = 'F'  
    and     (sear.code is null  or not (exn.appeal_indicator = 'T' and sear.code in ('R','O')))
    and     exn.person_id not in(select person_id from @studentExclusion)
    and     ((@exclusion_start_date >= @school_number_changed_date and exn.start_date between @exclusion_start_date and  @exclusion_end_date and @isAcademy = 1) 
	        or   
            (exn.start_date between @exclusion_start_date and  @exclusion_end_date and @isAcademy = 0)
            ) 
end
else
begin
    insert  @studentexclusion
    select  distinct exn.person_id
    from    sims.stud_exclusion exn
    join    sims.stud_exclusion_type exnt
	on      exnt.lookup_value_id = exn.exclusion_type_id
    and     exnt.external_code in ('PERM')
    left join sims.stud_exclusion_appeal_result sear           --KB107004 Fix
    on      sear.lookup_value_id = exn.appeal_result_id      
    where   exn.deleted = 'F' 
    and     (sear.code is null  or not (exn.appeal_indicator = 'T' and sear.code in ('R','O')))
    and     ((@annual_exclusion_start_date >= @school_number_changed_date and exn.start_date between @annual_exclusion_start_date and  @annual_exclusion_end_date and @isAcademy = 1) 
	        or
            -- Only get the exclusions that are contained in the exclusion collection period.    
            (exn.start_date between @annual_exclusion_start_date and @annual_exclusion_end_date and @isAcademy = 0)
            ) 

end

-- Delete the exclusions for the Onroll Guests
delete  ext 
from    @studentexclusion ext
join    sims.stud_school_history ssh
on      ssh.person_id=ext.person_id
and     isnull(ssh.start_date,'1900-01-01 00:00:00') <= @census_date 
and     isnull(ssh.end_date, @census_date) >= @census_date
join    sims.sims_school ss 
on      ss.school_id=ssh.school_id 
and     ss.status='S'
join    sims.stud_on_roll_mode sorm 
on      ssh.on_roll_mode_id = sorm.on_roll_mode_id
and     sorm.external_code in ('G')

-- Delete the exclusions for the Leaver Guests
delete  ext 
from    @studentExclusion ext
Join    sims.stud_school_history ssh
on      ssh.person_id=ext.person_id
Join    sims.sims_school ss 
on      ss.school_id=ssh.school_id 
and     ss.status='S'
Join    sims.stud_on_roll_mode sorm 
on      ssh.on_roll_mode_id = sorm.on_roll_mode_id
and     sorm.external_code in ('G')
where   isnull(ssh.end_date, @maxdate) <= @census_date
and     ssh.start_date = ( select   max(start_date) 
                           from     sims.stud_school_history 
                           where    person_id=ext.person_id
                           and      isnull(end_date, @maxdate) <= @census_date and school_id = @homeschool_id)


-- Includes pupils for attendance and exclusions
Insert  @studentAttEcxl
select  person_id 
from    @studentOnAttendance
union 
select  person_id 
from    @studentExclusion


-- Person ids for pupil not on roll
Insert  @studentNotOnRoll
select  person_id 
from    @studentAttEcxl
where   person_id not in (select person_id from @studentOnRoll)

--************************************************************-----
--Learner Support
-- collect learner support awarded since beginning of census academic year (1 August)

set @age_as_at_date         =   case when @isPru = 1 then DATEADD(yy, 1, @age_as_at_date) else @age_as_at_date end

if (@term = 'SUM' or @term = 'SPR' )    --PRB5457 LearnerSupport details to be included only for summer and spring terms.
begin 
SELECT @Min_Date_learnerSupport= 
Cast(DatePart(yyyy, DATEADD(yyyy, -16, @age_as_at_date)) as varchar(4))+ '/' + 
		            Cast(DatePart(mm,DATEADD(mm, 0, DATEADD(yyyy, -15, @age_as_at_date))) as varchar(2)) + '/' + 
                    Cast(DatePart(dd,DATEADD(dd, 0, DATEADD(yyyy, -15, @age_as_at_date))) as varchar(2))
 
SELECT @LearnerSupport_start = convert(char(4),year(pr.survey_date)-1) + '-08-01'
from sims.plasc_return pr
where pr.return_id = @return_id

insert into @studentLearnerSupport
select distinct ls.person_id
from sims.stud_learner_support ls
join sims.sims_person per
on ls.person_id=per.person_id
join sims.stud_learner_support_code lsc
  on lsc.learner_support_code_id = ls.learner_support_code_id
where ls.award_date >= @LearnerSupport_start and ls.award_date <=@census_date and lsc.code in('55','56') and per.dob<=@Min_Date_learnerSupport
and ls.person_id not in (select person_id from @studentOnRoll)
-- Delete the Onroll Dual Subsidiary Registered and Guest pupil 
delete  tbl 
from    @studentLearnerSupport tbl
join    sims.stud_school_history ssh
on      ssh.person_id=tbl.person_id
and     isnull(ssh.start_date,'1900-01-01 00:00:00') <= @census_date 
and     isnull(ssh.end_date, @census_date) >= @census_date
join    sims.sims_school ss 
on      ss.school_id=ssh.school_id 
and     ss.status='S'
join    sims.stud_on_roll_mode sorm 
on      ssh.on_roll_mode_id = sorm.on_roll_mode_id
and     sorm.external_code in ('G','S')

-- Delete the Leaver Dual Subsidiary Registered and Guest  pupil 
delete  tbl 
from    @studentLearnerSupport tbl
Join    sims.stud_school_history ssh
on      ssh.person_id=tbl.person_id
Join    sims.sims_school ss 
on      ss.school_id=ssh.school_id 
and     ss.status='S'
Join    sims.stud_on_roll_mode sorm 
on      ssh.on_roll_mode_id = sorm.on_roll_mode_id
and     sorm.external_code in ('G','S')
where   isnull(ssh.end_date, @maxdate) <= @census_date
and     ssh.start_date = ( select   max(start_date) 
                           from     sims.stud_school_history 
                           where    person_id=tbl.person_id
                           and      isnull(end_date, @maxdate) <= @census_date and school_id = @homeschool_id)
   
   -- Delete if school end date before academy date
delete  tbl 
from    @studentLearnerSupport tbl
Join    sims.stud_school_history ssh
on      ssh.person_id=tbl.person_id
Join    sims.sims_school ss 
on      ss.school_id=ssh.school_id 
and     ss.status='S'
where   isnull(ssh.end_date, @maxdate) <= @census_date and isnull(ssh.end_date, @maxdate) < @school_number_changed_date
and     ssh.start_date = ( select   max(start_date) 
                           from     sims.stud_school_history 
                           where    person_id=tbl.person_id
                           and      isnull(end_date, @maxdate) <= @census_date and school_id = @homeschool_id)
                        
 -- Person ids for pupil not on roll
Insert  @studentNotOnRoll
select  person_id 
from    @studentLearnerSupport
where   person_id not in (select person_id from @studentNotOnRoll)
end 
--************************************************************---
if  @flavour='ENGSS' or @flavour='ENGAT' or @isPRU=1
begin
--POPULATING ALL LEAVERS HAVING P16 DATA

    -- Person ids for leavers having p16 data  
    if  exists ( select 1 from sims.sims_school where status = 'S' and top_ncyear in ('12','13','14')
        or
		exists (select 1 from sims.sims_base_group_set_link bsl
                                        join sims.sims_base_group_set bs on bs.base_group_set_id = bsl.base_group_set_id
										join sims.sims_base_group_type bt on bt.base_group_type_id = bs.base_group_type_id
										join sims.sims_base_group bg on bg.base_group_id = bsl.base_group_id and bg.base_group_type_id = bt.base_group_type_id
										where bt.code ='ncyear' and bg.code in('12','13','14')
										and isnull(bs.start_date , '19000101') <= @prev_acad_end_date
										and isnull(bs.end_date , '20790606') >= @prev_acad_start_date)	   
			   ) --KB104112 Fix - Check whether the school having 6th form
    begin
        if (@term = 'AUT')
        begin
            --populating Acad start date for Autumn term         	
            Insert  @studentP16Leaver
            select distinct person_id from sims.census_LearningAims_Qans where return_id=@return_id
            and    person_id not in  (select person_id from @studentOnRoll)  
        end
    end

    -- P16 leavers Person ids for pupil not on roll
    Insert  @studentNotOnRoll
    select  distinct person_id 
    from    @studentP16Leaver
    where   person_id not in (select person_id from @studentNotOnRoll)
    -- Returning P16 Error Ids for not on roll autumn

    Insert  into @studentNotOnRoll
    select  distinct s.person_id 
    from    sims.stud_student s 
    join    sims.census_post16autumn_incorrectlearningaims c
    on      ltrim(rtrim(s.unique_pupil_no))=ltrim(rtrim(c.upn))
    where   return_id = @return_id
    and     error= 1 
    and     s.person_id not in(select person_id  from @studentOnRoll) 
    and     s.person_id not in(select person_id  from @studentNotOnRoll) 
end

-- get pupils not on roll
select  person_id 
from    @studentNotOnRoll


return 0
go


-----------------------------------------------------------------------------------------------------------------------

-- Grant Execute to Simsuser and provide skeleton test harness

-- grant execute on sims.sre_pix_datapackage_census_GetLeaverIDsForReturn to simsuser

/*  TEST
Description: Unit Tests
     Author: Piyush Vijaywargiya
       Date: 1 June 2005
    Dataset: <Sims>

<1> < This stored procedure get data for generating the return >

<Description of how to set up the test>

-- Perform the test

   exec sims.sre_pix_datapackage_census_GetLeaverIDsForReturn  16

*/






-- END PATCH SCRIPT 13
GO


-- START PATCH SCRIPT 14


if not exists(select 1 from sims.db_withdrawn_patch where patch_id = 21085)
insert sims.db_withdrawn_patch(patch_id) select 21085

if not exists(select 1 from sims.db_withdrawn_patch where patch_id = 21094)
insert sims.db_withdrawn_patch(patch_id) select 21094

if not exists(select 1 from sims.db_withdrawn_patch where patch_id = 21111)
insert sims.db_withdrawn_patch(patch_id) select 21111

if not exists(select 1 from sims.db_withdrawn_patch where patch_id = 21131)
insert sims.db_withdrawn_patch(patch_id) select 21131

if not exists(select 1 from sims.db_withdrawn_patch where patch_id = 21137)
insert sims.db_withdrawn_patch(patch_id) select 21137

--V6
if not exists(select 1 from sims.db_withdrawn_patch where patch_id = 21139)
insert sims.db_withdrawn_patch(patch_id) select 21139


GO




-- END PATCH SCRIPT 14
GO


-- END PATCH CODE



-- Record that this patch has been updated
INSERT INTO sims.db_update_log
(
	patch_id,
	update_id,
	successful,
	description,
	update_type,
	date_applied
)
VALUES
(
	21190,
	1,
	'T',
	'Summer Release 2015 - Stat Returns Consolidated Patch v7',
	'U',
	GetDate()
)

